@@ -1,6 +1,6 @@
 /*
- * $Id: kom-types.h,v 0.50 2002/03/29 22:29:52 ceder Exp $
- * Copyright (C) 1990-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: kom-types.h,v 0.64 2003/08/29 10:39:48 ceder Exp $
+ * Copyright (C) 1990-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,13 +20,13 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  *  KOM-types.h  --  Types used by both server and client of LysKOM
  *
  *
- *  Copyright (C) 1990-1999, 2001-2002	Lysator Computer Club,
+ *  Copyright (C) 1990-1999, 2001-2003	Lysator Computer Club,
  *			Linkoping University,  Sweden
  *
  *  Everyone is granted permission to copy, modify and redistribute
@@ -55,18 +55,12 @@
 /* Some objects in LysKOM need a type field.
    These are the valid values */
 
-enum object_type {
-    NO_OBJECT_TYPE = 0,         /* No object at all             */
-    TEXT_OBJECT_TYPE  = 1,      /* Text object                  */
-    CONF_OBJECT_TYPE  = 2,      /* Conference object            */
-#if 0
-    /* It is not currently possible to set aux-items on Person
-       structures.  But since they can be set on the corresponding
-       Conference object that is actually not a problem.  This should
-       probably be removed.  */
-    PERS_OBJECT_TYPE  = 3,      /* Person object                */
-#endif
-    INFO_OBJECT_TYPE  = 4       /* System info object           */
+enum object_type
+{
+    NO_OBJECT_TYPE,		/* No object at all             */
+    TEXT_OBJECT_TYPE,		/* Text object                  */
+    CONF_OBJECT_TYPE,		/* Conference object            */
+    INFO_OBJECT_TYPE,		/* System info object           */
 };
 
 
@@ -224,7 +218,7 @@ typedef struct {
 	Pers_no		  author;
 	unsigned short	  no_of_lines;
 	String_size	  no_of_chars;
-	unsigned short	  no_of_marks;	/* Antal markeringar */
+	unsigned short	  no_of_marks;	/* Including syntetic marks. */
 	unsigned short	  no_of_misc;	/* Recipients, times, comments, ... */
         unsigned long     highest_aux;
 	Misc_info	* misc_items;	/* List of miscellaneous info. */
@@ -255,7 +249,7 @@ typedef struct {
     unsigned int invitation : 1;
     unsigned int passive : 1;
     unsigned int secret : 1;
-    unsigned int reserved1 : 1;
+    unsigned int passive_message_invert : 1;
     unsigned int reserved2 : 1;
     unsigned int reserved3 : 1;
     unsigned int reserved4 : 1;
@@ -282,20 +276,35 @@ typedef struct {
 
 /*  Information about a person's membership in a conference  */
 
+struct read_range {
+    Local_text_no  first_read;
+    Local_text_no  last_read;
+};
+
+struct read_range_list {
+    unsigned short     length;
+    struct read_range *ranges;
+};
+
 typedef struct {
     unsigned long   position;
     Conf_no		conf_no;
     unsigned char	priority;	/* Interrupt priority */
     Time		last_time_read;	/* Updated every time a text in this
-					   conf. is marked as read.	*/
-    Local_text_no	last_text_read;	/* All texts before and inclusive this
-					   are read	*/
-    unsigned short	no_of_read;
-    Local_text_no * read_texts;	/* Texts after last_text_read. Sorted
-                                   in ascending order */
-    Pers_no added_by;
+					   conference is marked as read.  */
+    unsigned int	no_of_read_ranges;
+    struct read_range * read_ranges;    /* Sorted in ascending order. */
+    Pers_no             added_by;
     Time                added_at;
     Membership_type type;
+
+    /* read_ranges is cleared before it reaches the Protocol A output
+       layer in certain situations, even though that layer wants to
+       emit the last-text-read element.  This happens for example when
+       get-membership [99] is called with want-read-texts == 0.  This
+       variable maintains the last-text-read value. */
+    Bool skip_read_texts;
+
 } Membership;
 
 
@@ -364,7 +373,7 @@ typedef struct {
 	Conf_no		motd_conf; 	/* Conf that receive motds */
 	Conf_no		kom_news_conf;  /* News about kom */
 	Text_no		motd_of_lyskom; /* To be displayed after login */
-        long            highest_aux_no; /* Last aux-item number */
+        unsigned long   highest_aux_no; /* Last aux-item number */
         Aux_item_list   aux_item_list;  /* System aux items */
 	/* and maybe more... */
 } Info;
@@ -392,6 +401,18 @@ typedef struct {
     Text_no            tno;	       /* The Text_no to use */
 } L2g_iterator;
 
+typedef struct {
+    const Local_to_global *l2g;
+    const struct l2g_block_info *binfo;
+    int                arrindex;
+    Local_text_no      beginval;       /* First index in the search */
+    Local_text_no      endval;	       /* Last index in the search + 1 */
+
+    int                search_ended;   /* 1 if finished, 0 otherwise */
+    Local_text_no      lno;	       /* The Local_text_no to use */
+    Text_no            tno;	       /* The Text_no to use */
+} L2g_reverse_iterator;
+
 /* This struct holds the information that is needed to output a
    Text-Mapping.  Please note that the protocol A Text-Mapping data
    type looks very different from this structure.  */
@@ -401,6 +422,16 @@ typedef struct {
     const Local_to_global *l2g;
 } Text_mapping;
 
+/* This struct holds the information that is needed to output a
+   Text-Mapping in the reverse order.  Please note that the protocol A
+   Text-Mapping data type looks very different from this
+   structure.  */
+typedef struct {
+    Local_text_no          ceiling;
+    unsigned long          no_of_texts;
+    const Local_to_global *l2g;
+} Text_mapping_reverse;
+
 /* For performance reasons, sort the fields in increasing size.
   The comments about number of bits are true on a Sun Sparc-2, not
   necessarily anything else.  The code should work anyhow. */
@@ -443,6 +474,7 @@ typedef struct {
         Conf_no         supervisor;     /* Supervisor conference */
 	Local_text_no	highest_local_no; /* highest local text no */
 	Garb_nice	nice;	        /* Number of days to live */
+	Garb_nice	keep_commented; /* New comments protect texts */
 } Small_conf;
 
 typedef struct {
@@ -559,9 +591,13 @@ typedef struct {
 	Conf_no		working_conference;
 	Session_no	session;             /* Serial number of connection. */
 	Time		connection_time;     /* Not logintime. */
-	unsigned long		idle_time; 	     /* Seconds. */
+	unsigned long   idle_time; 	     /* Seconds. */
 } Session_info_ident;
 
+typedef struct {
+	unsigned short priority;
+    	unsigned short weight;
+} Scheduling_info;
 
 typedef struct {
 	int		  no_of_persons;
@@ -586,6 +622,34 @@ typedef struct {
         String        server_version;
 } Version_info;
 
+typedef struct {
+    float   average;
+    float   ascent_rate;
+    float   descent_rate;
+} Stats;
+
+typedef struct {
+    int    no_of_stats;
+    Stats *stats;
+} Stats_list;
+
+typedef struct {
+    int     no_of_stats;
+    String *stat_names;
+    Number_list intervals;
+} Stats_description;
+
+typedef struct {
+    time_t  boot_time;
+    time_t  save_time;
+    String  db_status;
+    Text_no existing_texts;
+    Text_no highest_text_no;
+    Conf_no existing_confs;
+    Conf_no existing_persons;
+    Conf_no highest_conf_no;
+} Static_server_info;
+
 #ifdef DEBUG_CALLS
 typedef struct {
     int arena;
