@@ -1,6 +1,6 @@
 /*
- * $Id: empty.c,v 1.10 2002/03/29 22:38:10 ceder Exp $
- * Copyright (C) 1993, 1998-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: empty.c,v 1.13 2003/08/23 16:38:21 ceder Exp $
+ * Copyright (C) 1993, 1998-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,11 +20,27 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-char neveruse='B';
+#include "linkansi.h"
+
+/* Some linkers warn if a library isn't used to resolve any symbol, so
+   we include this useless function in libansi.a and use it from
+   main() of all programs that may need to link with libansi.a.
+
+   A historic note: Some systems complain if a library is empty.  This
+   file was initially created as an empty C file, just so that we
+   always had an object file to include in libansi.a.  But then, some
+   system complained when a file contained nothing, so we added an
+   external variable.  Then we found the warnings from the linker, and
+   added linkansi(). */
+
+void
+link_ansi(void)
+{
+}
