@@ -34,6 +34,12 @@
 #include <time.h>
 
 
+#include "oop.h"
+#include "adns.h"
+#include "oop-adns.h"
+
+#include "s-string.h"
+
 #include "isc.h"
 #include "intern.h"
 
