@@ -19,8 +19,9 @@
 #ifndef UNUSED_H
 #define UNUSED_H
 
+#define UNUSED_H_CONCAT(a, b) a ## b
 #ifdef HAVE_ATTRIBUTE_UNUSED
-#  define UNUSED(var) var __attribute__((unused))
+#  define UNUSED(var) UNUSED_H_CONCAT(qazwsxedc, var) __attribute__((unused))
 #else
 #  if defined(__cplusplus)
 #    define UNUSED(var)
