@@ -1,6 +1,6 @@
 /*
- * $Id: s-string.c,v 1.26 2002/03/29 22:38:07 ceder Exp $
- * Copyright (C) 1990-1996, 1998-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: s-string.c,v 1.32 2003/08/23 16:38:19 ceder Exp $
+ * Copyright (C) 1990-1996, 1998-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,13 +20,13 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  *  s-string.c  --  Routines for manipulating objects of type String.
  *
  *
- *  Copyright (C) 1990-1996, 1998-1999, 2001-2002	Lysator Computer Club,
+ *  Copyright (C) 1990-1996, 1998-1999, 2001-2003	Lysator Computer Club,
  *			Linkoping University,  Sweden
  *
  *  Everyone is granted permission to copy, modify and redistribute
@@ -518,10 +518,9 @@ s_strcmp (String   str1,
  *  COLLAT_TAB is a table of collating values for every char.
  */
 EXPORT  int
-s_usr_strcmp (String	 str1,
-	      String     str2,
-	      char     * collat_tab)
-
+s_usr_strcmp(String	    str1,
+	     String	    str2,
+	     unsigned char *collat_tab)
 {
     String_size		index;
     String_size		shortest;	/* Length of the shortest string */
@@ -601,9 +600,9 @@ s_strhead (String	str1,
  *  s_usr_strcmp()).
  */
 EXPORT  Bool
-s_usr_strhead (String	str1,
-	       String	str2,
-	       char	collat_tab [ COLLAT_TAB_SIZE ])
+s_usr_strhead(String	     str1,
+	      String	     str2,
+	      unsigned char  collat_tab[COLLAT_TAB_SIZE])
 
 {
     String_size		i;	/*  Index in comparison loop  */
@@ -840,90 +839,110 @@ s_strtok (const String	  source,
  *  Convert a char to a number in base BASE.
  */
 static  int
-char2digit (const char	ch,
-	    const int	base)
+char2digit (const char ch)
 {
-    int			  index;
-    static const char	* translate_table	=
-				"0123456789abcdefghijklmnopqrstuvwxyz";
-    int			  c = (unsigned char)ch;
+    int c = (unsigned char)ch;
 
-    index = 0;
-    while (  (index < base)
-	   && (  translate_table[index]
-	       != (isalpha(c) ? tolower(c) : ch)))
-	index++;
-
-    if (index >= base)
+    if (c < '0' || c > '9')
 	return  -1;
     else
-	return  index;
+	return c - '0';
 }
 
 
 
 /*
- *  Convert the String STR to a long, using the base BASE.
+ *  Convert the String STR to a long, using base ten.
  *  Leading blanks are skipped according to isblank() in <ctype.h>.
  *  The index of the first character that couldn't be used to form
  *  the number is returned in *FIRST_ILL_CHAR.  Returns -1 in
  *  *first_ill_char if there was an error in the parameters.
- *  BASE may be in the range 2..36
+ *  Leading signs are not allowed.
  */
 #define MAXBASE	36
 EXPORT  long
 s_strtol (const String	  str,
-	  String_size	* first_ill_char,
-	  const int	  base		)
+	  String_size	* first_ill_char)
 
 {
     long		  number	= 0;	/* The result */
-    int			  sign		= 0;	/* Sign o' the times :-) */
     String_size		  char_no;
     int			  digit;
 
-
-    assert (base >= 2  &&  base <= MAXBASE);
-
-    if (s_empty(str))
-    {
-	*first_ill_char = -1;
-	return  0;
-    }
-
-
     /* Skip all blanks */
     char_no = 0;
     while ( (char_no < s_strlen(str)) && isspace(str.string[char_no]) )
 	char_no++;
 
-    /* Find any sign character (+ or -) */
-    if (str.string[char_no] == '+')
+    if (char_no == s_strlen(str))
     {
-	sign = 1;
-	char_no++;
+	*first_ill_char = -1;
+	return 0;
     }
-    else if (str.string[char_no] == '-')
-    {
-	sign = -1;
-	char_no++;
-    }
-
+    
     while (   (char_no < s_strlen(str))
-	   && ((digit = char2digit(str.string[char_no], base)) != -1) )
+	   && ((digit = char2digit(str.string[char_no])) != -1) )
     {
-	number = number * base + digit;
+	number = 10 * number + digit;
 	char_no++;
     }
 
-    *first_ill_char = char_no++;
-    if (sign != -1)
-	return  number;
-    else
-	return  -number;
+    *first_ill_char = char_no;
+    return  number;
 }
 
 
+extern Success
+s_trim_left(String *str,
+	    String_size rm)
+{
+    String tmp_str = EMPTY_STRING;
+
+    if (rm <= 0)
+	return OK;
+
+    if (rm >= s_strlen(*str))
+    {
+	s_clear(str);
+	return OK;
+    }
+
+    /* FIXME (bug XXX): in some cases, it might be more efficient
+       to move the data in-place instead of allocating a new block. */
+
+    if (s_substr(&tmp_str, *str, rm, END_OF_STRING) != OK)
+	return FAILURE;
+
+    s_clear(str);
+    *str = tmp_str;
+
+    return OK;
+}
+
+void *
+s_reserve(String *str,
+	  String_size sz)
+{
+    void *temp_ptr;
+    if (sz < 1)
+	return NULL;
+
+    temp_ptr = REALLOC_0(str->string, s_strlen(*str) + sz);
+    if (temp_ptr == NULL)
+	return NULL;
+
+    str->string = temp_ptr;
+    return (char*)temp_ptr + s_strlen(*str);
+}
+	
+
+extern void
+s_reserve_done(String *str,
+	       String_size sz)
+{
+    str->len += sz;
+}
+
 
 
 /************************************************
