@@ -1,6 +1,6 @@
 /*
- * $Id: aux-items.c,v 1.39 2001/12/30 22:32:54 ceder Exp $
- * Copyright (C) 1994-2001  Lysator Academic Computer Association.
+ * $Id: aux-items.c,v 1.61 2003/08/25 17:24:23 ceder Exp $
+ * Copyright (C) 1994-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
 
@@ -37,14 +37,16 @@
 #endif
 #include <setjmp.h>
 #include <assert.h>
+#include "timewrap.h"
 
+#include "aux-no.h"
 #include "kom-types.h"
 #include "com.h"
 #include "async.h"
 #include "connections.h"
+#include "kom-errno.h"
 #include "manipulate.h"
 #include "aux-items.h"
-#include "kom-errno.h"
 #include "kom-memory.h"
 #include "server/smalloc.h"
 #include "cache.h"
@@ -83,7 +85,7 @@ static void aux_item_trigger_link_item(Aux_item_trigger_data *data);
 
 /* Forward declarations of validators */
 
-static Bool aux_item_validate_existing_text(Aux_item_validation_data *data);
+static Success aux_item_validate_existing_text(Aux_item_validation_data *data);
 
 /* Other forward declarations */
 static const Aux_item *
@@ -130,6 +132,7 @@ Aux_item_definition empty_aux_item_definition =
     0,                      /* Supervisor only */
     0,                      /* System only */
     0,                      /* Unique */         
+    0,                      /* Unique-data */
     0,                      /* Can't delete */
     0,                      /* Owner delete */
     0,                      /* Inherit limit */  
@@ -166,6 +169,7 @@ static Aux_item_definition simple_aux_item =
     0,                      /* Supervisor only */
     0,                      /* System only */
     0,                      /* Unique */         
+    0,                      /* Unique-data */
     0,                      /* Can't delete */
     0,                      /* Owner delete */
     0,                      /* Inherit limit */  
@@ -426,7 +430,6 @@ aux_item_definition_cache_regexp(const Aux_item_definition *def,
         return;
     }
     pat_buf = def->validators[ix].v.re.cached_re_buf;
-    pat_buf->translate = DEFAULT_COLLAT_TAB;
     pat_buf->translate = NULL;
     pat_buf->fastmap = 0;
     pat_buf->allocated = 0;
@@ -443,7 +446,12 @@ aux_item_definition_cache_regexp(const Aux_item_definition *def,
                 def->tag,
                 def->name);
         if (def->validators[ix].v.re.cached_re_buf)
+        {
+	    /* If re_compile_pattern fails, it can leave allocated
+	       stuff in the buffer. */
+            regfree(def->validators[ix].v.re.cached_re_buf);
             sfree(def->validators[ix].v.re.cached_re_buf);
+        }
         if (def->validators[ix].v.re.regexp)
             string_free(def->validators[ix].v.re.regexp);
         def->validators[ix].v.re.cached_re_buf = NULL;
@@ -490,7 +498,10 @@ free_aux_item_definitions(void)
                 if (def->validators[i].v.re.regexp != NULL)
                     string_free(def->validators[i].v.re.regexp);
                 if (def->validators[i].v.re.cached_re_buf != NULL)
+                {
+                    regfree(def->validators[i].v.re.cached_re_buf);
                     sfree(def->validators[i].v.re.cached_re_buf);
+                }
             }
         }
         if (def->validators != NULL)
@@ -566,9 +577,6 @@ find_linked_aux_item_list(Aux_item *item)
     case INFO_OBJECT_TYPE:
         return &kom_info.aux_item_list;
 
-#if 0
-    case PERS_OBJECT_TYPE:
-#endif
     default:
         /* No lists in these items */
         kom_log("find_linked_aux_item_list: Bad aux_item somewhere: "
@@ -612,9 +620,6 @@ mark_linked_object_as_changed(Aux_item *item)
         break;
 
     case INFO_OBJECT_TYPE:
-#if 0
-    case PERS_OBJECT_TYPE:
-#endif
     default:
         /* Need no commit for these objects */
 	;
@@ -808,10 +813,9 @@ aux_item_list_add_items(Aux_item_list *add_to_list,
      * this function recursively.
      */
 
-    add_to_list->items = srealloc(add_to_list->items,
-                                  (add_to_list->length + items_to_add->length) *
-                                  sizeof(Aux_item));
     add_to_list->length += items_to_add->length;
+    add_to_list->items = srealloc(add_to_list->items,
+                                  add_to_list->length * sizeof(Aux_item));
     *highest_ptr = *highest_ptr + items_to_add->length;
 
     for (i = 0; i < items_to_add->length; i++)
@@ -863,9 +867,7 @@ prepare_aux_item_list(Aux_item_list *list,
         return;
     
     for (i = 0; i < list->length; i++)
-    {
         prepare_aux_item(&list->items[i], creator, NULL);
-    }
 }
 
 static void
@@ -889,18 +891,21 @@ prepare_aux_item(Aux_item *item,
     /* AUX_ADJUST_FLAG(deleted); */
     item->flags.deleted = 0;
 
-    if (def->inherit_limit != 0 &&
-        (item->inherit_limit > def->inherit_limit ||
-         item->inherit_limit == 0))
+    if (def->inherit_limit != 0
+	&& (item->inherit_limit > def->inherit_limit
+	    || item->inherit_limit == 0))
+    {
         item->inherit_limit = def->inherit_limit;
+    }
 
     item->creator = creator;
-    item->sent_at = current_time;
+    item->sent_at = current_time.tv_sec;
 }
 
 
 
-static Bool aux_item_validate(Aux_item_validation_data validation_data)
+static Success
+aux_item_validate(Aux_item_validation_data validation_data)
 {
     unsigned long               i;
     const Aux_item_definition        *def;
@@ -913,14 +918,14 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
         case AUX_VALIDATE_FUNCTION:
             /*
              * Call the validation function
-             * If it returns false, then abort validation
-             * If it returns true, then continue with the
+             * If it returns FAILURE, then abort validation
+             * If it returns OK, then continue with the
              * next validator
              */
 
-            if ((*def->validators[i].v.fn.function)(&validation_data) == FALSE)
+            if ((*def->validators[i].v.fn.function)(&validation_data) != OK)
             {
-                return FALSE;
+                return FAILURE;
             }
             break;
 
@@ -944,13 +949,13 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
             if (def->validators[i].v.re.cached_re_buf == NULL)
             {
                 kom_log("Failed to cache regexp validator for \"%s\". "
-			"Rejecting all such items\n",
+			"Rejecting all such items.\n",
                         def->name);
                 sfree(def->validators[i].v.re.regexp);
                 def->validators[i].v.re.regexp = NULL;
                 def->validators[i].type = AUX_VALIDATE_REJECT;
                 kom_errno = KOM_ILL_AUX;
-                return FALSE;
+                return FAILURE;
             }
 
             /*
@@ -958,7 +963,7 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
              */
 
             switch (re_search(def->validators[i].v.re.cached_re_buf,
-                              validation_data.item->data.string,
+                              (char*)validation_data.item->data.string,
                               s_strlen(validation_data.item->data),
                               0,
                               s_strlen(validation_data.item->data),
@@ -967,12 +972,12 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
             case -1:
                 /* No match */
                 kom_errno = KOM_ILL_AUX;
-                return FALSE;
+                return FAILURE;
             case -2:
                 /* Internal error may (be temporary) */
                 kom_log("Internal error in regex matching aux-item data.\n");
                 kom_errno = KOM_ILL_AUX;
-                return FALSE;
+                return FAILURE;
                 break;
             default:
                 /* Matched */
@@ -989,7 +994,7 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
              */
 
             kom_errno = KOM_ILL_AUX;
-            return FALSE;
+            return FAILURE;
             break;
 
         default:
@@ -1002,8 +1007,7 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
      * If we got all the way to here we passed all the validators
      */
 
-    return TRUE;
-
+    return OK;
 }
 
 
@@ -1014,38 +1018,39 @@ static Bool aux_item_validate(Aux_item_validation_data validation_data)
  *
  * ITEM is the item that you want to add.
  * DEF is the item's definition or NULL.
- * ITEM_CREATOR is the person who wants to create the item.
- * OBJECT_CREATOR is the person who created the object to which
- *     the item might be added. For texts it is the author, for
- *     persons it is the person and for conferences it is the
- *     conference supervisor.
- * OWNER_CHECK is TRUE when this function is supposed to check author
- *     and supervisor-related flags. The only case when this is to
- *     be FALSE is when checking author or supervisor makes not sense,
- *     such as when creating a completely new conference.
- * ADD_TO_LIST is the item list to add to, or NULL.
+ * CREATING_CONN is the connection that is causing this item to be
+ *     added.  It is NULL if the item is created due to inheritance.
+ * SUBORDINATE. If author-only or supervisor-only, the person logged
+ *     in on CREATING_CONN must be a supervisor of the SUBORDINATE to
+ *     be allowed to create the item.  For a text, the SUBORDINATE
+ *     should be set to the author of the text.  For a conference or
+ *     person, this is the conference or person itself.
+ * ADD_TO_LIST is the item list the item should be added to, or NULL.
  * START_LOOKING_AT is the first index in add_to_list to start
  *     looking at for possible duplicates.
- * CREATING is true if we are checking while creating an object.
+ * CREATING is true if we are checking while creating the object that
+ *     the item will be added to.
  * OBJECT_TYPE is the type of the object the item is added to.
  */
 
 static Bool
 aux_item_add_perm(const Aux_item *item,
                   const Aux_item_definition *def,
-                  Pers_no item_creator,
-                  Pers_no object_creator,
-                  Bool owner_check,
+                  Connection *creating_conn,
+                  Pers_no subordinate,
                   Aux_item_list *add_to_list,
                   unsigned long start_looking_at,
                   Bool creating,
-                  enum object_type object_type
-                  )
+                  enum object_type object_type)
 {
     short can_add_when = 0;
     
     kom_errno = KOM_NO_ERROR;
 
+    /* Either there is a connection responsible for the creation, or
+       we are inheriting.  If we are inheriting, creating must be true.  */
+    assert(creating_conn != NULL || creating);
+
     if (def == NULL)
 	def = find_aux_item_definition(item);
 
@@ -1072,55 +1077,49 @@ aux_item_add_perm(const Aux_item *item,
 
     if (can_add_when)
     {
-        if (!((creating  && (can_add_when & AUX_ITEM_ADD_ON_CREATE)) ||
-              (!creating && (can_add_when & AUX_ITEM_ADD_ON_MODIFY))))
+        if (!((creating && (can_add_when & AUX_ITEM_ADD_ON_CREATE))
+	      || (!creating && (can_add_when & AUX_ITEM_ADD_ON_MODIFY))))
         {
             kom_errno = KOM_AUX_PERM;
             return FALSE;
         }
     }
 
-    if (def->system_only &&
-        owner_check)
+    if (def->system_only)
     {
         kom_errno = KOM_AUX_PERM;
         return FALSE;
     }
 
-    /* FIXME (bug 334): is_strictly_supervisor() or is_supervisor()? */ 
-    /* FIXME (bug 334): the is_strictly_supervisor() call checks if
-       item_creator is supervisor of object_creator.  The intention is
-       to check if item_creator is supervisor of conference X, where
-       X's supervisor is object_creator. */
-    if (def->author_only &&
-        owner_check &&
-        object_creator != item_creator &&
-        !is_strictly_supervisor(object_creator, item_creator, NULL) &&
-        !(item_creator == ACTPERS && ENA(wheel, 8))) /* NOT OK! */
+    if ((def->author_only || def->supervisor_only)
+	&& !creating
+	&& creating_conn != NULL
+	&& !ENA_C(creating_conn, wheel, 8))
     {
-        kom_errno = KOM_AUX_PERM;
-        return FALSE;
-    }
+	if (def->author_only
+	    && !is_supervisor(subordinate, creating_conn))
+	{
+	    kom_errno = KOM_AUX_PERM;
+	    return FALSE;
+	}
 
-    /* FIXME (bug 334): is_strictly_supervisor() or is_supervisor()? */ 
-    /* FIXME (bug 334): the is_strictly_supervisor() call checks if
-       item_creator is supervisor of object_creator.  The intention is
-       to check if item_creator is supervisor of conference X, where
-       X's supervisor is object_creator. */
-    if (def->supervisor_only &&
-         owner_check &&
-        !is_strictly_supervisor(object_creator, item_creator, NULL) &&
-        !(item_creator == ACTPERS && ENA(wheel,8))) /* NOT OK! */
-    {
-        kom_errno = KOM_AUX_PERM;
-        return FALSE;
+	/* FIXME (bug 313): is_strictly_supervisor() or is_supervisor()? */ 
+	if (def->supervisor_only
+	    && !is_strictly_supervisor(subordinate,
+				       creating_conn->pers_no,
+				       creating_conn->person))
+	{
+	    kom_errno = KOM_AUX_PERM;
+	    return FALSE;
+	}
     }
 
     /* Even the administrator can't override this */
 
+    kom_errno = KOM_NO_ERROR;
     if (!aux_item_check_unique(item, def, add_to_list, start_looking_at))
     {
-        kom_errno = KOM_AUX_PERM;
+        kom_errno = kom_errno?kom_errno:KOM_AUX_PERM;
         return FALSE;
     }
 
@@ -1132,15 +1131,20 @@ aux_item_add_perm(const Aux_item *item,
 
         validation_data.item = item;
         validation_data.def = def;
-        validation_data.item_creator = item_creator;
-        validation_data.object_creator = object_creator;
-        validation_data.owner_check = owner_check;
+        validation_data.creating_conn = creating_conn;
+	validation_data.creating = creating;
+        validation_data.subordinate = subordinate;
+#if 0
+	/* Since no validator currently uses these fields, they
+	   have been temporarily removed from the
+	   Aux_item_validation_data type. */
+	
         validation_data.add_to_list = add_to_list;
         validation_data.start_looking_at = start_looking_at;
-        validation_data.creating = creating;
         validation_data.object_type = object_type;
+#endif
 
-        if (aux_item_validate(validation_data) != TRUE)
+        if (aux_item_validate(validation_data) != OK)
         {
             kom_errno = KOM_ILL_AUX;
             return FALSE;
@@ -1169,22 +1173,33 @@ aux_item_check_unique(const Aux_item *item,
 {
     unsigned long i;
 
-    if (add_to_list == NULL ||
-        item == NULL ||
-        start_looking_at >= add_to_list->length)
+    if (add_to_list == NULL
+	|| item == NULL
+	|| start_looking_at >= add_to_list->length)
+    {
         return TRUE;
+    }
 
     if (def != NULL)
 	def = find_aux_item_definition(item);
-    if (def == NULL || !def->one_per_person)
+    if (def == NULL)
         return TRUE;
 
     for (i = start_looking_at; i < add_to_list->length; i++)
     {
-        if (add_to_list->items[i].tag == item->tag &&
-            add_to_list->items[i].creator == item->creator &&
-            !add_to_list->items[i].flags.deleted)
+        if (def->one_per_person
+	    && add_to_list->items[i].tag == item->tag
+	    && add_to_list->items[i].creator == item->creator
+	    && !add_to_list->items[i].flags.deleted)
             return FALSE;
+
+	if (def->unique_data
+	    && add_to_list->items[i].tag == item->tag
+	    && s_streq(add_to_list->items[i].data, item->data))
+	{
+	    kom_errno = KOM_ILL_AUX;
+	    return FALSE;
+	}
     }
 
     return TRUE;
@@ -1194,20 +1209,18 @@ aux_item_check_unique(const Aux_item *item,
 /*
  * aux_inherit_items
  *
- * Inherit aux items from PARENT to TARGET. COUNTER is a pointer
- * to the aux_no counter in the object being inherited to and CREATOR
- * is the author or equivalent of the target object.
+ * Inherit aux items from PARENT to TARGET. COUNTER is a pointer to
+ * the aux_no counter in the object being inherited to and SUBORDINATE
+ * is the author of the target object.  The object is a text.
  */
 
 void
 aux_inherit_items(Aux_item_list *target,
                   const Aux_item_list *parent,
                   unsigned long *counter,
-                  Pers_no target_creator,
-                  Bool creating,
-                  enum object_type object_type,
-                  unsigned long object_no,
-                  void *object)
+                  Conf_no subordinate,
+                  Text_no object_no,
+                  Text_stat *object)
 {
     int i;
     Aux_item item;
@@ -1220,22 +1233,20 @@ aux_inherit_items(Aux_item_list *target,
 
     for (i = 0; i < parent->length; i++)
     {
-        if (parent->items[i].flags.inherit &&
-            !parent->items[i].flags.deleted &&
-            parent->items[i].inherit_limit != 1)
+        if (parent->items[i].flags.inherit
+	    && !parent->items[i].flags.deleted
+	    && parent->items[i].inherit_limit != 1)
 	{
             def = find_aux_item_definition(&parent->items[i]);
-            if (def == NULL ||
-                !aux_item_add_perm(&parent->items[i],
-                                   def,
-                                   parent->items[i].creator,
-                                   target_creator,
-                                   TRUE,
-                                   target,
-                                   0,
-                                   creating,
-                                   object_type
-                                   ))
+            if (def == NULL
+		|| !aux_item_add_perm(&parent->items[i],
+				      def,
+				      NULL,
+				      subordinate,
+				      target,
+				      0,
+				      TRUE,
+				      TEXT_OBJECT_TYPE))
                 continue;
 
             copy_aux_item(&item, &parent->items[i]);
@@ -1252,7 +1263,7 @@ aux_inherit_items(Aux_item_list *target,
             target->length += 1;
 
             aux_item_call_add_triggers(def,
-                                       object_type,
+                                       TEXT_OBJECT_TYPE,
                                        target->length - 1,
                                        object_no,
                                        object,
@@ -1275,8 +1286,8 @@ filter_aux_item_list(const Aux_item_list *original,
 {
     Aux_item        *orig_aux;
     unsigned long    from, to;
-    /*    Pers_no          viewer;
-          Person        *viewer_p;*/
+
+    assert(viewer_conn != NULL);
 
     result->items = tmp_alloc(original->length * sizeof (Aux_item));
     result->length = 0;
@@ -1285,19 +1296,21 @@ filter_aux_item_list(const Aux_item_list *original,
     for (from = 0; from < original->length; from++)
     {
         orig_aux = &original->items[from];
-        if ( orig_aux->flags.secret &&
-             !is_supervisor(orig_aux->creator,
-                            viewer_conn->pers_no, viewer_conn->person) &&
-             !ENA_C(viewer_conn, admin, 4))
+        if (orig_aux->flags.secret
+	    && !is_supervisor(orig_aux->creator, viewer_conn)
+	    && !ENA_C(viewer_conn, admin, 4))
+	{
             continue;
-
+	}
+	
         result->items[to] = *orig_aux;
 
-        if (orig_aux->flags.hide_creator &&
-             !is_supervisor(orig_aux->creator,
-                            viewer_conn->pers_no, viewer_conn->person) &&
-             !ENA_C(viewer_conn, admin, 4))
+        if (orig_aux->flags.hide_creator
+	    && !is_supervisor(orig_aux->creator, viewer_conn)
+	    && !ENA_C(viewer_conn, admin, 4))
+	{
             result->items[to].creator = 0;
+	}
 
         result->length += 1;
         to += 1;
@@ -1314,7 +1327,7 @@ filter_aux_item_list(const Aux_item_list *original,
 Success
 check_delete_aux_item_list(const Number_list *items_to_delete,
                            const Aux_item_list *list_to_delete_from,
-                           const Conf_no owner)
+                           const Conf_no subordinate)
 {
     long i;
     const Aux_item *item;
@@ -1365,9 +1378,10 @@ check_delete_aux_item_list(const Number_list *items_to_delete,
         }
 	else if (item->creator != ACTPERS)
 	{
-            if (!ENA(wheel, 8) &&
-                !is_supervisor(item->creator, ACTPERS, NULL) &&
-                (!def->owner_delete || !is_supervisor(owner, ACTPERS, NULL)))
+            if (!ENA(wheel, 8)
+		&& !is_supervisor(item->creator, active_connection)
+		&& (!def->owner_delete
+		    || !is_supervisor(subordinate, active_connection)))
 	    {
                 kom_errno = KOM_AUX_PERM;
                 err_stat = i;
@@ -1427,25 +1441,25 @@ delete_aux_item_list(const Number_list *items_to_delete,
                                       object_no,
                                       object,
                                       item);
-        /* If we have linked items, undelete them too.
+        /* If we have linked items, delete them too.
            It is safe to do this with a recursive call
-           since we have marked this item as undeleted.
+           since we have marked this item as deleted.
            But just to avoid pointless recursion, only
            do the recursive call if the corresponding
-           item is marked as deleted */
+           item isn't marked as deleted */
 
         linked_item = find_linked_aux_item(item);
         if (linked_item && !linked_item->flags.deleted)
         {
-            /* Set up the undelete list for the target object */
+            /* Set up the delete list for the target object */
             item_to_delete = item->linked_item.target_item;
             linked_delete.length = 1;
             linked_delete.data = &item_to_delete;
 
-            /* Find the item list to undelete from */
+            /* Find the item list to delete from */
             linked_item_list = find_linked_aux_item_list(item);
 
-            /* Set up the object and object_no parameters for undelete */
+            /* Set up the object and object_no parameters for delete */
             switch (item->linked_item.target_type)
             {
             case CONF_OBJECT_TYPE:
@@ -1609,15 +1623,14 @@ commit_aux_item_list_internal(Aux_item_list *list_to_commit, Bool shallow)
                linked item is marked as deleted */
 
             linked_item = find_linked_aux_item(&list_to_commit->items[i]);
-            if (linked_item                     &&
-                linked_item->flags.deleted      &&
-                !shallow)
+            if (linked_item
+		&& linked_item->flags.deleted
+		&& !shallow)
             {
                 aux_item_list = find_linked_aux_item_list(&list_to_commit->items[i]);
                 commit_aux_item_list_internal(aux_item_list, TRUE);
                 mark_linked_object_as_changed(&list_to_commit->items[i]);
             }
-            continue;
         }
         else if (target != i)
         {
@@ -1685,7 +1698,7 @@ find_aux_item(const Aux_item_list *list, unsigned long aux_no)
  * The following functions are defined for each type
  *
  * X_check_add_aux_item_list(X *obj, Aux_item_list *list,
- *                      Pers_no item_creator)
+ *                           Connection *creating_conn)
  *
  * Return OK if CREATOR is allowed to add ITEM or LIST to OBJ.
  * DEF is a pointer to the definition of ITEM or NULL. You must call
@@ -1708,13 +1721,14 @@ find_aux_item(const Aux_item_list *list, unsigned long aux_no)
 Success
 text_stat_check_add_aux_item_list(Text_stat *text_s,
                                   Aux_item_list *list,
-                                  Pers_no item_creator,
-                                  Bool creating)
+                                  Connection *creating_conn)
 {
     Aux_item *item;
     unsigned long i;
     const Aux_item_definition *def;
 
+    assert(creating_conn != NULL);
+
     if (list == NULL)
         return OK;
 
@@ -1734,15 +1748,14 @@ text_stat_check_add_aux_item_list(Text_stat *text_s,
         
         if (!aux_item_add_perm(item,
                                def,
-                               item_creator,
-                               text_s?text_s->author:item_creator,
-                               TRUE,
+                               creating_conn,
+                               text_s?text_s->author:creating_conn->pers_no,
                                text_s?&text_s->aux_item_list:NULL,
                                0,
-                               creating,
-                               TEXT_OBJECT_TYPE) ||
-            !aux_item_check_unique(item, def, list, i + 1) ||
-            !def->texts)
+                               text_s == NULL,
+                               TEXT_OBJECT_TYPE)
+	    || !aux_item_check_unique(item, def, list, i + 1)
+	    || !def->texts)
         {
             kom_errno = kom_errno?kom_errno:KOM_AUX_PERM;
             err_stat = i;
@@ -1775,13 +1788,17 @@ Success
 conf_stat_check_add_aux_item_list(Conference    *conf,
                                   Conf_no        conf_no,
                                   Aux_item_list *list,
-                                  Pers_no        creator,
+                                  Connection    *creating_conn,
                                   Bool           creating)
 {
     unsigned long i;
     Aux_item *item;
     const Aux_item_definition *def;
 
+    assert(conf != NULL);
+    assert(conf_no != 0);
+    assert(creating_conn != NULL);
+
     if (list == NULL)
         return OK;
     
@@ -1800,16 +1817,15 @@ conf_stat_check_add_aux_item_list(Conference    *conf,
 
         if (!aux_item_add_perm(item,
                                def,
-                               creator,
+                               creating_conn,
                                conf_no,
-                               TRUE,
                                &conf->aux_item_list,
                                0,
                                creating,
-                               CONF_OBJECT_TYPE)            ||
-            !aux_item_check_unique(item, def, list, i + 1)  ||
-            (!conf->type.letter_box && !def->confs)         ||
-            (conf->type.letter_box && !def->letterboxes))
+                               CONF_OBJECT_TYPE)
+	    || !aux_item_check_unique(item, def, list, i + 1)
+	    || (!conf->type.letter_box && !def->confs)
+	    || (conf->type.letter_box && !def->letterboxes))
         {
             kom_errno = kom_errno?kom_errno:KOM_AUX_PERM;
             err_stat = i;
@@ -1839,12 +1855,14 @@ conf_stat_add_aux_item_list(Conference    *conf,
 
 Success system_check_add_aux_item_list(Info *info,
                                        Aux_item_list *list,
-                                       Pers_no creator)
+                                       Connection *creating_conn)
 {
     unsigned long i;
     Aux_item *item;
     const Aux_item_definition *def;
 
+    assert(creating_conn != NULL);
+
     if (list == NULL)
         return OK;
     
@@ -1861,17 +1879,21 @@ Success system_check_add_aux_item_list(Info *info,
             return FAILURE;
         }
 
+	/* Pass creating_conn->pers_no as the subordinate, so that the
+	   owner checks will be short-circuited.  When we arrive here,
+	   we already know that the user has enough privileges to
+	   create aux-items on the system.  See modify_system_info(),
+	   especially the call to ENA().  */
         if (!aux_item_add_perm(item,                    /* item */
                                def,                     /* definition */
-                               creator,                 /* item_creator */
-                               creator,                 /* object_creator */
-                               TRUE,                    /* owner_check */
+                               creating_conn,           /* creating_conn */
+                               creating_conn->pers_no,  /* subordinate */
                                &info->aux_item_list,    /* add_to_list */
                                0,                       /* start_looking_at */
                                FALSE,                   /* creating */
-                               INFO_OBJECT_TYPE) ||
-            !aux_item_check_unique(item, def, list, i + 1) ||
-            !def->system)
+                               INFO_OBJECT_TYPE)
+	    || !aux_item_check_unique(item, def, list, i + 1)
+	    || !def->system)
         {
             kom_errno = kom_errno?kom_errno:KOM_AUX_PERM;
             err_stat = i;
@@ -1932,7 +1954,7 @@ aux_item_trigger_mark_text(Aux_item_trigger_data *data)
     Text_stat *text_stat;
     String_size ill_char;
 
-    text_no = s_strtol(data->item->data, &ill_char, 10);
+    text_no = s_strtol(data->item->data, &ill_char);
     VOID_GET_T_STAT(text_stat, text_no);
     if (text_stat->no_of_marks < param.max_marks_text)
       text_stat->no_of_marks += 1;
@@ -1946,7 +1968,7 @@ aux_item_trigger_unmark_text(Aux_item_trigger_data *data)
     Text_stat       *text_stat;
     String_size      ill_char;
 
-    text_no = s_strtol(data->item->data, &ill_char, 10);
+    text_no = s_strtol(data->item->data, &ill_char);
     VOID_GET_T_STAT(text_stat, text_no);
     if (text_stat->no_of_marks > 0)
       text_stat->no_of_marks -= 1;
@@ -1963,12 +1985,18 @@ aux_item_trigger_mirror_faq(Aux_item_trigger_data *data)
     String           str;
     
 
-    if (data->object_type != CONF_OBJECT_TYPE || data->object == NULL)
+    if (data->object == NULL)
+	return;
+
+    if (data->object_type != CONF_OBJECT_TYPE
+	&& data->object_type != INFO_OBJECT_TYPE)
+    {
         return;
+    }
 
     /* Get the object number of where we want to put the new item */
 
-    text_no = s_strtol(data->item->data, &ill_char, 10);
+    text_no = s_strtol(data->item->data, &ill_char);
 
     /* Create the string for the new item */
 
@@ -1978,7 +2006,7 @@ aux_item_trigger_mirror_faq(Aux_item_trigger_data *data)
     /* Fill in the new item  */
 
     init_aux_item(&item_data);
-    item_data.tag = 28;         /* Mirror of FAQ item */
+    item_data.tag = aux_faq_for_conf;         /* Mirror of FAQ item */
     item_data.data = str;
 
 
@@ -2112,15 +2140,21 @@ aux_item_trigger_link_item(Aux_item_trigger_data *data)
  */
 
 
-static Bool
+static Success
 aux_item_validate_existing_text(Aux_item_validation_data *v_data)
 {
     Text_stat       *text_stat;
     Text_no          text_no;
     String_size      ill_char;
 
-    text_no = s_strtol(v_data->item->data, &ill_char, 10);
-    GET_T_STAT(text_stat, text_no, FALSE);
-    return person_text_read_access(text_no, text_stat,
-                                   v_data->item_creator, NULL);
+    text_no = s_strtol(v_data->item->data, &ill_char);
+    GET_T_STAT(text_stat, text_no, FAILURE);
+
+    if (v_data->creating_conn != NULL
+	&& text_read_access(v_data->creating_conn, text_no, text_stat))
+    {
+	return OK;
+    }
+
+    return FAILURE;
 }
