@@ -1,6 +1,6 @@
 /*
- * $Id: conference.c,v 0.70 2001/12/28 20:00:03 ceder Exp $
- * Copyright (C) 1991-2001  Lysator Academic Computer Association.
+ * $Id: conference.c,v 0.92 2003/08/23 16:38:17 ceder Exp $
+ * Copyright (C) 1991-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * conference.c
@@ -35,7 +35,7 @@
 
 #include <ctype.h>
 #include <stdio.h>
-#include <time.h>
+#include "timewrap.h"
 #include <setjmp.h>
 #include <sys/types.h>
 
@@ -50,12 +50,12 @@
 #include "com.h"
 #include "async.h"
 #include "connections.h"
+#include "kom-errno.h"
 #include "manipulate.h"
 #include "server/smalloc.h"
 #include "kom-config.h"
 #include "parser.h"
 #include "internal-connections.h"
-#include "kom-errno.h"
 #include "lyskomd.h"
 #include "debug.h"
 #include "send-async.h"
@@ -63,6 +63,7 @@
 #include "aux-items.h"
 #include "local-to-global.h"
 #include "server-time.h"
+#include "stats.h"
 
 BUGDECL;
 
@@ -101,14 +102,7 @@ do_delete_conf (Conf_no      conf_no,
 
     sfree( conf_c->members.members );
     conf_c->members.members = NULL;
-    
-    /* Delete the name.
-       FIXME: This is done by cached_delete_conf */
-    /* FIXME (bug 145): I don't understand the above comment.  Check
-       this for memory leaks and memory corruption. */
-    s_clear( &conf_c->name );
-    cached_change_name( conf_no, EMPTY_STRING);
-    
+
     /* texts */
 
     /*
@@ -123,9 +117,10 @@ do_delete_conf (Conf_no      conf_no,
      * cannot modify all those text statuses in a single atomic call.
      */
 
-    /* FIXME (bug 93): There should be an async message about the deletion. */
+    /* FIXME (bug 905): There should be an async message about the deletion. */
 
     cached_delete_conf( conf_no );
+    update_stat(STAT_CONFS, -1);
     
     return;
 }
@@ -134,6 +129,19 @@ do_delete_conf (Conf_no      conf_no,
  * Functions that are exported to the server.
  */
 
+void
+set_conf_errno(const Connection *viewer_conn,
+	       Conf_no conf_no,
+	       enum kom_err errcode)
+{
+    err_stat = conf_no;
+    if (!has_access(conf_no, viewer_conn, read_protected))
+	kom_errno = KOM_UNDEF_CONF;
+    else
+	kom_errno = errcode;
+}
+
+
 /*
  * Return TRUE if NAME is not already used. Set kom_errno to
  * KOM_LONG_STR or KOM_BAD_NAME if it fails.
@@ -264,6 +272,7 @@ do_create_conf(String	 name,
 	       Conf_no	 supervisor,
 	       Conf_no	 super_conf,
 	       Conf_type type,
+	       Connection *creating_connection,
                Aux_item_list *aux)
 {
     Conf_no 	 conf_no;
@@ -287,7 +296,7 @@ do_create_conf(String	 name,
     }
 
     conf_c->creator	= creator;
-    conf_c->creation_time = current_time;
+    conf_c->creation_time = current_time.tv_sec;
     conf_c->presentation= 0;		/* No presentation yet */
     conf_c->supervisor	= supervisor;
     conf_c->permitted_submitters = 0;
@@ -308,7 +317,7 @@ do_create_conf(String	 name,
     if (conf_stat_check_add_aux_item_list(conf_c,
                                           conf_no,
                                           aux,
-                                          creator,
+                                          creating_connection,
                                           TRUE) != OK)
     {
 	/* FIXME (bug 146): Conf_no leak: We create a conference, check the
@@ -323,6 +332,7 @@ do_create_conf(String	 name,
     conf_stat_add_aux_item_list(conf_c, conf_no, aux, creator);
 
     mark_conference_as_changed (conf_no);
+    update_stat(STAT_CONFS, 1);
 
     return conf_no;
 }
@@ -334,13 +344,16 @@ do_create_conf(String	 name,
 
 Bool
 is_supervisor(Conf_no conf,
-              Pers_no viewer,
-              const Person *viewer_p)
+	      const Connection *viewer)
 {
-    if (viewer == conf)	/* A person is ALWAYS supervisor to	*/
-        return TRUE;		/* his/her own mailbox!		*/
+    if (viewer->pers_no == 0)
+	return FALSE;
 
-    return is_strictly_supervisor(conf, viewer, viewer_p);
+    /* A person is ALWAYS supervisor to his/her own mailbox!  */
+    if (viewer->pers_no == conf)
+        return TRUE;
+
+    return is_strictly_supervisor(conf, viewer->pers_no, viewer->person);
 }
 
 
@@ -490,7 +503,8 @@ create_conf_generic(const String  name,
 	return 0;
     }
     
-    conf_no = do_create_conf(name, ACTPERS, ACTPERS, ACTPERS, type, aux);
+    conf_no = do_create_conf(name, ACTPERS, ACTPERS, ACTPERS, type,
+			     active_connection, aux);
 
     if ( conf_no != 0)
     {
@@ -612,7 +626,7 @@ lookup_name (const String    name,
 
     for ( i = result->no_of_conf_nos; i > 0; i-- )
     {
-	if (access_perm(*no, active_connection, read_protected) <= none)
+	if (!has_access(*no, active_connection, read_protected))
 	    --result->no_of_conf_nos;
 	else
 	{
@@ -640,6 +654,8 @@ lookup_z_name (const String       name,
     Conf_list_old raw_matches;
     
     CHK_CONNECTION(FAILURE);
+    CHK_BOOL(want_persons, FAILURE);
+    CHK_BOOL(want_confs, FAILURE);
     if (cached_lookup_name(name, &raw_matches) != OK)
 	return FAILURE;
 
@@ -652,7 +668,7 @@ lookup_z_name (const String       name,
 	    restart_kom("Internal error detected in lookup_z_name\n");
 
 	if ((type->letter_box ? want_persons : want_confs) == 0
-	    || access_perm(*no, active_connection, read_protected) <= none)
+	    || !has_access(*no, active_connection, read_protected))
 	{
 	    *no = 0;
 	    n_filtered--;
@@ -695,6 +711,7 @@ do_lookup (Connection *conn,
     Conf_list_old raw_match;
     unsigned long i;
     unsigned short retsize;
+    unsigned int letterflag = want_persons;
 
     if ( cached_lookup_name( name, &raw_match ) != OK )
 	return FAILURE;
@@ -705,7 +722,7 @@ do_lookup (Connection *conn,
     {
 	/* Don't call check access permissions here. It doesn't matter
 	   that much if we allocate slightly too much memory. */
-	if (raw_match.type_of_conf[i].letter_box == want_persons)
+	if (raw_match.type_of_conf[i].letter_box == letterflag)
 	{
 	    retsize++;
 	    /* i is "unsigned long", and retsize is "unsigned short".
@@ -725,8 +742,8 @@ do_lookup (Connection *conn,
 
     for (i = 0; i < raw_match.no_of_conf_nos; i++)
     {
-	if (raw_match.type_of_conf[i].letter_box == want_persons
-	    && access_perm(raw_match.conf_nos[i], conn, read_protected) > none)
+	if (raw_match.type_of_conf[i].letter_box == letterflag
+	    && has_access(raw_match.conf_nos[i], conn, read_protected))
 	{
 	    result->conf_nos[result->no_of_confs++] = raw_match.conf_nos[i];
 	    if (result->no_of_confs > retsize)
@@ -908,7 +925,7 @@ set_presentation (Conf_no	conf_no,
  *
  * Only the supervisor can change the motd.
  *
- * FIXME (bug 93): There is no asynchronous message that reports new motds.
+ * FIXME (bug 906): There is no asynchronous message that reports new motds.
  */
 extern Success
 set_etc_motd(	Conf_no		conf_no,
@@ -1000,9 +1017,9 @@ set_permitted_submitters (Conf_no  conf_no,
 	CHK_EXIST(new_perm_sub, FAILURE);
     GET_C_STAT(conf_c, conf_no, FAILURE);
 
-    if ( !is_supervisor(conf_no, ACTPERS, ACT_P) &&
-         !ENA(wheel, 8) &&      /* OK -- In an RPC call */
-         !ENA(admin, 6) )       /* OK -- In an RPC call */
+    if (!is_supervisor(conf_no, active_connection)
+	&& !ENA(wheel, 8)	/* OK -- In an RPC call */
+	&& !ENA(admin, 6))	/* OK -- In an RPC call */
     {
         err_stat = conf_no;
 	kom_errno = conf_c->type.secret ? KOM_UNDEF_CONF : KOM_PERM;
@@ -1033,9 +1050,9 @@ set_super_conf (Conf_no  conf_no,
     if (new_super_conf != 0)
 	CHK_EXIST(new_super_conf, FAILURE);
 
-    if ( !is_supervisor(conf_no, ACTPERS, ACT_P) &&
-         !ENA(wheel, 8) &&      /* OK -- In an RPC call */
-         !ENA(admin, 5) )       /* OK -- In an RPC call */
+    if (!is_supervisor(conf_no, active_connection)
+	&& !ENA(wheel, 8)	/* OK -- In an RPC call */
+	&& !ENA(admin, 5))	/* OK -- In an RPC call */
     {
         err_stat = conf_no;
 	kom_errno = conf_c->type.secret ? KOM_UNDEF_CONF : KOM_PERM;
@@ -1053,8 +1070,7 @@ set_super_conf (Conf_no  conf_no,
  * set the conf_type. The letter_box flag can not be changed.
  *
  * FIXME (bug 70): It is allowed to set the type to 'secret' for
- * persons. I don't think we want it to be that way.
- * Well, I do /DCB.
+ * persons. I don't think we want it to be that way. /ceder
  */
 extern Success
 set_conf_type (Conf_no    conf_no,
@@ -1195,9 +1211,7 @@ set_expire(	Conf_no		conf_no,
 }
 
 /*
- * Set expire for a conference. This controls nothing at the moment, but
- * will probably control when a conference is killed automatically by
- * the server in cases of extreme inactivity.
+ * Set keep_commented for a conference.
  */
 extern Success
 set_keep_commented(Conf_no	conf_no,
@@ -1225,6 +1239,44 @@ set_keep_commented(Conf_no	conf_no,
     return OK;
 }
 
+static void
+send_async_new_presentation(Conf_no     conf_no,
+			    Text_no     old_tno,
+			    Text_stat  *old_stat,
+			    Text_no     new_tno,
+			    Text_stat  *new_stat)
+{
+    Session_no i = 0;
+    Connection *cptr;
+    Text_no used_old;
+    Text_no used_new;
+
+    if (!param.send_async_messages)
+	return;
+     
+    while ((i = traverse_connections(i)) != 0)
+    {
+	cptr = get_conn_by_number(i);
+     
+	/* Check the want_async here to avoid the potentially
+	    expensive calls to access_perm. */
+	if (cptr->want_async[ay_new_presentation] == FALSE)
+	    continue;
+
+	if (!has_access(conf_no, cptr, read_protected))
+	    continue;
+
+	used_old = text_read_access(cptr, old_tno, old_stat) ? old_tno : 0;
+	used_new = text_read_access(cptr, new_tno, new_stat) ? new_tno : 0;
+
+	if (used_old == used_new)
+	    continue;
+
+	async_new_presentation(cptr, conf_no, used_old, used_new);
+    }
+}
+
+
 /*
  * Change presentation of a conference. If text_no is 0, there will be
  * no presentation.
@@ -1233,25 +1285,24 @@ set_keep_commented(Conf_no	conf_no,
 Success
 do_set_presentation(Conf_no 	 conf_no,
 		    Conference * conf_c,
-		    Text_no	 text_no)
+		    Text_no	 new_text_no)
 {
-    Text_stat * old_pres;
-    Text_stat * new_pres = NULL; /* Initialized to stop gcc complaining
-				  * about this being used uninitialized.
-				  * This initialization is in fact not needed
-				  * since it would not be used uninitialized.*/
+    Text_stat * old_stat = NULL;
+    Text_stat * new_stat = NULL;
+    Text_no     old_text_no = 0;
 
     /* Check that the new presentation exists before deleting the old*/
 
-    if ( text_no != 0 )
+    if (new_text_no != 0)
     {
-	GET_T_STAT(new_pres, text_no, FAILURE);
-	if ( new_pres->no_of_marks >= param.max_marks_text )
+	GET_T_STAT(new_stat, new_text_no, FAILURE);
+	if (new_stat->no_of_marks >= param.max_marks_text)
 	{
-	    kom_log("%s(%d, ptr, %lu): New presentation has %d marks.\n", 
-		"LIMIT: do_set_presentation",
-		conf_no, (unsigned long)text_no, new_pres->no_of_marks);
-            err_stat = text_no;
+	    kom_log("LIMIT: do_set_presentation(%d, ptr, %lu): "
+		    "New presentation has %d marks.\n",
+		    conf_no,
+		    (unsigned long)new_text_no, new_stat->no_of_marks);
+            err_stat = new_text_no;
 	    kom_errno = KOM_MARK_LIMIT;
 	    return FAILURE;
 	}
@@ -1259,34 +1310,88 @@ do_set_presentation(Conf_no 	 conf_no,
     
     /* Unmark the previous presentation if it exists. */
 
-    if ( conf_c->presentation != 0
-	&& (old_pres = cached_get_text_stat(conf_c->presentation)) != NULL)
+    old_text_no = conf_c->presentation;
+    if (old_text_no != 0)
     {
-	if ( old_pres->no_of_marks > 0 )
+	if ((old_stat = cached_get_text_stat(old_text_no)) == NULL)
 	{
-	    --old_pres->no_of_marks;
-	    mark_text_as_changed( conf_c->presentation );
+	    /* FIXME (bug 85): This is not currently an error, but it
+	       will be once bug 85 is fixed.  */
+	    kom_log("do_set_presentation(): Old presentation %lu of conf %lu"
+		    " does not exist.\n",
+		    (unsigned long)old_text_no, (unsigned long)conf_no);
+	    old_text_no = 0;
+	}
+	else if (old_stat->no_of_marks > 0)
+	{
+	    --old_stat->no_of_marks;
+	    mark_text_as_changed(old_text_no);
 	}
 	else
 	{
-	    kom_log("ERROR: do_set_presentation(): Old presentation not marked\n");
+	    kom_log("ERROR: do_set_presentation(%d, ptr, %lu): "
+		    "Old presentation %lu not marked.\n",
+		    conf_no, 
+		    (unsigned long)new_text_no,
+		    (unsigned long)old_text_no);
 	}
     }
 
     /* Mark the new presentation */
 
-    if ( text_no != 0 )
+    if (new_text_no != 0)
     {
-	++new_pres->no_of_marks;
-	mark_text_as_changed( text_no );
+	++new_stat->no_of_marks;
+	mark_text_as_changed(new_text_no);
     }
     
-    conf_c->presentation = text_no;
-    mark_conference_as_changed( conf_no );
+    conf_c->presentation = new_text_no;
+    mark_conference_as_changed(conf_no);
 
+    send_async_new_presentation(conf_no,
+				old_text_no, old_stat,
+				new_text_no, new_stat);
     return OK;
 }
 
+
+static void
+send_async_new_motd(Conf_no     conf_no,
+		    Text_no     old_tno,
+		    Text_stat  *old_stat,
+		    Text_no     new_tno,
+		    Text_stat  *new_stat)
+{
+    Session_no i = 0;
+    Connection *cptr;
+    Text_no used_old;
+    Text_no used_new;
+
+    if (!param.send_async_messages)
+	return;
+     
+    while ((i = traverse_connections(i)) != 0)
+    {
+	cptr = get_conn_by_number(i);
+     
+	/* Check the want_async here to avoid the potentially
+	    expensive calls to access_perm. */
+	if (cptr->want_async[ay_new_motd] == FALSE)
+	    continue;
+
+	if (!has_access(conf_no, cptr, read_protected))
+	    continue;
+
+	used_old = text_read_access(cptr, old_tno, old_stat) ? old_tno : 0;
+	used_new = text_read_access(cptr, new_tno, new_stat) ? new_tno : 0;
+
+	if (used_old == used_new)
+	    continue;
+
+	async_new_motd(cptr, conf_no, used_old, used_new);
+    }
+}
+
 /*
  * Change motd of a conference. If text_no is 0, there will be
  * no motd.
@@ -1295,21 +1400,24 @@ do_set_presentation(Conf_no 	 conf_no,
 Success
 do_set_etc_motd(Conf_no      conf_no,
 		Conference * conf_c,
-		Text_no      text_no)
+		Text_no      new_text_no)
 {
-    Text_stat * old_motd;
-    Text_stat * new_motd = NULL; /* To stop gcc complaining. */
+    Text_stat * old_stat = NULL;
+    Text_stat * new_stat = NULL;
+    Text_no     old_text_no = 0;
 
     /* Check that the new motd exists before deleting the old*/
 
-    if ( text_no != 0 )
+    if (new_text_no != 0)
     {
-	GET_T_STAT(new_motd, text_no, FAILURE);
-	if ( new_motd->no_of_marks >= param.max_marks_text )
+	GET_T_STAT(new_stat, new_text_no, FAILURE);
+	if (new_stat->no_of_marks >= param.max_marks_text)
 	{
-	    kom_log("LIMIT: do_set_motd(%d, ptr, %lu): New motd has %d marks.\n",
-		conf_no, (unsigned long)text_no, new_motd->no_of_marks);
-            err_stat = text_no;
+	    kom_log("LIMIT: do_set_etc_motd(%d, ptr, %lu): "
+		    "New motd has %d marks.\n",
+		    conf_no,
+		    (unsigned long)new_text_no, new_stat->no_of_marks);
+            err_stat = new_text_no;
 	    kom_errno = KOM_MARK_LIMIT;
 	    return FAILURE;
 	}
@@ -1317,31 +1425,47 @@ do_set_etc_motd(Conf_no      conf_no,
     
     /* Unmark the previous motd if it exists. */
 
-    if ( conf_c->msg_of_day != 0
-	&& (old_motd = cached_get_text_stat(conf_c->msg_of_day)) != NULL)
+    old_text_no = conf_c->msg_of_day;
+    if (old_text_no != 0)
     {
-	if ( old_motd->no_of_marks > 0 )
+	if ((old_stat = cached_get_text_stat(old_text_no)) == NULL)
 	{
-	    --old_motd->no_of_marks;
-	    mark_text_as_changed( conf_c->msg_of_day );
+	    /* FIXME (bug 86): This is not currently an error, but it
+	       will be once bug 86 is fixed.  */
+	    kom_log("do_set_etc_motd(): Old motd %lu of conf %lu"
+		    " does not exist.\n",
+		    (unsigned long)old_text_no, (unsigned long)conf_no);
+	    old_text_no = 0;
+	}
+	else if (old_stat->no_of_marks > 0)
+	{
+	    --old_stat->no_of_marks;
+	    mark_text_as_changed(old_text_no);
 	}
 	else
 	{
-	    kom_log("ERROR: do_set_motd(): Old motd not marked\n");
+	    kom_log("ERROR: do_set_etc_motd(%d, ptr, %lu): "
+		    "Old motd %lu not marked.\n",
+		    conf_no, 
+		    (unsigned long)new_text_no,
+		    (unsigned long)old_text_no);
 	}
     }
 
     /* Mark the new motd */
 
-    if ( text_no != 0 )
+    if (new_text_no != 0)
     {
-	++new_motd->no_of_marks;
-	mark_text_as_changed( text_no );
+	++new_stat->no_of_marks;
+	mark_text_as_changed(new_text_no);
     }
     
-    conf_c->msg_of_day = text_no;
-    mark_conference_as_changed( conf_no );
+    conf_c->msg_of_day = new_text_no;
+    mark_conference_as_changed(conf_no);
 
+    send_async_new_motd(conf_no,
+			old_text_no, old_stat,
+			new_text_no, new_stat);
     return OK;
 }
 
@@ -1371,7 +1495,7 @@ modify_conf_info(Conf_no        conf_no,
     }
 
     GET_C_STAT(conf, conf_no, FAILURE);
-    if (access_perm(conf_no, active_connection, read_protected) <= none)
+    if (!has_access(conf_no, active_connection, read_protected))
     {
         err_stat  = conf_no;
         kom_errno = KOM_UNDEF_CONF;
@@ -1384,14 +1508,14 @@ modify_conf_info(Conf_no        conf_no,
 
     if (check_delete_aux_item_list(items_to_delete,
                                    &conf->aux_item_list,
-                                   conf->supervisor)!=OK)
+                                   conf_no)!=OK)
         return FAILURE;
     delete_aux_item_list(items_to_delete, &conf->aux_item_list,
                          CONF_OBJECT_TYPE,
                          conf_no, conf);
 
-    if (conf_stat_check_add_aux_item_list(conf, conf_no, aux, ACTPERS,
-                                          FALSE) != OK)
+    if (conf_stat_check_add_aux_item_list(conf, conf_no, aux,
+					  active_connection, FALSE) != OK)
     {
         undelete_aux_item_list(items_to_delete, &conf->aux_item_list,
                                CONF_OBJECT_TYPE,
@@ -1404,8 +1528,65 @@ modify_conf_info(Conf_no        conf_no,
     conf_stat_add_aux_item_list(conf, conf_no, aux, ACTPERS);
     commit_aux_item_list(&conf->aux_item_list);
 
-    /* FIXME (bug 93): async_conf_changed(conf_no, conf);  */
+    /* FIXME (bug 907): async_conf_changed(conf_no, conf);  */
     mark_conference_as_changed(conf_no);
 
     return OK;
 }
+
+
+Success
+first_unused_conf_no(Conf_no *result)
+{
+    CHK_CONNECTION(FAILURE);
+
+    *result = query_next_conf_no();
+    return OK;
+}
+
+Success
+find_next_conf_no(Conf_no start,
+		  Conf_no *result)
+{
+    Conf_no    highest = query_next_conf_no();
+
+    CHK_CONNECTION(FAILURE);
+    CHK_LOGIN(FAILURE);
+    err_stat = start;
+    while (++start < highest)
+	if (has_access(start, active_connection, read_protected))
+	{
+	    *result = start;
+	    return OK;
+	}
+
+    kom_errno = KOM_UNDEF_CONF;
+    return FAILURE;
+}
+
+
+extern  Success
+find_previous_conf_no(Conf_no start,
+		      Conf_no *result)
+{
+    Conf_no next_cno;
+    const Conf_no saved_start = start;
+    
+    CHK_CONNECTION(FAILURE);
+    CHK_LOGIN(FAILURE);
+    
+    if (start > (next_cno = query_next_conf_no()))
+	start = next_cno;
+
+    while (start-- > 0)
+	if (has_access(start, active_connection, read_protected))
+	{
+	    *result = start;
+	    return OK;
+	}
+
+    kom_errno = KOM_UNDEF_CONF;
+    err_stat = saved_start;
+    return FAILURE;
+}
+
