@@ -1,6 +1,6 @@
 /*
- * $Id: dbck-cache.c,v 0.50 2002/03/29 22:38:06 ceder Exp $
- * Copyright (C) 1991-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: dbck-cache.c,v 0.56 2003/08/23 16:38:17 ceder Exp $
+ * Copyright (C) 1991-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * This module contains some very simple simulations of the routines in
@@ -51,7 +51,7 @@
 #ifdef HAVE_STDLIB_H
 #  include <stdlib.h>
 #endif
-#include <time.h>
+#include "timewrap.h"
 #include <sys/types.h>
 #ifdef TIME_SYNC
 #  include <sys/resource.h>
@@ -75,6 +75,7 @@
 #include "dbck-cache.h"
 #include "local-to-global.h"
 #include "unused.h"
+#include "eintr.h"
 
 /*
  * All functions that can fail sets kom_errno to a suitable value
@@ -85,7 +86,7 @@ Person		** pers_arr;
 Conference 	** conf_arr;
 String		 * name_list; /* "cache" list */
 			        	  /* Global var auto init to NULL */
-int		  next_free_num = 1;
+static Conf_no     next_free_num = 1;
 
 Text_stat 	** text_arr;
 static Text_no next_text_num = 1;
@@ -466,6 +467,12 @@ cached_get_garb_nice (Conf_no conf_no)
     return conf_arr [ conf_no ]->nice;
 }
 
+extern Garb_nice
+cached_get_keep_commented(Conf_no conf_no)
+{
+    return conf_arr[conf_no]->keep_commented;
+}
+
 extern Local_text_no
 cached_get_highest_local_no (Conf_no conf_no)
 {
@@ -500,17 +507,17 @@ get_version(const char *fn)
     char	  c;
     long	  version;
     
-    if ( (fp = fopen(fn, "rb")) == NULL)
+    if ((fp = i_fopen(fn, "rb")) == NULL)
 	return -1;
 
     fseek(fp, 5, SEEK_SET);
     if ( (c = getc(fp)) == '\n')
     {
-	fclose(fp);
+	i_fclose(fp);
 	return 0;
     }
     version = fparse_long(fp);
-    fclose(fp);
+    i_fclose(fp);
 
     return version;
 }
@@ -521,7 +528,7 @@ is_clean(const char *fn)
 {
     FILE *fp;
 
-    if ( (fp = fopen(fn, "rb")) == NULL )
+    if ((fp = i_fopen(fn, "rb")) == NULL)
 	return FALSE;
 
     if ( getc(fp) == 'C' && 
@@ -530,12 +537,12 @@ is_clean(const char *fn)
          getc(fp) == 'A' &&
          getc(fp) == 'N' )
     {
-	fclose(fp);
+	i_fclose(fp);
 	return TRUE;
     }
     else
     {
-	fclose(fp);
+	i_fclose(fp);
 	return FALSE;
     }
 }
@@ -577,7 +584,8 @@ extern void			/* Write out everything. */
 cache_sync_all(void)
 {
     FILE *fp;
-    long i;
+    Conf_no ic;
+    Text_no it;
 
 #ifdef TIME_SYNC
     struct rusage start, after_confs, after_persons, after_text_stats,
@@ -595,20 +603,20 @@ cache_sync_all(void)
     {
 	if (is_clean(param.backupfile_name))
 	{
-	    if (rename(param.backupfile_name,
-		       param.backupfile_name_2) != 0)
+	    if (i_rename(param.backupfile_name,
+			 param.backupfile_name_2) != 0)
 	    {
 		kom_log("pre_sync: can't do extra backup.\n");
 	    }
 	}
 
-	if ( rename(param.datafile_name, param.backupfile_name) != 0 )
+	if (i_rename(param.datafile_name, param.backupfile_name) != 0)
 	    kom_log("WARNING: cache_sync_all: can't backup.\n");
     }
     else
 	kom_log("cache_sync_all: datafile not clean. No backup taken.\n");
     
-    if ( (fp=fopen(param.datafile_name, "w") ) == NULL )
+    if ((fp = i_fopen(param.datafile_name, "w")) == NULL)
     {
 	kom_log("WARNING: cache_sync_all: can't open file to save in.\n");
 	return;
@@ -637,26 +645,26 @@ cache_sync_all(void)
         restart_kom("Unknown output file format: %ld\n", oformat);
     }
     
-    for ( i = 1; i < next_free_num; i++ ) /* CONFS */
+    for (ic = 1; ic < next_free_num; ic++) /* CONFS */
     {
         switch (oformat)
         {
         case 0:
-            if ( conf_arr[ i ] == NULL)
+            if (conf_arr[ic] == NULL)
                 fprintf(fp, "@");
             else
             {
                 fprintf(fp, "+ ");
-                foutput_conference(fp, conf_arr[ i ]);
+                foutput_conference(fp, conf_arr[ic]);
             }
             putc('\n', fp);
             break;
         case 1:
         case 2:
-            if ( conf_arr[ i ] != NULL )
+            if (conf_arr[ic] != NULL)
             {
-                fprintf(fp, "C %lu", i);
-                foutput_conference(fp, conf_arr[ i ]);
+                fprintf(fp, "C %lu", (unsigned long)ic);
+                foutput_conference(fp, conf_arr[ic]);
                 putc('\n', fp);
             }
             break;
@@ -669,30 +677,30 @@ cache_sync_all(void)
     getrusage(0, &after_confs);
     time(&ac);
 #endif
-    for ( i = 1; i < next_free_num; i++ ) /* PERSONS */
+    for (ic = 1; ic < next_free_num; ic++) /* PERSONS */
     {
         switch (oformat)
         {
         case 0:
-            if ( pers_arr[ i ] == NULL)
+            if (pers_arr[ic] == NULL)
                 fprintf(fp, "@");
             else
             {
                 fprintf(fp, "+ ");
                 fprintf(fp, "%dH", PASSWD_LEN);
-                fwrite(pers_arr[i]->pwd, PASSWD_LEN, 1, fp);
-                foutput_person(fp, pers_arr[ i ]);
+                fwrite(pers_arr[ic]->pwd, PASSWD_LEN, 1, fp);
+                foutput_person(fp, pers_arr[ic]);
             }
             putc('\n', fp);
             break;
         case 1:
         case 2:
-            if ( pers_arr[ i ] != NULL )
+            if (pers_arr[ic] != NULL )
             {
-                fprintf(fp, "P %lu", i);
+                fprintf(fp, "P %lu", (unsigned long)ic);
                 fprintf(fp, " %dH", PASSWD_LEN);
-                fwrite(pers_arr[i]->pwd, PASSWD_LEN, 1, fp);
-                foutput_person(fp, pers_arr[ i ]);
+                fwrite(pers_arr[ic]->pwd, PASSWD_LEN, 1, fp);
+                foutput_person(fp, pers_arr[ic]);
                 putc('\n', fp);
             }
             break;
@@ -723,26 +731,26 @@ cache_sync_all(void)
     }
     
 
-    for ( i = 1; i < next_text_num; i++ ) /* TEXT_STATS */
+    for (it = 1; it < next_text_num; it++) /* TEXT_STATS */
     {
         switch (oformat)
         {
         case 0:
-            if ( text_arr[ i ] == NULL )
+            if (text_arr[it] == NULL)
                 fprintf(fp, "@");
             else
             {
                 fprintf(fp, "+ ");
-                foutput_text_stat(fp, text_arr[ i ]);
+                foutput_text_stat(fp, text_arr[it]);
             }
             putc('\n', fp);
             break;
         case 1:
         case 2:
-            if ( text_arr[ i ] != NULL )
+            if (text_arr[it] != NULL)
             {
-                fprintf(fp, "T %lu", i);
-                foutput_text_stat(fp, text_arr[ i ]);
+                fprintf(fp, "T %lu", it);
+                foutput_text_stat(fp, text_arr[it]);
                 putc('\n', fp);
             }
             break;
@@ -781,7 +789,7 @@ cache_sync_all(void)
 
     rewind(fp);
     sync_output_header(fp, "CLEAN", oformat);
-    fclose(fp);
+    i_fclose(fp);
 
 #ifdef TIME_SYNC
     kom_log("Sync ready.\n"
@@ -817,7 +825,7 @@ cache_sync_all(void)
 void
 cache_open_new_text_file(void)
 {
-    if ( ( new_text_file = fopen(param.textfile_name, "w")) == NULL )
+    if ((new_text_file = i_fopen(param.textfile_name, "w")) == NULL)
     {
 	kom_log("Can't open file to save new texts. Goodbye.\n");
 	exit(1);
@@ -828,7 +836,7 @@ extern Success
 init_cache(void)
 {
     FILE *fp = NULL;
-    long i;
+    unsigned long i;
     extern int vflag;		/* from dbck.c */
     extern Bool truncated_texts; /* from dbck.c */
     long num;
@@ -839,7 +847,7 @@ init_cache(void)
     extern int pers_pres_conf;
     extern int conf_pres_conf;
     extern int motd_conf;
-    extern int motd_of_lyskom;
+    extern Text_no motd_of_lyskom;
     extern int kom_news_conf;
        
     read_text_num = 0;
@@ -860,7 +868,7 @@ init_cache(void)
     
     new_text_file = NULL;
     
-    if ( (text_file = fopen(param.textfile_name, "rb")) == NULL )
+    if ((text_file = i_fopen(param.textfile_name, "rb")) == NULL)
     {
 	perror(param.textfile_name);
 	restart_kom("ERROR: init_cache: can't open text file %s.\n",
@@ -870,7 +878,7 @@ init_cache(void)
     if ( is_clean(param.datafile_name) )
     {
 	data_file_version = get_version(param.datafile_name);
-	if ( (fp = fopen(param.datafile_name, "rb")) == NULL )
+	if ((fp = i_fopen(param.datafile_name, "rb")) == NULL)
 	{
 	    kom_log("WARNING: init_cache: can't open datafile.\n");
 	    return FAILURE;
@@ -880,7 +888,7 @@ init_cache(void)
     else if ( is_clean(param.backupfile_name) )
     {
 	data_file_version = get_version(param.backupfile_name);
-	if ( (fp = fopen(param.backupfile_name, "rb")) == NULL )
+	if ((fp = i_fopen(param.backupfile_name, "rb")) == NULL)
 	{
 	    kom_log("WARNING: init_cache: can't open backupfile.\n");
 	    return FAILURE;
@@ -1025,7 +1033,7 @@ init_cache(void)
                                       kom_news_conf);
                     kom_info.kom_news_conf = kom_news_conf;
                 }
-                if (motd_of_lyskom != -1)
+                if (motd_of_lyskom != (Text_no)-1)
                 {
                     modifications += (kom_info.motd_of_lyskom !=
                                       motd_of_lyskom);
@@ -1299,7 +1307,7 @@ init_cache(void)
 
     if (read_text_num == 0 || read_conf_num == 0)
       restart_kom("init_cache(): Failed to read next_conf or text num.\n");
-    fclose(fp);
+    i_fclose(fp);
 
     return OK;
 }
