@@ -1,6 +1,6 @@
 /*
- * $Id: dbck.c,v 0.66 2002/03/29 22:29:48 ceder Exp $
- * Copyright (C) 1991-2002  Lysator Academic Computer Association.
+ * $Id: dbck.c,v 0.76 2003/08/23 16:38:17 ceder Exp $
+ * Copyright (C) 1991-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * dbck.c - A simple database checker and corrector.
@@ -40,7 +40,7 @@
 #ifdef HAVE_STDARG_H
 #  include <stdarg.h>
 #endif
-#include <time.h>
+#include "timewrap.h"
 #include <sys/types.h>
 #ifdef HAVE_CRYPT_H
 #  include <crypt.h>
@@ -53,6 +53,7 @@
 #endif
 #include <assert.h>
 #include <setjmp.h>
+#include <errno.h>
 
 #include "getopt.h"
 #include "misc-types.h"
@@ -71,12 +72,15 @@
 #include "async.h"
 #include "com.h"
 #include "connections.h"
+#include "kom-errno.h"
 #include "manipulate.h"
 #include "version-info.h"
 #include "ram-output.h"
 #include "unused.h"
 #include "local-to-global.h"
 #include "lockdb.h"
+#include "linkansi.h"
+#include "eintr.h"
 
 
 #define OPT_PERS_PRESENTATION_CONF      1
@@ -135,7 +139,7 @@ static long force_output=0;	/* Force sync (for conversions) */
 int pers_pres_conf = -1;
 int conf_pres_conf = -1;
 int motd_conf = -1;
-int motd_of_lyskom = -1;
+Text_no motd_of_lyskom = (Text_no)-1;
 int kom_news_conf = -1;
 
 
@@ -801,9 +805,12 @@ check_membership(Pers_no pno,
 {
     int errors=0;
     Conference *conf;
-    int i;
     Local_text_no last=0;
     Member *mem;
+    struct read_range *begin;
+    struct read_range *end;
+    struct read_range *ptr;
+    
     
     conf = cached_get_conf_stat(mship->conf_no);
     if ( conf == NULL )
@@ -815,30 +822,56 @@ check_membership(Pers_no pno,
     else
     {
 	/* Check read texts */
-	if (mship->last_text_read >= l2g_first_appendable_key(&conf->texts))
-	{
-	    kom_log("Person %lu %s %lu in conf %lu, which only has %lu texts.\n",
-		(unsigned long)pno,
-		"has read text",
-		(unsigned long)mship->last_text_read,
-		(unsigned long)mship->conf_no,
-		(unsigned long)(l2g_first_appendable_key(&conf->texts) -1));
-	    errors++;
-	}
 
-	for ( last = i = 0; i < mship->no_of_read; i++)
+	last = 0;
+	if (mship->no_of_read_ranges > 0)
 	{
-	    if ( mship->read_texts[i] <= last )
+	    begin = &mship->read_ranges[0];
+	    end = begin + mship->no_of_read_ranges;
+
+	    for (ptr = begin; ptr < end; ptr++)
 	    {
-		kom_log("Person %lu's membership in %lu %s %lu<%lu> <= %lu.\n",
-		    (unsigned long)pno, (unsigned long)mship->conf_no,
-		    "is corrupt: read text number",
-		    (unsigned long)mship->read_texts[i], (unsigned long)i,
-		    (unsigned long)last);
-		errors++;
+		if (ptr->first_read > ptr->last_read)
+		{
+		    kom_log("Person %lu's membership in %lu is corrupt: "
+			    "bad range: %lu-%lu.\n",
+			    (unsigned long)pno, (unsigned long)mship->conf_no,
+			    (unsigned long)ptr->first_read,
+			    (unsigned long)ptr->last_read);
+		    errors++;
+		}
+		if (ptr != begin && last + 1 == ptr->first_read)
+		{
+		    kom_log("Person %lu's membership in %lu is corrupt: "
+			    "adjoining ranges not joined at around %lu.\n",
+			    (unsigned long)pno, (unsigned long)mship->conf_no,
+			    (unsigned long)ptr->first_read);
+		    errors++;
+		}
+		if (ptr != begin && last >= ptr->first_read)
+		{
+		    kom_log("Person %lu's membership in %lu is corrupt: "
+			    "overlapping ranges: %lu-%lu, %lu-%lu.\n",
+			    (unsigned long)pno, (unsigned long)mship->conf_no,
+			    (unsigned long)(ptr-1)->first_read,
+			    (unsigned long)(ptr-1)->last_read,
+			    (unsigned long)ptr->first_read,
+			    (unsigned long)ptr->last_read);
+		    errors++;
+		}
+		last = ptr->last_read;
 	    }
-
-	    last = mship->read_texts[i];
+	}
+	
+	if (last >= l2g_first_appendable_key(&conf->texts))
+	{
+	    kom_log("Person %lu has read text %lu in conf %lu, "
+		    "which only has %lu texts.\n",
+		    (unsigned long)pno,
+		    (unsigned long)last,
+		    (unsigned long)mship->conf_no,
+		    (unsigned long)(l2g_first_appendable_key(&conf->texts)-1));
+	    errors++;
 	}
 
 	/* Check that he is a member */
@@ -858,7 +891,8 @@ check_membership(Pers_no pno,
             if (mem->type.invitation != mship->type.invitation ||
                 mem->type.passive != mship->type.passive ||
                 mem->type.secret != mship->type.secret ||
-                mem->type.reserved1 != mship->type.reserved1 ||
+                mem->type.passive_message_invert !=
+		mship->type.passive_message_invert ||
                 mem->type.reserved2 != mship->type.reserved2 ||
                 mem->type.reserved3 != mship->type.reserved3 ||
                 mem->type.reserved4 != mship->type.reserved4 ||
@@ -878,7 +912,8 @@ check_membership(Pers_no pno,
                        mship->type.invitation ? "invitation " : "",
                        mship->type.passive ? "passive " : "",
                        mship->type.secret ? "secret " : "",
-                       mship->type.reserved1 ? "rsv1 "  : "",
+                       mship->type.passive_message_invert ?
+		       "passive_message_invert "  : "",
                        mship->type.reserved2 ? "rsv2 "  : "",
                        mship->type.reserved3 ? "rsv3 "  : "",
                        mship->type.reserved4 ? "rsv4 "  : "",
@@ -890,7 +925,8 @@ check_membership(Pers_no pno,
                        mem->type.invitation ? "invitation " : "",
                        mem->type.passive ? "passive " : "",
                        mem->type.secret ? "secret " : "",
-                       mem->type.reserved1 ? "rsv1 "  : "",
+                       mem->type.passive_message_invert ?
+		       "passive_message_invert "  : "",
                        mem->type.reserved2 ? "rsv2 "  : "",
                        mem->type.reserved3 ? "rsv3 "  : "",
                        mem->type.reserved4 ? "rsv4 "  : "",
@@ -1315,7 +1351,10 @@ garb_text_file(void)
     String text;
 
     kom_log("Renaming %s to %s\n", param.textfile_name, param.textbackupfile_name);
-    rename(param.textfile_name, param.textbackupfile_name);
+    if (i_rename(param.textfile_name, param.textbackupfile_name) < 0)
+    {
+	restart_kom("rename failed: %s\n", strerror(errno));
+    }
     kom_log("Writing texts to (new) %s\n", param.textfile_name);
     fflush(stdout);
     fflush(stderr);
@@ -1417,12 +1456,13 @@ main (int    argc,
 {
     int hflag = 0;
     int errors;
-    char *default_config_file;
-    char *config_file;
+    const char *config_file;
     int optc;
     int need_rw = 0;
     int have_lock = 0;
 
+    link_ansi();
+
 #ifdef TRACED_ALLOCATIONS
     /* We must do this before we allocate any memory... */
     {
@@ -1471,7 +1511,7 @@ main (int    argc,
             break;
 
         case OPT_MOTD_OF_KOM:
-            motd_of_lyskom = atoi(optarg);
+            motd_of_lyskom = atol(optarg);
             if (motd_of_lyskom == 0 && *optarg != '0')
                 restart_kom("%s: bad text number %s\n", argv[0], optarg);
 	    need_rw = 1;
@@ -1561,25 +1601,19 @@ main (int    argc,
 	}
     }
 
-    /* Start reading in the configuration file. */
-
-    default_config_file = smalloc(strlen(DEFAULT_DBASE_DIR) +
-				  strlen(CONFIG_FILE) + 2);
-    sprintf(default_config_file, "%s/%s", DEFAULT_DBASE_DIR, CONFIG_FILE);
-
     /* Check if help was requested. */
     if (hflag > 0)
-	give_help(default_config_file);
+	give_help(get_default_config_file_name());
 
     /* Continue reading in the configuration file. */
 
     if (optind < argc)
 	config_file = argv[optind++];
     else
-	config_file = default_config_file;
+	config_file = get_default_config_file_name();
 
     read_configuration(config_file);
-    sfree(default_config_file);
+    free_default_config_file_name();
 
     /*
      *      Override configuration with command-line options
@@ -1605,7 +1639,7 @@ main (int    argc,
         kom_info.kom_news_conf = kom_news_conf;
 	modifications++;
     }
-    if (motd_of_lyskom != -1)
+    if (motd_of_lyskom != (Text_no)-1)
     {
 	kom_info.motd_of_lyskom = motd_of_lyskom;
 	modifications++;
