@@ -1,6 +1,6 @@
 /*
- * $Id: end-of-atomic.h,v 0.3 1991/09/15 10:32:18 linus Exp $
- * Copyright (C) 1991  Lysator Academic Computer Association.
+ * $Id: end-of-atomic.h,v 0.7 2003/08/23 16:38:17 ceder Exp $
+ * Copyright (C) 1991, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,17 +20,21 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
- * $Id: end-of-atomic.h,v 0.3 1991/09/15 10:32:18 linus Exp $
+ * $Id: end-of-atomic.h,v 0.7 2003/08/23 16:38:17 ceder Exp $
  *
  * end-of-atomic.c
  */
 
 /*
- * Return value X: end_of_atomic() should be called again within the
- * next X milliseconds.
+ * Clean up after an atomic request.  This will throw things from the
+ * cache, free memory allocated with tmp_alloc(), and save part of the
+ * database.  The function should be called after every atomic
+ * request, and it should also be called periodically; the caller
+ * should wait no more than the return value before the function is
+ * called again.
  */
-long
-end_of_atomic( Bool  idle );
+struct timeval
+end_of_atomic(void);
