@@ -1,6 +1,6 @@
 /*
- * $Id: internal-connections.h,v 0.4 1991/09/15 10:31:59 linus Exp $
- * Copyright (C) 1991  Lysator Academic Computer Association.
+ * $Id: internal-connections.h,v 0.11 2003/08/23 16:38:16 ceder Exp $
+ * Copyright (C) 1991, 2002-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,10 +20,10 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
- * $Id: internal-connections.h,v 0.4 1991/09/15 10:31:59 linus Exp $
+ * $Id: internal-connections.h,v 0.11 2003/08/23 16:38:16 ceder Exp $
  *
  * internal-connections.c
  *
@@ -37,6 +37,7 @@ extern void
 kill_client(Connection *cp);
 
 
+/* Return a Connection, or NULL if the specified session doesn't exist. */
 extern Connection *
 get_conn_by_number (Session_no session_no);
 
@@ -46,3 +47,13 @@ traverse_connections (Session_no session_no);
 
 extern void
 dump_allocated_connections(FILE *fp);
+
+enum ignored_conditions
+{
+    ignore_dns = 1,
+    ignore_ident = 2,
+};
+
+extern Bool
+handshake_ok(struct connection *cptr,
+	     enum ignored_conditions ignored);
