@@ -1,6 +1,6 @@
 /*
- * $Id: isc-interface.h,v 0.11 1999/05/24 09:34:26 ceder Exp $
- * Copyright (C) 1991-1995, 1998-1999  Lysator Academic Computer Association.
+ * $Id: isc-interface.h,v 0.14 2003/08/23 16:38:16 ceder Exp $
+ * Copyright (C) 1991-1995, 1998-1999, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,13 +20,15 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
- * $Id: isc-interface.h,v 0.11 1999/05/24 09:34:26 ceder Exp $
+ * $Id: isc-interface.h,v 0.14 2003/08/23 16:38:16 ceder Exp $
  *
  * Wrapper around <isc.h> to ensure that ISC_UDGTYPE is always correct.
  */
 
 #define ISC_UDGTYPE struct connection
 #include "isc.h"
+
+extern oop_source_sys *kom_server_oop_src;
