@@ -1,6 +1,6 @@
 /*
- * $Id: isc-parse.h,v 0.3 1991/09/15 10:31:48 linus Exp $
- * Copyright (C) 1991  Lysator Academic Computer Association.
+ * $Id: isc-parse.h,v 0.6 2003/08/23 16:38:16 ceder Exp $
+ * Copyright (C) 1991, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,15 +20,15 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
- * $Id: isc-parse.h,v 0.3 1991/09/15 10:31:48 linus Exp $
+ * $Id: isc-parse.h,v 0.6 2003/08/23 16:38:16 ceder Exp $
  *
  */
-#define ISC_PROTOCOL_ERR 1
-#define ISC_MSG_INCOMPLETE 2
-#define ISC_LOGOUT 3
+#define KOM_PROTOCOL_ERR 1
+#define KOM_MSG_INCOMPLETE 2
+#define KOM_LOGOUT 3
 
 int
 parse_char(Connection *client);
