@@ -1,7 +1,7 @@
 /*
  * File: local_to_global.c
  *
- * Copyright 1996-1999, 2001-2002 Inge Wallin, Per Cederqvist
+ * Copyright 1996-1999, 2001-2003 Inge Wallin, Per Cederqvist
  */
 
 
@@ -18,9 +18,8 @@
 #ifndef NULL
 #  include <stdio.h>
 #endif
-
 #include <sys/types.h>
-#include <time.h>
+#include "timewrap.h"
 
 #include "s-string.h"
 
@@ -446,6 +445,93 @@ find_block_index_key(const Local_to_global *l2g, Local_text_no lno,
     restart_kom("find_block_index_key found nothing\n");
 }
 
+/* Find the first local text number lower than LNO and return it.
+   Return 0 if LNO is lower than any local text number in the
+   structure.  The block where the returned value was found is stored
+   in BINFO_OUT, and the position within the block is stored in
+   INDEX_OUT.  BINFO_OUT and INDEX_OUT are only filled in if the
+   return value is non-zero.  */
+
+static Local_text_no
+find_block_index_key_reverse(const Local_to_global        *l2g,
+			     Local_text_no                 lno,
+			     const struct l2g_block_info **binfo_out,
+			     int                          *index_out)
+{
+    const struct l2g_block_info *binfo;
+    int i;
+
+    /* Adjust lno to the local number we are hunting for.  */
+    if (lno > 0)
+	--lno;
+
+    /* If the number is not to be found, return 0. */
+    binfo = find_block(l2g, lno);
+    if (binfo == NULL)
+	return 0;
+
+    /* Check if not present in the first block. */
+    if (lno < binfo->start)
+	return 0;
+
+    /* Check the found block and return the last entry found, if any. */
+    if (is_dense(binfo))
+    {
+	i = lno - binfo->start;
+	if (i >= binfo->first_free)
+	    i = binfo->first_free - 1;
+	for (; i >= 0; --i)
+	    if (binfo->value_block[i] != 0)
+	    {
+		*binfo_out = binfo;
+		*index_out = i;
+
+		return binfo->start + i;
+	    }
+    }
+    else
+    {
+	i = sparse_locate_value(binfo, lno);
+	if (i >= binfo->first_free)
+	    i = binfo->first_free - 1;
+	while (i > 0 && (binfo->value_block[i] == 0
+			 || binfo->key_block[i] > lno))
+	{
+	    ++sparse_skip_cost;
+	    --i;
+	}
+	if (binfo->value_block[i] != 0 && binfo->key_block[i] <= lno)
+	{
+	    *binfo_out = binfo;
+	    *index_out = i;
+
+	    return binfo->key_block[i];
+	}
+    }
+	
+    /*
+     * Try the previous block, if there is one.
+     */
+    if (binfo == l2g->blocks)
+	return 0;
+
+    binfo--;
+
+    /*
+     * We want the last existing entry in the block that binfo points
+     * to.
+     */
+    for (i = binfo->first_free - 1; i >= 0; --i)
+	if (binfo->value_block[i] != 0)
+	{
+	    *binfo_out = binfo;
+	    *index_out = i;
+	    return key_value(binfo, i);
+	}
+
+    restart_kom("find_block_index_key_reverse found nothing\n");
+}
+
 static void
 join_range(Local_to_global *l2g,
 	   struct l2g_block_info *first,
@@ -698,7 +784,7 @@ l2g_append(Local_to_global *l2g,
        point to it. */
     if (is_dense(binfo))
     {
-	if (lno - binfo->start >= L2G_BLOCKSIZE)
+	if (lno - binfo->start >= (Local_text_no)L2G_BLOCKSIZE)
 	{
 	    /* The last block is a dense block, and this text does not
 	       fit in it.  Or is there anything we can do to make it fit?  */
@@ -833,7 +919,8 @@ l2g_expensive_set(Local_to_global *l2g,
     else
     {
 	ix = sparse_locate_value(binfo, lno);
-	if (ix < binfo->first_free && binfo->key_block[ix] == lno)
+	if (ix < (Local_text_no_iter)binfo->first_free
+	    && binfo->key_block[ix] == lno)
 	{
 	    /* We found the value in this sparse block.  */
 	    assert(binfo->value_block[ix] != 0);
@@ -898,7 +985,8 @@ l2g_expensive_set(Local_to_global *l2g,
 	    }
 
 	    assert(ix == 0 || binfo->key_block[ix-1] < lno);
-	    assert(ix == binfo->first_free || binfo->key_block[ix+1] > lno);
+	    assert(ix == (Local_text_no_iter)binfo->first_free
+		   || binfo->key_block[ix+1] > lno);
 
 	    binfo->key_block[ix] = lno;
 	    binfo->value_block[ix] = tno;
@@ -1280,6 +1368,8 @@ l2gi_next(L2g_iterator *l2gi)
     const struct l2g_block_info *binfo;
     int                arrindex;
 
+    assert(!l2gi->search_ended);
+
     l2g      = l2gi->l2g;
     arrindex = l2gi->arrindex + 1;
     for (binfo = l2gi->binfo;
@@ -1327,6 +1417,89 @@ l2gi_end(const L2g_iterator *l2gi)
 }
 
 
+/* ================================================================ */
+/* ===                Reverse Iterator code                     === */
+/* ================================================================ */
+
+void
+l2gi_searchsome_reverse(L2g_reverse_iterator *l2gi,
+			const Local_to_global *l2g, 
+			Local_text_no begin,
+			Local_text_no end)
+{
+    const struct l2g_block_info  * binfo;
+    int               index;
+
+    l2gi->l2g = l2g;
+    l2gi->beginval = begin;
+    l2gi->endval   = end;
+    l2gi->search_ended = 1;
+
+    if (begin >= end)
+	return;
+
+    if (find_block_index_key_reverse(l2g, end, &binfo, &index) == 0)
+	return;
+
+    l2gi->binfo    = binfo;
+    l2gi->arrindex = index;
+
+    assert(binfo != NULL);
+    
+    l2gi->lno = key_value(binfo, index);
+    l2gi->tno = binfo->value_block[index];
+    l2gi->search_ended = l2gi->lno < begin;
+}
+
+
+void
+l2gi_prev(L2g_reverse_iterator *l2gi)
+{
+    const Local_to_global *l2g;
+    const struct l2g_block_info *binfo;
+    int                arrindex;
+
+    assert(!l2gi->search_ended);
+
+    l2g      = l2gi->l2g;
+    arrindex = l2gi->arrindex - 1;
+    binfo = l2gi->binfo;
+    while (1)
+    {
+	for (; arrindex >= 0; --arrindex)
+	{
+	    if (key_value(binfo, arrindex) < l2gi->beginval)
+	    {
+		l2gi->search_ended = 1;
+		return;
+	    }
+
+	    if (binfo->value_block[arrindex] != 0)
+	    {
+		l2gi->binfo    = binfo;
+		l2gi->arrindex = arrindex;
+
+		l2gi->lno = key_value(binfo, arrindex);
+		l2gi->tno = binfo->value_block[arrindex];
+
+		return;
+	    }
+	}
+
+	if (binfo == l2g->blocks)
+	{
+	    l2gi->search_ended = 1;
+	    return;
+	}
+	
+	--binfo;
+	arrindex = binfo->first_free - 1;
+    }
+}
+
+
+
+
 /* ---------- */
 
 extern void
