@@ -1,6 +1,6 @@
 /*
- * $Id: person.c,v 0.65 2001/12/28 20:00:02 ceder Exp $
- * Copyright (C) 1991-1999, 2001  Lysator Academic Computer Association.
+ * $Id: person.c,v 0.86 2003/08/23 16:38:15 ceder Exp $
+ * Copyright (C) 1991-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * person.c
@@ -42,7 +42,7 @@
 #ifdef HAVE_STRING_H
 #  include <string.h>
 #endif
-#include <time.h>
+#include "timewrap.h"
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>		/* for crypt() on Linux */
 #endif
@@ -59,11 +59,11 @@
 #include "com.h"
 #include "async.h"
 #include "connections.h"
+#include "kom-errno.h"
 #include "manipulate.h"
 #include "string-malloc.h"
 #include "debug.h"
 #include "cache.h"
-#include "kom-errno.h"
 #include "server/smalloc.h"
 #include "kom-config.h"
 #include "log.h"
@@ -74,6 +74,8 @@
 #include "local-to-global.h"
 #include "kom-memory.h"
 #include "server-time.h"
+#include "send-async.h"
+#include "stats.h"
 
 BUGDECL;
 
@@ -285,59 +287,76 @@ do_unmark_text(Pers_no    pers_no,
 static Success
 do_set_user_area(Pers_no     pers_no,
 		 Person	   * pers_p, /* May be NULL */
-		 Text_no     user_area)
+		 Text_no     new_area)
 {
-    Text_stat * old_user_area;
-    Text_stat * new_user_area = NULL; /* To stop gcc complaining. */
+    Text_no old_area;
+    Text_stat *old_stat;
+    Text_stat *new_stat = NULL; /* To stop gcc complaining. */
 
     /* Check that the new user_area exists before deleting the old*/
 
-    if ( user_area != 0 )
+    if (new_area != 0)
     {
-	GET_T_STAT(new_user_area, user_area, FAILURE);
-	if ( new_user_area->no_of_marks >= param.max_marks_text )
+	GET_T_STAT(new_stat, new_area, FAILURE);
+
+        if (!text_read_access(active_connection, new_area, new_stat)
+            && !ENA(admin, 2)) /* OK -- In an RPC call */
+        {
+            kom_errno = KOM_NO_SUCH_TEXT;
+            err_stat = new_area;
+            return FAILURE;
+        }
+
+	if (new_stat->no_of_marks >= param.max_marks_text)
 	{
-	    kom_log("%s(%d, %lu): New user_area's mark count (%d) > %d.\n",
-		"LIMIT: set_user_area",
-		pers_no, (unsigned long)user_area, new_user_area->no_of_marks,
-		param.max_marks_text);
-            err_stat = user_area;
+	    kom_log("LIMIT: set_user_area(%d, %lu): "
+		    "New user_area's mark count (%d) > %d.\n",
+		    pers_no,
+		    (unsigned long)new_area,
+		    new_stat->no_of_marks,
+		    param.max_marks_text);
+
+            err_stat = new_area;
 	    kom_errno = KOM_MARK_LIMIT;
 	    return FAILURE;
 	}
     }
 
-    if ( pers_p == NULL )
+    if (pers_p == NULL)
 	GET_P_STAT(pers_p, pers_no, FAILURE);
     
     /* Unmark the previous user_area if it exists. */
 
-    if ( pers_p->user_area != 0
-	&& (old_user_area = cached_get_text_stat(pers_p->user_area)) != NULL)
+    old_area = pers_p->user_area;
+    if (old_area != 0 && (old_stat = cached_get_text_stat(old_area)) != NULL)
     {
-	if ( old_user_area->no_of_marks > 0 )
+	if (old_stat->no_of_marks > 0)
 	{
-	    --old_user_area->no_of_marks;
-	    mark_text_as_changed( pers_p->user_area );
+	    --old_stat->no_of_marks;
+	    mark_text_as_changed(old_area);
 	}
 	else
 	{
-	    kom_log("ERROR: set_user_area(%d, %lu): Old user_area %lu unmarked\n",
-		pers_no, (unsigned long)user_area,
-		(unsigned long)pers_p->user_area);
+	    kom_log("ERROR: set_user_area(%d, %lu): "
+		    "Old user_area %lu unmarked.\n",
+		    pers_no, (unsigned long)new_area,
+		    (unsigned long)old_area);
 	}
     }
 
     /* Mark the new user_area */
 
-    if ( user_area != 0 )
+    if (new_area != 0)
     {
-	++new_user_area->no_of_marks;
-	mark_text_as_changed( user_area );
+	++new_stat->no_of_marks;
+	mark_text_as_changed(new_area);
     }
     
-    pers_p->user_area = user_area;
-    mark_person_as_changed (pers_no);
+    pers_p->user_area = new_area;
+    mark_person_as_changed(pers_no);
+
+    if (old_area != new_area)
+	async_new_user_area(pers_no, old_area, new_area);
 
     return OK;
 }
@@ -393,6 +412,7 @@ do_delete_pers (Pers_no pers_no)
     l2g_clear(&pers_p->created_texts);
 
     cached_delete_person(pers_no);
+    update_stat(STAT_PERSONS, -1);
     
     /* ASYNC */
     return OK;    
@@ -589,7 +609,7 @@ create_person_generic ( const String  name,
     }
     
     mailbox->creator		= ACTPERS ? ACTPERS : new_user;
-    mailbox->creation_time 	= current_time;
+    mailbox->creation_time 	= current_time.tv_sec;
     mailbox->presentation	= 0;		/* No presentation yet */
     mailbox->supervisor		= ACTPERS ? ACTPERS : new_user;
     mailbox->permitted_submitters = 0;
@@ -644,7 +664,7 @@ create_person_generic ( const String  name,
     mship_type.invitation = 0;
     mship_type.passive = 0;
     mship_type.secret = 0;
-    mship_type.reserved1 = 0;
+    mship_type.passive_message_invert = 0;
     mship_type.reserved2 = 0;
     mship_type.reserved3 = 0;
     mship_type.reserved4 = 0;
@@ -659,7 +679,7 @@ create_person_generic ( const String  name,
     if (conf_stat_check_add_aux_item_list(mailbox,
                                           new_user,
                                           conf_aux,
-                                          new_user,
+                                          active_connection,
                                           TRUE) != OK)
     {
 	/* FIXME (bug 160): Conf_no leak: We create a person, check the
@@ -691,6 +711,8 @@ create_person_generic ( const String  name,
 	login (new_user, passwd, FALSE);
     }
 
+    update_stat(STAT_CONFS, 1);
+    update_stat(STAT_PERSONS, 1);
     return new_user;
 }
 
@@ -718,7 +740,7 @@ create_person(const String name,
     pers = create_person_generic(name, passwd, conf_aux, flags, FALSE);
     if (pers != 0)
     {
-        /* FIXME (bug 93): Send asynch message */
+        /* FIXME (bug 909): Send asynch message */
     }
 
     return pers;
@@ -908,6 +930,54 @@ map_created_texts(Pers_no       pers_no,
 }
 
 
+extern Success
+map_created_texts_reverse(Pers_no               pers_no,
+			  Local_text_no	        local_no_ceiling,
+			  unsigned long         no_of_texts,
+			  Text_mapping_reverse *result)
+{
+    Person       *pers_p;
+    enum access   acc;
+    Local_text_no ceiling;
+
+    CHK_CONNECTION(FAILURE);
+
+    if (no_of_texts > 255)
+    {
+	err_stat = 255;
+	kom_errno = KOM_LONG_ARRAY;
+	return FAILURE;
+    }
+
+    GET_P_STAT(pers_p, pers_no, FAILURE);
+
+    CHK_CONNECTION(FAILURE);
+    CHK_LOGIN(FAILURE);
+    acc = access_perm(pers_no, active_connection, read_protected);
+
+    if (acc == error)
+	return FAILURE;
+
+    if (acc <= none)
+    {
+        err_stat = pers_no;
+	kom_errno = KOM_UNDEF_PERS;
+	return FAILURE;
+    }
+
+    ceiling = l2g_first_appendable_key(&pers_p->created_texts);
+    if (local_no_ceiling == 0 || local_no_ceiling > ceiling)
+	result->ceiling = ceiling;
+    else
+	result->ceiling = local_no_ceiling;
+
+    result->no_of_texts = no_of_texts;
+    result->l2g = &pers_p->created_texts;
+
+    return OK;
+}
+
+
 
 /*
  * Set privilege bits of a person. You must have the wheel bit set
@@ -956,10 +1026,8 @@ set_passwd (Pers_no	    person,
 
     GET_P_STAT(p, person, FAILURE);
 
-    if ( person != ACTPERS &&
-         !ENA(wheel, 7)  &&     /* OK -- In an RPC call */
-         !ENA(wheel, 8)  &&     /* OK -- In an RPC call */
-         !is_supervisor (person, ACTPERS, ACT_P) )
+    if (!is_supervisor(person, active_connection)
+	&& !ENA(wheel, 7))	/* OK -- In an RPC call */
     {
         err_stat = person;
 	kom_errno = KOM_PERM;
@@ -987,13 +1055,10 @@ set_passwd (Pers_no	    person,
 
 /*
  * Ask which texts a person has read in a certain conference.
- * Can be done before login. Will return EMPTY_MEMBERSHIP if VICTIM
+ * Can be done before login. Will return an empty membership if VICTIM
  * has his unread_is_secret-flag set. This can not be distinguished
  * from the case when VICTIM has not read any texts in CONF_NO. This
- * is a feature, not a bug. (FIXME (bug 161): At least I think so. /ceder)
- *
- * FIXME (bug 161): Well, I've changed my mind. This call is full of
- * security holes.   /ceder
+ * is a feature, not a bug.
  *
  * Will fail if VICTIM is not a member of CONF_NO.
  */
@@ -1001,57 +1066,73 @@ set_passwd (Pers_no	    person,
 static Success
 do_query_read_texts(Pers_no	  victim,
                     Conf_no	  conf_no,
-                    Membership * result)
+		    Bool          want_read_ranges,
+		    unsigned long max_ranges,
+                    Membership   *result)
 {
     Person	* victim_p;
     Membership  * membp;
-    enum access	  victim_acc;
+    enum memb_visibility vis;
 
     CHK_CONNECTION(FAILURE);
     GET_P_STAT( victim_p, victim, FAILURE);
 
-    victim_acc = access_perm(victim, active_connection, unlimited);
-    if ( victim_acc <= none )
+    if (!has_access(victim, active_connection, read_protected))
     {
         err_stat = victim;
 	kom_errno = KOM_UNDEF_PERS;
 	return FAILURE;
     }
 
-    if (access_perm(conf_no, active_connection, read_protected) <= none)
-    {
-        err_stat = conf_no;
-	kom_errno = KOM_UNDEF_CONF;
-	return FAILURE;
-    }
-    
     membp = locate_membership (conf_no, victim_p);
-
     if (membp == NULL)
     {
-        err_stat = conf_no;
-	kom_errno = KOM_NOT_MEMBER;
+	set_conf_errno(active_connection, conf_no, KOM_NOT_MEMBER);
 	return FAILURE;
     }
 
-    if (membp->type.secret &&
-        !is_supervisor(victim, ACTPERS, ACT_P) &&
-        !ENA(admin, 4))         /* OK -- Guarded */
+    vis = membership_visible(active_connection, victim, conf_no, membp->type,
+			     FALSE, FALSE);
+    if (vis == mv_none)
     {
-        err_stat = conf_no;
-        kom_errno = KOM_NOT_MEMBER;
+	set_conf_errno(active_connection, conf_no, KOM_NOT_MEMBER);
         return FAILURE;
     }
 
+    if (want_read_ranges == TRUE)
+    {
+	const Conference *conf_c;
+
+	if ((conf_c = cached_get_conf_stat(conf_no)) == NULL)
+	{
+	    kom_log("do_query_read_texts: %lu is a member of %lu, but "
+		    "that conference doesn't exist\n",
+		    (unsigned long)victim,
+		    (unsigned long)conf_no);
+	    return FAILURE;
+	}
+
+	if (adjust_read(membp, conf_c) == TRUE)
+	    mark_person_as_changed(victim);
+    }
+
     *result = *membp;
 
-    if ( victim_p->flags.unread_is_secret &&
-         victim_acc != unlimited)
+    if (vis == mv_censor_unread)
     {
 	result->last_time_read = NO_TIME;
-	result->last_text_read = 0;
-	result->no_of_read = 0;
-	result->read_texts = NULL;
+	result->no_of_read_ranges = 0;
+	result->read_ranges = NULL;
+    }
+
+    if (want_read_ranges)
+    {
+	if (max_ranges != 0 && result->no_of_read_ranges > max_ranges)
+	    result->no_of_read_ranges = max_ranges;
+    }
+    else
+    {
+	result->read_ranges = NULL;
     }
 
     return OK;
@@ -1059,13 +1140,32 @@ do_query_read_texts(Pers_no	  victim,
 
 
 extern Success
-query_read_texts(Pers_no	  victim,
-                 Conf_no	  conf_no,
-                 Membership     * result)	/* Points to area to store result in */
+query_read_texts(Pers_no        pers_no,
+		 Conf_no        conf_no,
+		 Bool           want_read_ranges,
+		 unsigned long  max_ranges,
+		 Membership    *result)
+{
+    CHK_BOOL(want_read_ranges, FAILURE);
+
+    /* CHK_CONNECTION in do_query_read_texts */
+    return do_query_read_texts(pers_no,
+                               conf_no,
+			       want_read_ranges,
+			       max_ranges,
+                               result);
+}
+
+extern Success
+query_read_texts_10(Pers_no	 victim,
+		    Conf_no	 conf_no,
+		    Membership  *result)
 {
     /* CHK_CONNECTION in do_query_read_texts */
     return do_query_read_texts(victim,
                                conf_no,
+			       TRUE,
+			       0,
                                result);
 }
 
@@ -1076,7 +1176,7 @@ query_read_texts_old(Pers_no victim,
                      Membership *result)
 {
     /* CHK_CONNECTION in do_query_read_texts */
-    return do_query_read_texts(victim, conf_no, result);
+    return do_query_read_texts(victim, conf_no, TRUE, 0, result);
 }
 
 
