@@ -1,6 +1,6 @@
 /*
- * $Id: prot-a-output.c,v 0.54 2002/03/29 22:29:45 ceder Exp $
- * Copyright (C) 1991-2002  Lysator Academic Computer Association.
+ * $Id: prot-a-output.c,v 0.71 2003/08/23 16:38:15 ceder Exp $
+ * Copyright (C) 1991-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * prot-a-output.c  -  write objects through an ISC connection.
@@ -34,7 +34,7 @@
 #  include <config.h>
 #endif
 
-#include <time.h>
+#include "timewrap.h"
 #include <setjmp.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -43,6 +43,8 @@
 #endif
 #include <assert.h>
 
+#include "oop.h"
+
 #include "s-string.h"
 #include "kom-types.h"
 #include "isc-interface.h"
@@ -62,6 +64,21 @@ prot_a_output_ul(Connection *fp,
     isc_putul(ul, fp->isc_session);
 }
 
+static void
+prot_a_output_float(Connection *fp,
+		    float v)
+{
+    char buf[80];
+
+#ifdef HAVE_SNPRINTF
+    snprintf(buf, sizeof(buf), " %g", v);
+#else
+    sprintf(buf, " %g", v);
+#endif
+
+    isc_puts(buf, fp->isc_session);
+}
+
 void
 prot_a_output_person (Connection *fp,
                       Person *person)
@@ -103,15 +120,46 @@ void prot_a_output_membership_type(Connection *fp,
     isc_putc(type.invitation + '0', fp->isc_session);
     isc_putc(type.passive    + '0', fp->isc_session);
     isc_putc(type.secret     + '0', fp->isc_session);
-    isc_putc(type.reserved1  + '0', fp->isc_session);
+    isc_putc(type.passive_message_invert + '0', fp->isc_session);
     isc_putc(type.reserved2  + '0', fp->isc_session);
     isc_putc(type.reserved3  + '0', fp->isc_session);
     isc_putc(type.reserved4  + '0', fp->isc_session);
     isc_putc(type.reserved5  + '0', fp->isc_session);
 }
 
-void prot_a_output_membership(Connection *fp,
-                              const Membership *mship)
+void
+prot_a_output_membership(Connection *fp,
+			 const Membership *mship)
+{
+    unsigned int i;
+
+    prot_a_output_ul(fp, mship->position);
+    prot_a_output_time(fp, mship->last_time_read);
+    prot_a_output_ul(fp, mship->conf_no);
+    prot_a_output_ul(fp, mship->priority);
+
+    prot_a_output_ul(fp, mship->no_of_read_ranges);
+    if (mship->read_ranges != NULL && mship->no_of_read_ranges > 0)
+    {
+	isc_puts(" {", fp->isc_session);
+	for (i = 0; i < mship->no_of_read_ranges; i++)
+	{
+	    prot_a_output_ul(fp, mship->read_ranges[i].first_read);
+	    prot_a_output_ul(fp, mship->read_ranges[i].last_read);
+	}
+	isc_puts(" }", fp->isc_session);
+    }
+    else
+	isc_puts(" *", fp->isc_session);
+
+    prot_a_output_ul(fp, mship->added_by);
+    prot_a_output_time(fp, mship->added_at);
+    prot_a_output_membership_type(fp, mship->type);
+}
+
+
+void prot_a_output_membership_10(Connection *fp,
+				 const Membership *mship)
 {
     prot_a_output_ul(fp, mship->position);
     prot_a_output_membership_old(fp, mship);
@@ -120,32 +168,92 @@ void prot_a_output_membership(Connection *fp,
     prot_a_output_membership_type(fp, mship->type);
 }
 
+static void
+prot_a_output_read_texts(Connection *fp,
+			 const Membership *mship)
+{
+    if (mship->read_ranges == NULL)
+	isc_puts(" 0 0 *", fp->isc_session);
+    else
+    {
+	struct read_range *begin;
+	struct read_range *end;
+
+	assert(mship->read_ranges != NULL);
+	assert(mship->no_of_read_ranges > 0);
+
+	begin = &mship->read_ranges[0];
+	end = begin + mship->no_of_read_ranges;
+
+	if (begin->first_read == 1)
+	{
+	    prot_a_output_ul(fp, begin->last_read);
+	    begin++;
+	}
+	else
+	    isc_puts(" 0", fp->isc_session);
+
+	if (begin == end)
+	    isc_puts(" 0 *", fp->isc_session);
+	else
+	{
+	    unsigned long no_of_read = 0;
+	    const struct read_range *ptr;
+
+	    for (ptr = begin; ptr < end; ++ptr)
+		no_of_read += ptr->last_read - ptr->first_read + 1;
+
+	    prot_a_output_ul(fp, no_of_read);
+	    if (mship->skip_read_texts)
+		isc_puts(" *", fp->isc_session);
+	    else
+	    {
+		isc_puts(" {", fp->isc_session);
+
+		for (ptr = begin; ptr < end; ++ptr)
+		{
+		    Local_text_no lno;
+
+		    for (lno = ptr->first_read; lno <= ptr->last_read; lno++)
+			prot_a_output_ul(fp, lno);
+		}
+	    
+		isc_puts(" }", fp->isc_session);
+	    }
+	}
+    }
+}
+
 void
 prot_a_output_membership_old(Connection *fp,
 			     const Membership *mship)
 {
-    int i;
-    
     prot_a_output_time(fp, mship->last_time_read );
-    
     prot_a_output_ul(fp, mship->conf_no);
     prot_a_output_ul(fp, mship->priority);
-    prot_a_output_ul(fp, mship->last_text_read);
-    prot_a_output_ul(fp, mship->no_of_read);
+    prot_a_output_read_texts(fp, mship);
+}
+
+	
+void
+prot_a_output_membership_list(Connection      *fp,
+			      Membership_list  mlist)
+{
+    int i;
     
-    if ( mship->read_texts != NULL && mship->no_of_read > 0)
+    prot_a_output_ul(fp, mlist.no_of_confs);
+
+    if ( mlist.confs != NULL && mlist.no_of_confs > 0 )
     {
 	isc_puts(" {", fp->isc_session);
-	for ( i = 0; i < mship->no_of_read; i++)
-	    prot_a_output_ul(fp, mship->read_texts[i]);
-	
+	for ( i = 0; i < mlist.no_of_confs; i++)
+	    prot_a_output_membership(fp, mlist.confs + i);
 	isc_puts(" }", fp->isc_session);
     }
     else
 	isc_puts(" *", fp->isc_session);
 }
 
-	
 void
 prot_a_output_membership_list_old (Connection	* fp,
                                    Membership_list	  mlist)
@@ -166,8 +274,8 @@ prot_a_output_membership_list_old (Connection	* fp,
 }
 
 void
-prot_a_output_membership_list (Connection	* fp,
-                               Membership_list	  mlist)
+prot_a_output_membership_list_10(Connection	 *fp,
+				 Membership_list  mlist)
 {
     int i;
     
@@ -177,7 +285,7 @@ prot_a_output_membership_list (Connection	* fp,
     {
 	isc_puts(" {", fp->isc_session);
 	for ( i = 0; i < mlist.no_of_confs; i++)
-	    prot_a_output_membership(fp, mlist.confs + i);
+	    prot_a_output_membership_10(fp, mlist.confs + i);
 	isc_puts(" }", fp->isc_session);
     }
     else
@@ -747,7 +855,10 @@ prot_a_output_time(Connection *fp,
 {
     struct tm *t;
 
-    t = localtime( &clk );
+    if (fp->use_utc)
+	t = gmtime(&clk);
+    else
+	t = localtime(&clk);
 
     prot_a_output_ul(fp, t->tm_sec);
     prot_a_output_ul(fp, t->tm_min);
@@ -775,6 +886,13 @@ prot_a_output_text_no(Connection *fp,
     prot_a_output_ul(fp, text);
 }
 
+void
+prot_a_output_conf_no(Connection *fp,
+		      Conf_no conf)
+{
+    prot_a_output_ul(fp, conf);
+}
+
 static void
 prot_a_output_conf_z_info (Connection  *fp,
 			   Conf_z_info *conf_c)
@@ -958,7 +1076,7 @@ prot_a_output_text_mapping(Connection *fp,
     else
 	prot_a_output_ul(fp, limit);
 
-    /* Emit the "later-texts-exists" flag. */
+    /* Emit the "more-texts-exists" flag. */
 
     isc_puts(iter.search_ended ? " 0" : " 1", fp->isc_session);
 
@@ -1010,6 +1128,170 @@ prot_a_output_text_mapping(Connection *fp,
     }
 }
 
+void
+prot_a_output_text_mapping_reverse(Connection *fp,
+				   Text_mapping_reverse *map)
+{
+    Local_text_no lowest;
+    Local_text_no zeroes = 0;
+    Local_text_no nonzeroes = 0;
+    L2g_iterator iter;
+    L2g_reverse_iterator riter;
+
+    /* Initialize iterators to something. Otherwise some tools will flag
+       a read uninitialized memory in the for initializer. */
+
+    iter.l2g = NULL;
+    iter.binfo = NULL;
+    iter.arrindex = 0;
+    iter.beginval = 0;
+    iter.endval = 0;
+    iter.search_ended = 0;
+    iter.lno = 0;
+    iter.tno = 0;
+
+    riter.l2g = NULL;
+    riter.binfo = NULL;
+    riter.arrindex = 0;
+    riter.beginval = 0;
+    riter.endval = 0;
+    riter.search_ended = 0;
+    riter.lno = 0;
+    riter.tno = 0;
+    
+    /* Count the number of internal zeroes to determine if we should
+       use a dense or sparse representation. */
+
+    for (l2gi_searchsome_reverse(&riter, map->l2g, 0, map->ceiling),
+	     lowest = riter.search_ended ? map->ceiling : riter.lno + 1;
+	 !riter.search_ended && nonzeroes < map->no_of_texts;
+	 l2gi_prev(&riter))
+    {
+	zeroes += (lowest - riter.lno) - 1;
+	nonzeroes++;
+	lowest = riter.lno;
+    }
+
+    prot_a_output_ul(fp, lowest);
+    prot_a_output_ul(fp, map->ceiling);
+
+    /* Emit the "more-texts-exists" flag. */
+    isc_puts(riter.search_ended ? " 0" : " 1", fp->isc_session);
+
+    /* Emit the block, either a sparse or a dense one. */
+    if (nonzeroes == 0)
+    {
+	/* Special case: an empty array.  Use the sparse format so
+	   that we don't have to philosophize about what the lower
+	   limit in the text-list should be. */
+	isc_puts(" 0 0 *", fp->isc_session);
+    }
+    else if (zeroes >= nonzeroes)
+    {
+	/* There are at least as many zeroes as real mappings, so use
+	   a sparse representation. */
+
+	isc_puts(" 0", fp->isc_session);
+	prot_a_output_ul(fp, nonzeroes);
+	isc_puts(" {", fp->isc_session);
+
+	for (l2gi_searchsome(&iter, map->l2g, lowest, map->ceiling);
+	     !iter.search_ended;
+	     l2gi_next(&iter))
+	{
+	    prot_a_output_ul(fp, iter.lno);
+	    prot_a_output_ul(fp, iter.tno);
+	}
+	isc_puts(" }", fp->isc_session);
+    }
+    else
+    {
+	/* Emit a dense block. */
+	isc_puts(" 1", fp->isc_session);
+	prot_a_output_ul(fp, lowest);
+	prot_a_output_ul(fp, zeroes + nonzeroes);
+	isc_puts(" {", fp->isc_session);
+	for (l2gi_searchsome(&iter, map->l2g, lowest, map->ceiling);
+	     !iter.search_ended;
+	     l2gi_next(&iter))
+	{
+	    while (lowest++ < iter.lno)
+		isc_puts(" 0", fp->isc_session);
+	    prot_a_output_ul(fp, iter.tno);
+	}
+	isc_puts(" }", fp->isc_session);
+    }
+}
+
+void
+prot_a_output_stats_description(Connection *fp,
+				Stats_description *result)
+{
+    int i;
+
+    prot_a_output_ul(fp, result->no_of_stats);
+    if (result->no_of_stats == 0)
+        isc_puts(" *", fp->isc_session);
+    else
+    {
+        isc_puts(" {", fp->isc_session);
+        for (i = 0; i < result->no_of_stats; i++)
+            prot_a_output_string(fp, result->stat_names[i]);
+        isc_puts(" }", fp->isc_session);
+    }
+
+    prot_a_output_num_list(fp, &result->intervals);
+}
+
+static void
+prot_a_output_stats(Connection *fp,
+		    const Stats *result)
+{
+    prot_a_output_float(fp, result->average);
+    prot_a_output_float(fp, result->ascent_rate);
+    prot_a_output_float(fp, result->descent_rate);
+}
+
+void
+prot_a_output_stats_list(Connection *fp,
+			 const Stats_list *result)
+{
+    int i;
+
+    prot_a_output_ul(fp, result->no_of_stats);
+    if (result->no_of_stats == 0)
+        isc_puts(" *", fp->isc_session);
+    else
+    {
+        isc_puts(" {", fp->isc_session);
+        for (i = 0; i < result->no_of_stats; i++)
+            prot_a_output_stats(fp, &result->stats[i]);
+        isc_puts(" }", fp->isc_session);
+    }
+}
+
+void
+prot_a_output_static_server_info(Connection *fp,
+				 const Static_server_info *result)
+{
+    prot_a_output_time(fp, result->boot_time);
+    prot_a_output_time(fp, result->save_time);
+    prot_a_output_string(fp, result->db_status);
+    prot_a_output_ul(fp, result->existing_texts);
+    prot_a_output_ul(fp, result->highest_text_no);
+    prot_a_output_ul(fp, result->existing_confs);
+    prot_a_output_ul(fp, result->existing_persons);
+    prot_a_output_ul(fp, result->highest_conf_no);
+}
+
+void
+prot_a_output_scheduling_info(Connection *fp,
+			      const Scheduling_info *result)
+{
+    prot_a_output_ul(fp, result->priority);
+    prot_a_output_ul(fp, result->weight);
+}
+
 
 #ifdef DEBUG_CALLS
 void
