@@ -1,6 +1,6 @@
 /*
- * $Id: prot-a-parse.c,v 0.52 2002/03/29 22:29:45 ceder Exp $
- * Copyright (C) 1991-2002  Lysator Academic Computer Association.
+ * $Id: prot-a-parse.c,v 0.63 2003/08/23 16:38:15 ceder Exp $
+ * Copyright (C) 1991-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * prot-a-parse.c - parse protocol-A messages.
@@ -36,12 +36,15 @@
 #include <assert.h>
 #include <stdio.h>
 #include <setjmp.h>
-#include <time.h>
+#include "timewrap.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #ifdef HAVE_STDLIB_H
 #  include <stdlib.h>
 #endif
+#include <string.h>
+
+#include "oop.h"
 
 #include "debug.h"
 #include "s-string.h"
@@ -90,10 +93,10 @@ prot_a_get_token(Connection *client)
 	{
 	    isc_puts("%%Insane token length.\n", client->isc_session);
 	    isc_flush(client->isc_session);
-	    longjmp(parse_env, ISC_LOGOUT);
+	    longjmp(parse_env, KOM_LOGOUT);
 	}
 	client->first_to_parse = old_first;
-	longjmp(parse_env, ISC_MSG_INCOMPLETE);
+	longjmp(parse_env, KOM_MSG_INCOMPLETE);
     }
 
     return result;
@@ -108,9 +111,9 @@ prot_a_parse_long(Connection *client)
     long	res;
 
     token = prot_a_get_token(client);
-    res = s_strtol(token, &end, PROTOCOL_NUMBER_BASE);
+    res = s_strtol(token, &end);
     if (end != s_strlen(token))
-	longjmp(parse_env, ISC_PROTOCOL_ERR);
+	longjmp(parse_env, KOM_PROTOCOL_ERR);
     
     return res;
 }
@@ -143,16 +146,16 @@ prot_a_parse_num_list(Connection *client,
 	{
 	    isc_puts("%%Insane array size.\n", client->isc_session);
 	    isc_flush(client->isc_session);
-	    longjmp(parse_env, ISC_LOGOUT);
+	    longjmp(parse_env, KOM_LOGOUT);
 	}
 
-	res->length = min(maxlen+1, client->array_parse_parsed_length);
 	client->array_parse_pos = 1;
 	/* Fall through */
     case 1:			/* The opening curly brace. */
 	if ( parse_nonwhite_char(client) != '{' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
 
+	res->length = min(maxlen+1, client->array_parse_parsed_length);
         res->data   = smalloc(sizeof(*res->data) * res->length);
 
 	client->array_parse_index = 0;
@@ -179,7 +182,7 @@ prot_a_parse_num_list(Connection *client,
 
         /* Read the closing brace */
 	if ( parse_nonwhite_char(client) != '}' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
     default:
 	client->array_parse_pos = 0;
     }
@@ -195,7 +198,7 @@ prot_a_parse_priv_bits(Connection *client,
     token = prot_a_get_token(client);
     len = s_strlen(token);
     if (len <= 0 )
-	longjmp(parse_env, ISC_PROTOCOL_ERR);
+	longjmp(parse_env, KOM_PROTOCOL_ERR);
     
     init_priv_bits(res);
     switch (len)
@@ -230,7 +233,7 @@ prot_a_parse_pers_flags(Connection *client,
     token = prot_a_get_token(client);
     len = s_strlen(token);
     if (len <= 0)
-        longjmp(parse_env, ISC_PROTOCOL_ERR);
+        longjmp(parse_env, KOM_PROTOCOL_ERR);
 
     init_personal_flags(res);
     switch (len)
@@ -257,7 +260,7 @@ prot_a_parse_membership_type(Connection *client,
     token = prot_a_get_token(client);
     len = s_strlen(token);
     if (len <= 0)
-	longjmp(parse_env, ISC_PROTOCOL_ERR);
+	longjmp(parse_env, KOM_PROTOCOL_ERR);
   
     init_membership_type(res);
     switch (len)
@@ -267,7 +270,7 @@ prot_a_parse_membership_type(Connection *client,
     case 7: res->reserved4  = token.string[ 6 ] != '0';
     case 6: res->reserved3  = token.string[ 5 ] != '0';
     case 5: res->reserved2  = token.string[ 4 ] != '0';
-    case 4: res->reserved1  = token.string[ 3 ] != '0';
+    case 4: res->passive_message_invert = token.string[ 3 ] != '0';
     case 3: res->secret     = token.string[ 2 ] != '0';
     case 2: res->passive    = token.string[ 1 ] != '0';
     case 1: res->invitation = token.string[ 0 ] != '0';
@@ -284,7 +287,7 @@ prot_a_parse_conf_type(Connection *client,
     token = prot_a_get_token(client);
     len = s_strlen(token);
     if (len <= 0)
-	longjmp(parse_env, ISC_PROTOCOL_ERR);
+	longjmp(parse_env, KOM_PROTOCOL_ERR);
   
     init_conf_type(res);
     switch (len)
@@ -335,13 +338,13 @@ prot_a_parse_string(Connection  *client,
 	client_len = s_strtol(s_fsubstr(client->unparsed,
 					client->first_to_parse,
 					END_OF_STRING),
-			      &hptr, PROTOCOL_NUMBER_BASE);
+			      &hptr);
 
 	if ( hptr == -1
 	    || client->first_to_parse + hptr
 	    >= s_strlen(client->unparsed) )
 	{
-	    longjmp(parse_env, ISC_MSG_INCOMPLETE);
+	    longjmp(parse_env, KOM_MSG_INCOMPLETE);
 	}
 	
 	if (client_len < 0)
@@ -349,7 +352,7 @@ prot_a_parse_string(Connection  *client,
 	    isc_puts("%%Insane string length.\n", client->isc_session);
 	    isc_flush(client->isc_session);
 	    BUG(("%%%%Insane string length.\n"));
-	    longjmp(parse_env, ISC_LOGOUT);
+	    longjmp(parse_env, KOM_LOGOUT);
 	}
 	
 	/* Check that
@@ -360,13 +363,20 @@ prot_a_parse_string(Connection  *client,
 					  + hptr ] != 'H'
 	    || hptr <= 0 )
 	{
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
 	}
 
 	client->first_to_parse += 1 + hptr;
 	client->string_parse_pos = 1;
 	/* Transfer the total length across the restart point, by
 	   storing it in result->len.  */
+	/* FIXME (bug 162): It would be better to add a client_len
+	   field to the Connection struct.  Adding four bytes to that
+	   struct won't measurably change the size of the lyskomd
+	   process, and it will make this code easier to maintain.
+	   This is also dangerous, since it means that result->len > 0
+	   while result->string == NULL, and that might confuse some
+	   code somewhere. */
 	result->len = client_len;
 	/* Fall through */
     case 1:
@@ -379,7 +389,7 @@ prot_a_parse_string(Connection  *client,
 	if ( client->first_to_parse + truncated_len
 	    > s_strlen(client->unparsed) )
 	{
-	    longjmp(parse_env, ISC_MSG_INCOMPLETE);
+	    longjmp(parse_env, KOM_MSG_INCOMPLETE);
 	}
 
 	*result = EMPTY_STRING;
@@ -420,7 +430,7 @@ prot_a_parse_string(Connection  *client,
 	result->len = client_len;
 	
 	if ( client_len > truncated_len )
-	    longjmp(parse_env, ISC_MSG_INCOMPLETE);
+	    longjmp(parse_env, KOM_MSG_INCOMPLETE);
 	/* Fall through */
     default:
 	client->string_parse_pos = 0;
@@ -437,7 +447,7 @@ prot_a_parse_aux_item_flags(Connection *client,
     token = prot_a_get_token(client);
     len = s_strlen(token);
     if (len <= 0)
-	longjmp(parse_env, ISC_PROTOCOL_ERR);
+	longjmp(parse_env, KOM_PROTOCOL_ERR);
     
     init_aux_item_flags(res);
     switch (len)
@@ -506,17 +516,17 @@ prot_a_parse_aux_item_list(Connection *client,
         {
 	    isc_puts("%%Insane array size.\n", client->isc_session);
 	    isc_flush(client->isc_session);
-	    longjmp(parse_env, ISC_LOGOUT);
+	    longjmp(parse_env, KOM_LOGOUT);
         }
 
-        result->length = min(maxlen+1, client->array_parse_parsed_length);
 	client->array_parse_pos = 1;
         /* Fall through */
 
     case 1:
 	if ( parse_nonwhite_char(client) != '{' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
 
+        result->length = min(maxlen+1, client->array_parse_parsed_length);
         result->items = smalloc(result->length * sizeof(Aux_item));
         
         for (i = 0; i < result->length; i++)
@@ -549,7 +559,7 @@ prot_a_parse_aux_item_list(Connection *client,
 
         /* Read the closing brace */
 	if ( parse_nonwhite_char(client) != '}' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
     default:
 	client->array_parse_pos = 0;
     }
@@ -569,8 +579,10 @@ prot_a_parse_misc_info_list(Connection *client,
     case 0:
 	if ((result->no_of_misc != 0 || result->misc != NULL) && err_cnt++ < 20)
 	{
-	    kom_log("WNG: prot_a_parse_aux_item_list(): len = %lu data = %lu\n",
-		(unsigned long)result->no_of_misc, (unsigned long)result->misc);
+	    kom_log("WNG: prot_a_parse_misc_info_list():"
+		    " len = %lu data = %lu\n",
+		    (unsigned long)result->no_of_misc,
+		    (unsigned long)result->misc);
 	    result->no_of_misc = 0;
 	    result->misc = NULL;
 	    if (err_cnt == 20)
@@ -582,17 +594,17 @@ prot_a_parse_misc_info_list(Connection *client,
         {
 	    isc_puts("%%Insane array size.\n", client->isc_session);
 	    isc_flush(client->isc_session);
-	    longjmp(parse_env, ISC_LOGOUT);
+	    longjmp(parse_env, KOM_LOGOUT);
         }
 
-        result->no_of_misc = min(maxlen+1, client->array_parse_parsed_length);
 	client->array_parse_pos = 1;
         /* Fall through */
 
     case 1:
 	if ( parse_nonwhite_char(client) != '{' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
 
+        result->no_of_misc = min(maxlen+1, client->array_parse_parsed_length);
         result->misc = smalloc(result->no_of_misc * sizeof(Misc_info));
         
         for (i = 0; i < result->no_of_misc; i++)
@@ -632,7 +644,7 @@ prot_a_parse_misc_info_list(Connection *client,
 
         /* Read the closing brace */
 	if ( parse_nonwhite_char(client) != '}' )
-	    longjmp(parse_env, ISC_PROTOCOL_ERR);
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
     default:
 	client->array_parse_pos = 0;
     }
@@ -673,7 +685,14 @@ prot_a_parse_misc_info(Connection *client,
 	    
 	case rec_time:
             prot_a_parse_time_date(client, &tmp_time);
-            result->datum.received_at = mktime(&tmp_time);
+
+	    /* This value is never used.  It is illegal to send a
+	       rec_time to the server.  To avoid portability problems
+	       with timegm() we simply store a 0, instead of switching
+	       between using timegm or mktime depending on the use_utc
+	       setting.  We used to store "mktime(&tmp_time)". */
+
+            result->datum.received_at = 0;
             break;
             
 	case sent_by:
@@ -682,7 +701,14 @@ prot_a_parse_misc_info(Connection *client,
             
 	case sent_at:
             prot_a_parse_time_date(client, &tmp_time);
-            result->datum.sent_at = mktime(&tmp_time);
+
+	    /* This value is never used.  It is illegal to send a
+	       sent_at to the server.  To avoid portability problems
+	       with timegm() we simply store a 0, instead of switching
+	       between using timegm or mktime depending on the use_utc
+	       setting.  We used to store "mktime(&tmp_time)". */
+
+            result->datum.sent_at = 0;
             break;
             
 	default:
@@ -696,6 +722,89 @@ prot_a_parse_misc_info(Connection *client,
     }
 }
 
+static void
+prot_a_parse_read_range(Connection *client,
+			struct read_range *result)
+{
+    switch (client->struct_parse_pos)
+    {
+    case 0:
+	result->first_read = prot_a_parse_long(client);
+	client->struct_parse_pos = 1;
+	/* Fall through */
+    case 1:
+	result->last_read = prot_a_parse_long(client);
+	/* Fall through */
+    default:
+	client->struct_parse_pos = 0;
+    }
+}
+
+void
+prot_a_parse_read_range_list(Connection *client,
+			     struct read_range_list *res,
+			     int maxlen)
+{
+    static unsigned long err_cnt = 0;
+    struct read_range dummy_read_range;
+
+    switch (client->array_parse_pos)
+    {
+    case 0:
+	if ((res->length != 0 || res->ranges != NULL) && err_cnt < 20)
+	{
+	    kom_log("WNG: prot_a_parse_read_range_list():"
+		    " len = %lu data = %lu\n",
+		    (unsigned long)res->length,
+		    (unsigned long)res->ranges);
+	    res->length = 0;
+	    res->ranges = NULL;
+	    if (++err_cnt == 20)
+		kom_log("The above warning is now turned off.\n");
+	}
+
+	client->array_parse_parsed_length = prot_a_parse_long(client);
+        if (client->array_parse_parsed_length < 0)
+        {
+	    isc_puts("%%Insane array size.\n", client->isc_session);
+	    isc_flush(client->isc_session);
+	    longjmp(parse_env, KOM_LOGOUT);
+        }
+
+	client->array_parse_pos = 1;
+        /* Fall through */
+
+    case 1:
+	if (parse_nonwhite_char(client) != '{')
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
+
+        res->length = min(maxlen+1, client->array_parse_parsed_length);
+        res->ranges = smalloc(res->length * sizeof(struct read_range));
+
+	client->array_parse_index = 0;
+	client->array_parse_pos = 2;
+    case 2:
+	while(client->array_parse_index < client->array_parse_parsed_length)
+	{
+            /* Enter a parsed range into the list only if we
+               have parsed less than the no_of_misc of the list. */
+            if (client->array_parse_index < res->length)
+                prot_a_parse_read_range(
+		    client, &res->ranges[client->array_parse_index]);
+            else
+                prot_a_parse_read_range(client, &dummy_read_range);
+	    client->array_parse_index += 1;
+	}
+	client->array_parse_pos = 3;
+    case 3:
+        /* Read the closing brace */
+	if ( parse_nonwhite_char(client) != '}' )
+	    longjmp(parse_env, KOM_PROTOCOL_ERR);
+    default:
+	client->array_parse_pos = 0;
+    }
+}
+
 
 void
 prot_a_parse_time_date(Connection *client,
@@ -794,7 +903,7 @@ prot_a_hunt_nl(Connection *client)
 	{
 	case 0:			/* whitspace/simple tokens */
 	    if ( client->first_to_parse >= s_strlen(client->unparsed) )
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 
 	    switch(client->unparsed.string[client->first_to_parse])
 	    {
@@ -817,7 +926,7 @@ prot_a_hunt_nl(Connection *client)
 		client->fnc_parse_pos = 1;
 		break;
 	    default:
-		longjmp(parse_env, ISC_PROTOCOL_ERR);
+		longjmp(parse_env, KOM_PROTOCOL_ERR);
 	    }
 	    break;
 	case 1:			/* number/string */
@@ -841,7 +950,7 @@ prot_a_hunt_nl(Connection *client)
 	    }
 
 	    if (number_end == s_strlen(client->unparsed))
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 
 	    if (client->unparsed.string[number_end] == 'H')
 	    {
@@ -879,7 +988,7 @@ prot_a_hunt_nl(Connection *client)
 		/* Skip as much of the string as possible. */
 		client->num0 -= len;
 		client->first_to_parse = s_strlen(client->unparsed);
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 	    }
 	    abort();
 	default:
@@ -908,7 +1017,7 @@ prot_a_hunt_array_end(Connection *client)
 	{
 	case 0:			/* whitspace/simple tokens */
 	    if ( client->first_to_parse >= s_strlen(client->unparsed) )
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 
 	    switch(client->unparsed.string[client->first_to_parse])
 	    {
@@ -943,7 +1052,7 @@ prot_a_hunt_array_end(Connection *client)
 		client->hunt_parse_pos = 1;
 		break;
 	    default:
-		longjmp(parse_env, ISC_PROTOCOL_ERR);
+		longjmp(parse_env, KOM_PROTOCOL_ERR);
 	    }
 	    break;
 	case 1:			/* number/string */
@@ -967,7 +1076,7 @@ prot_a_hunt_array_end(Connection *client)
 	    }
 
 	    if (number_end == s_strlen(client->unparsed))
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 
 	    if (client->unparsed.string[number_end] == 'H')
 	    {
@@ -1005,7 +1114,7 @@ prot_a_hunt_array_end(Connection *client)
 		/* Skip as much of the string as possible. */
 		client->array_hunt_num -= len;
 		client->first_to_parse = s_strlen(client->unparsed);
-		longjmp(parse_env, ISC_MSG_INCOMPLETE);
+		longjmp(parse_env, KOM_MSG_INCOMPLETE);
 	    }
 	    abort();            /* NOTREACHED */
 	default:
@@ -1013,3 +1122,12 @@ prot_a_hunt_array_end(Connection *client)
 	}
     }
 }
+
+void
+prot_a_parse_skip_whitespace(Connection *client)
+{
+    while (client->first_to_parse < client->unparsed.len
+	   && strchr(WHITESPACE,
+		     client->unparsed.string[client->first_to_parse]) != NULL)
+	client->first_to_parse++;
+}
