@@ -1,6 +1,6 @@
 /*
- * $Id: ram-parse.c,v 0.48 2002/03/29 22:29:43 ceder Exp $
- * Copyright (C) 1991, 1993-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: ram-parse.c,v 0.58 2003/08/28 23:11:02 ceder Exp $
+ * Copyright (C) 1991, 1993-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
 /*
@@ -37,7 +37,7 @@
 #ifdef HAVE_STRING_H
 #  include <string.h>
 #endif
-#include <time.h>
+#include "timewrap.h"
 #include <sys/types.h>
 
 #include "s-string.h"
@@ -451,7 +451,7 @@ fparse_membership_type(FILE *fp,
     result->invitation = getc(fp) != '0';
     result->passive    = getc(fp) != '0';
     result->secret     = getc(fp) != '0';
-    result->reserved1  = getc(fp) != '0';
+    result->passive_message_invert = getc(fp) != '0';
     result->reserved2  = getc(fp) != '0';
     result->reserved3  = getc(fp) != '0';
     result->reserved4  = getc(fp) != '0';
@@ -460,61 +460,88 @@ fparse_membership_type(FILE *fp,
     return OK;
 }
 
+
 static Success
-fparse_membership_2(FILE *fp,
-                    Membership *mship)
+fparse_read_range_0(FILE *fp,
+		    Membership *mship)
 {
-    int i;
-    
-    mship->last_time_read = fparse_time(fp);
+    Local_text_no last_text_read;
+    unsigned int no_of_read;
+    unsigned int i;
 
-    mship->conf_no = fparse_long(fp);
-    mship->priority = fparse_long(fp);
-    mship->last_text_read = fparse_long(fp);
-    mship->no_of_read = fparse_long(fp);
+    unsigned int n;
+    struct read_range *res;
+    struct read_range *tail;
 
-    if ( mship->no_of_read > 0)
+    Local_text_no tmp;
+
+    last_text_read = fparse_long(fp);
+    no_of_read = fparse_long(fp);
+
+    sfree(mship->read_ranges);
+    if (last_text_read == 0)
+    {
+	mship->read_ranges = NULL;
+	mship->no_of_read_ranges = 0;
+    }
+    else
+    {
+	mship->no_of_read_ranges = 1;
+	mship->read_ranges = smalloc(sizeof(mship->read_ranges[0]));
+	mship->read_ranges->first_read = 1;
+	mship->read_ranges->last_read = last_text_read;
+    }
+
+    if (no_of_read > 0)
     {
 	fskipwhite(fp);
 	switch(getc(fp))
 	{
 	case '{':
-	    mship->read_texts
-		= REALLOC(mship->read_texts, (mship->no_of_read
-					      * sizeof(Local_text_no)));
-	    if ( mship->read_texts == NULL )
+	    n = mship->no_of_read_ranges;
+	    tail = res = mship->read_ranges;
+
+	    for (i = 0; i < no_of_read; i++)
 	    {
-                err_stat = 0;
-		kom_errno = KOM_OUT_OF_MEMORY;
-		return FAILURE;
+		tmp = fparse_long(fp);
+		if (tail != NULL && tmp <= tail->last_read)
+		{
+		    kom_log("fparse_read_range_0: discarded out-of-order"
+			    " local number %lu probably introduced by"
+			    " bug 1121\n", (unsigned long)tmp);
+		}
+		else if (tail != NULL && tmp == tail->last_read + 1)
+		    tail->last_read = tmp;
+		else
+		{
+		    res = REALLOC(res, ++n * sizeof(*res));
+		    tail = &res[n-1];
+		    tail->first_read = tail->last_read = tmp;
+		}
 	    }
 
-	    for ( i = 0; i < mship->no_of_read; i++)
-		mship->read_texts[ i ] = fparse_long(fp);
+	    mship->no_of_read_ranges = n;
+	    mship->read_ranges = res;
 
 	    fskipwhite(fp);
-	    if ( getc(fp) != '}' )
+	    if (getc(fp) != '}')
 	    {
                 err_stat = 1;
 		kom_errno = KOM_LDB_ERR;
-		kom_log("fparse_membership(): expected '}' at pos %lu.\n",
-		    (unsigned long)ftell(fp));
+		kom_log("fparse_read_range_0(): expected '}' at pos %lu.\n",
+			(unsigned long)ftell(fp));
 		return FAILURE;
 	    }
 	    break;
 	    
 	case '*':
-	    if ( mship->read_texts != NULL )
-	    {
-		sfree(mship->read_texts);
-		mship->read_texts = NULL;
-	    }
-	    kom_log("fparse_membership(): %s with %lu elements (corrected)\n",
-		"empty read_texts", (unsigned long)mship->no_of_read);
-	    mship->no_of_read = 0;
+	    kom_log("fparse_read_range_0(): "
+		    "empty read_ranges with %lu elements (corrected)\n",
+		    (unsigned long)no_of_read);
+	    mship->no_of_read_ranges = 0;
 	    break;
 	default:
-	    kom_log("fparse_membership(): expected '*' or '{' at pos %lu.\n",
+	    kom_log("fparse_read_range_0(): expected '*' or '{' at pos %lu.\n",
 		(unsigned long)ftell(fp));
             err_stat = 2;
 	    kom_errno = KOM_LDB_ERR;
@@ -526,20 +553,26 @@ fparse_membership_2(FILE *fp,
 	fskipwhite(fp);
 	if ( getc(fp) != '*' ) 
 	{
-	    kom_log("fparse_membership(): expected '*' at pos %lu.\n",
+	    kom_log("fparse_read_range_0(): expected '*' at pos %lu.\n",
 		(unsigned long)ftell(fp));
             err_stat = 3;
 	    kom_errno = KOM_LDB_ERR;
 	    return FAILURE;
 	}
-
-	if ( mship->read_texts != NULL )
-	{
-	    sfree(mship->read_texts);
-	    mship->read_texts = NULL;
-	}
     }
+    return OK;
+}
 
+static Success
+fparse_membership_2(FILE *fp,
+                    Membership *mship)
+{
+    mship->last_time_read = fparse_time(fp);
+
+    mship->conf_no = fparse_long(fp);
+    mship->priority = fparse_long(fp);
+    if (fparse_read_range_0(fp, mship) != OK)
+	return FAILURE;
     mship->added_by = fparse_long(fp);
     mship->added_at = fparse_time(fp);
     if (fparse_membership_type(fp, &mship->type) != OK)
@@ -552,81 +585,12 @@ static Success
 fparse_membership_0(FILE *fp,
                     Membership *mship)
 {
-    int i;
-    
     mship->last_time_read = fparse_time(fp);
 
     mship->conf_no = fparse_long(fp);
     mship->priority = fparse_long(fp);
-    mship->last_text_read = fparse_long(fp);
-    mship->no_of_read = fparse_long(fp);
-
-    if ( mship->no_of_read > 0)
-    {
-	fskipwhite(fp);
-	switch(getc(fp))
-	{
-	case '{':
-	    mship->read_texts
-		= REALLOC(mship->read_texts, (mship->no_of_read
-					      * sizeof(Local_text_no)));
-	    if ( mship->read_texts == NULL )
-	    {
-                err_stat = 0;
-		kom_errno = KOM_OUT_OF_MEMORY;
-		return FAILURE;
-	    }
-
-	    for ( i = 0; i < mship->no_of_read; i++)
-		mship->read_texts[ i ] = fparse_long(fp);
-
-	    fskipwhite(fp);
-	    if ( getc(fp) != '}' )
-	    {
-                err_stat = 1;
-		kom_errno = KOM_LDB_ERR;
-		kom_log("fparse_membership(): expected '}' at pos %lu.\n",
-		    (unsigned long)ftell(fp));
-		return FAILURE;
-	    }
-	    break;
-	    
-	case '*':
-	    if ( mship->read_texts != NULL )
-	    {
-		sfree(mship->read_texts);
-		mship->read_texts = NULL;
-	    }
-	    kom_log("fparse_membership(): %s with %lu elements (corrected)\n",
-		"empty read_texts", (unsigned long)mship->no_of_read);
-	    mship->no_of_read = 0;
-	    break;
-	default:
-	    kom_log("fparse_membership(): expected '*' or '{' at pos %lu.\n",
-		(unsigned long)ftell(fp));
-            err_stat = 2;
-	    kom_errno = KOM_LDB_ERR;
-	    return FAILURE;
-	}
-    }
-    else
-    {
-	fskipwhite(fp);
-	if ( getc(fp) != '*' ) 
-	{
-	    kom_log("fparse_membership(): expected '*' at pos %lu.\n",
-		(unsigned long)ftell(fp));
-            err_stat = 3;
-	    kom_errno = KOM_LDB_ERR;
-	    return FAILURE;
-	}
-
-	if ( mship->read_texts != NULL )
-	{
-	    sfree(mship->read_texts);
-	    mship->read_texts = NULL;
-	}
-    }
+    if (fparse_read_range_0(fp, mship) != OK)
+	return FAILURE;
 
     return OK;
 }
@@ -664,7 +628,7 @@ fparse_membership_list_0(FILE *fp,
     if ( result->confs != NULL )
     {
 	for ( i = 0; i < result->no_of_confs; i++)
-	    sfree(result->confs[ i ].read_texts);
+	    sfree(result->confs[ i ].read_ranges);
     }
     
     result->no_of_confs = fparse_long(fp);
@@ -754,7 +718,7 @@ fparse_membership_list_2(FILE *fp,
     if ( result->confs != NULL )
     {
 	for ( i = 0; i < result->no_of_confs; i++)
-	    sfree(result->confs[ i ].read_texts);
+	    sfree(result->confs[ i ].read_ranges);
     }
     
     result->no_of_confs = fparse_long(fp);
@@ -1756,7 +1720,7 @@ extern Success
 fparse_aux_item_link(FILE *fp,
                      Aux_item_link *link)
 {
-    link->target_type = (enum object_type)fparse_long(fp);
+    link->target_type = (enum object_type)(long)fparse_long(fp);
     link->target_item = fparse_long(fp);
     switch (link->target_type)
     {
