@@ -1,6 +1,6 @@
 /*
- * $Id: ramkomd.c,v 0.99 2002/03/29 22:38:04 ceder Exp $
- * Copyright (C) 1991-1999, 2001-2002  Lysator Academic Computer Association.
+ * $Id: ramkomd.c,v 0.130 2003/08/23 16:38:14 ceder Exp $
+ * Copyright (C) 1991-1999, 2001-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * The next comment block is a historic comment, written in Swedish
@@ -55,9 +55,8 @@
 #  include <string.h>
 #endif
 #include <sys/wait.h>
-#include <time.h>
+#include "timewrap.h"
 #ifdef HAVE_SYS_RESOURCE_H
-#  include <sys/time.h>
 #  include <sys/resource.h>
 #endif
 #include <unistd.h>
@@ -70,6 +69,8 @@
 #include <sys/socket.h>
 #include <fcntl.h>
 
+#include "oop.h"
+
 #include "exp.h"
 #include "s-string.h"
 #include "misc-types.h"
@@ -79,6 +80,7 @@
 #include "connections.h"
 #include "internal-connections.h"
 #include "kom-errno.h"
+#include "oop-malloc.h"
 #include "isc-malloc.h"
 #include "isc-interface.h"
 #include "kom-config.h"
@@ -100,68 +102,156 @@
 #include "local-to-global.h"
 #include "server-time.h"
 #include "lockdb.h"
+#include "linkansi.h"
 #ifdef TRACED_ALLOCATIONS
 #  include "trace-alloc.h"
 #endif
+#include "eintr.h"
+#include "stats.h"
 
 #if defined(HAVE_SETRLIMIT) && defined(RLIMIT_OFILE) && !defined(RLIMIT_NOFILE)
 #  define RLIMIT_NOFILE RLIMIT_OFILE
 #endif
 
-time_t            current_time = 0;
+#if !HAVE_RLIM_T
+typedef int rlim_t;
+#endif
+
+struct timeval current_time = { 0, 0 };
 
 
 #ifndef NDEBUG
 int	buglevel = 0;
 #endif
 
-static IscSession *listen_client = NULL;      /* ISC listen identifier */
+/* Don't place lyskomd in the background.  Write the log to stdout.
+   Prompt for input on stdin before exiting.  This is primarily
+   intended for debugging and the test suite. */
+static int foreground = 0;
+
+static oop_adapter_signal *kom_signal_adapter;
 
 static void dump_exit_statistics(void);
 static void free_kom_info(void);
+static oop_call_signal sighandler_term;
+static oop_call_signal sighandler_quit;
+static oop_call_signal sighandler_usr1;
+static oop_call_signal sighandler_usr2;
+static oop_call_signal sighandler_winch;
+isc_accept_callback handle_accept_event;
+
+
+#ifdef HAVE_STRUCT_SIGACTION
+
+/* Assigning SIG_IGN to sa_handler results in
+
+    ramkomd.c:310: warning: function declaration isn't a prototype
+
+  on certain compilers.  By performing the assignment in a function
+  the number of warnings are reduced. */
+
+static inline void
+set_handler_sig_ign(struct sigaction *ptr)
+{
+    ptr->sa_handler = SIG_IGN;
+}
+
+#endif
+
+static isc_write_queue_change_cb write_queue_change_callback;
+
+static void
+write_queue_change_callback(int delta)
+{
+    update_stat(STAT_SEND_QUEUE, delta);
+}
+
 
 static void
 server_init(const char *host, const char * client_port)
 {
-    IscConfig config;
-    IscAddress *isc_adr = NULL;
+    struct isc_scb *listen_client;
+    oop_source *src;
 #ifdef HAVE_STRUCT_SIGACTION
     struct sigaction act;
 #endif
 
+    oop_malloc = &oop_malloc_wrapper;
+    oop_realloc = &oop_realloc_wrapper;
+    oop_free = &oop_free_wrapper;
+
+    kom_server_oop_src = oop_sys_new();
+    if (kom_server_oop_src == NULL)
+	restart_kom("server_init: can't get system event source\n");
+
+    kom_signal_adapter = oop_signal_new(oop_sys_source(kom_server_oop_src));
+    if (kom_signal_adapter == NULL)
+	restart_kom("server_init: can't create signal adapter\n");
+
+    /* Ignore the signals before we register them with liboop.  That
+       way, when liboop reinstalls the old signal handler during
+       shutdown, we will ignore them rather than stop prematurely
+       during the shutdown.  This is especially important for the
+       SIGTERM signal, which is sent periodically by updateLysKOM. */
+#ifdef HAVE_STRUCT_SIGACTION
+    sigemptyset(&act.sa_mask);
+    act.sa_flags = 0;
+    set_handler_sig_ign(&act);
+    sigaction(SIGHUP, &act, NULL);
+    sigaction(SIGTERM, &act, NULL);
+    sigaction(SIGINT, &act, NULL);
+    sigaction(SIGQUIT, &act, NULL);
+    sigaction(SIGUSR1, &act, NULL);
+    sigaction(SIGUSR2, &act, NULL);
+    sigaction(SIGWINCH, &act, NULL);
+#else
+    signal(SIGHUP, SIG_IGN);
+    signal(SIGTERM, SIG_IGN);
+    signal(SIGINT, SIG_IGN);
+    signal(SIGQUIT, SIG_IGN);
+    signal(SIGUSR1, SIG_IGN);
+    signal(SIGUSR2, SIG_IGN);
+    signal(SIGWINCH, SIG_IGN);
+#endif
+
+    src = oop_signal_source(kom_signal_adapter);
+    oop_signal_use_sa_restart();
+    src->on_signal(src, SIGHUP, sighandler_term, NULL);
+    src->on_signal(src, SIGTERM, sighandler_term, NULL);
+    src->on_signal(src, SIGINT, sighandler_term, NULL);
+    src->on_signal(src, SIGQUIT, sighandler_quit, NULL);
+    src->on_signal(src, SIGUSR1, sighandler_usr1, NULL);
+    src->on_signal(src, SIGUSR2, sighandler_usr2, NULL);
+    src->on_signal(src, SIGWINCH, sighandler_winch, NULL);
+
     /*
     ** Setup some parameters here
     */
-    config.version = 1006;
-    config.master.version = 1001;
-    config.master.memfn.alloc = &isc_malloc_wrapper;
-    config.master.memfn.realloc = &isc_realloc_wrapper;
-    config.master.memfn.free = &isc_free_wrapper;
-    config.master.abortfn = NULL; /* Use default abort function. */
-    config.session.version = 1002;
-    config.session.max.msgsize = -1; /* Use default sizes. */
-    config.session.max.queuedsize = -1;
-    config.session.max.dequeuelen = -1;
-    config.session.max.openretries = -1;
-    config.session.max.backlog = -1;
-    config.session.fd_relocate = PROTECTED_FDS;
+    isc_setallocfn(&isc_malloc_wrapper,
+		   &isc_realloc_wrapper,
+		   &isc_free_wrapper);
 
-    kom_server_mcb  = isc_initialize(&config);
+    kom_server_mcb  = isc_initialize(oop_sys_source(kom_server_oop_src),
+				     write_queue_change_callback);
     if ( kom_server_mcb == NULL )
 	restart_kom("server_init: can't isc_initialize()\n");
+    isc_cfg_fd_relocate(kom_server_mcb, PROTECTED_FDS);
+    isc_cfg_stale_timeout(kom_server_mcb, param.stale_timeout,
+			  param.connect_timeout);
+    isc_cfg_queue_size(kom_server_mcb,
+		       param.maxqueuedsize_bytes,
+		       param.maxmsgsize,
+		       param.maxqueuedsize,
+		       param.maxdequeuelen);
 
-    listen_client = isc_listentcp(kom_server_mcb, host, client_port);
+    listen_client = isc_listentcp(kom_server_mcb, host, client_port,
+				  handle_accept_event);
     if (listen_client == NULL)
         restart_kom("server_init: can't isc_listentcp(CLIENT)\n");
 
-    isc_adr = isc_getladdress (listen_client);
-    if (isc_adr == NULL)
-	restart_kom("server_init(): can't isc_getladdress (listen_client)\n");
-
-    kom_log("Listening for clients on %d.\n", isc_getportnum(isc_adr));
-    isc_freeaddress (isc_adr);
+    kom_log("Listening for clients on %d.\n",
+	    isc_getportnum(listen_client->laddr));
     
-
     /*
      * Ignore SIGPIPE, which the server gets if it tries to write to a
      * socket and the client has died. The server will anyhow handle
@@ -170,7 +260,7 @@ server_init(const char *host, const char * client_port)
 #ifdef HAVE_STRUCT_SIGACTION
     sigemptyset(&act.sa_mask);
     act.sa_flags = 0;
-    act.sa_handler = SIG_IGN;
+    set_handler_sig_ign(&act);
     sigaction(SIGPIPE, &act, NULL);
 #else
     signal(SIGPIPE, SIG_IGN);
@@ -189,18 +279,31 @@ init_data_base(void)
 	restart_kom ("Cannot find database.\n");
 }
 
-static void
-sighandler_hup (int UNUSED(sig))
+static void *
+sighandler_term(oop_source *UNUSED(source),
+	       int sig,
+	       void *UNUSED(user))
 {
-#ifndef HAVE_STRUCT_SIGACTION
-    signal(SIGHUP, sighandler_hup);
-#endif
-    kom_log ("Signal HUP received. Shutting down server.\n");
-    go_and_die = 1;
+    if (sig == SIGTERM)
+	kom_log("Signal TERM received. Shutting down server.\n");
+    else if (sig == SIGHUP)
+	kom_log("Signal HUP received. Shutting down server."
+		" Please use SIGTERM instead.\n");
+    else if (sig == SIGINT)
+	kom_log("Signal INT received. Shutting down server."
+		" Please use SIGTERM instead.\n");
+    else
+	kom_log("Some signal received. Shutting down server."
+		" Please use SIGTERM instead.\n");
+
+    go_and_die = TRUE;
+    return OOP_HALT;
 }
 
-static void
-sighandler_quit (int UNUSED(sig))
+static void *
+sighandler_quit(oop_source *UNUSED(source),
+		int UNUSED(sig),
+		void *UNUSED(user))
 {
     kom_log ("Signal QUIT received - syncing...\n");
     cache_sync_all();
@@ -208,24 +311,22 @@ sighandler_quit (int UNUSED(sig))
     abort();
 }
 
-static void
-sighandler_usr1 (int UNUSED(sig))
+static void *
+sighandler_usr1(oop_source *UNUSED(source),
+		int UNUSED(sig),
+		void *UNUSED(user))
 {
-#ifndef HAVE_STRUCT_SIGACTION
-    signal(SIGUSR1, sighandler_usr1);
-#endif
-    do_statistics = 1;
+    dump_statistics();
+    return OOP_CONTINUE;
 }
 
-static void
-sighandler_usr2 (int UNUSED(sig))
+static void *
+sighandler_usr2(oop_source *UNUSED(source),
+		int UNUSED(sig),
+		void *UNUSED(user))
 {
     int		  child;
 
-#ifndef HAVE_STRUCT_SIGACTION
-    signal(SIGUSR2, sighandler_usr2);
-#endif
-
     kom_log ("Signal USR2 received - will dump core now.  (Check that child dies.)\n");
     if ((child = fork()) == 0)
     {
@@ -241,16 +342,18 @@ sighandler_usr2 (int UNUSED(sig))
     {
 	wait (NULL);
     }
+    return OOP_CONTINUE;
 }
 
-static void
-sighandler_winch(int UNUSED(sig))
+static void *
+sighandler_winch(oop_source *UNUSED(source),
+		 int UNUSED(sig),
+		 void *UNUSED(user))
 {
-#ifndef HAVE_STRUCT_SIGACTION
-    signal(SIGWINCH, sighandler_winch);
-#endif
-    kom_log("Signal WINCH received. Will re-read config now.\n");
-    reread_param = TRUE;
+    free_aux_item_definitions();
+    initialize_aux_items(param.aux_def_file);
+    kom_log("Signal WINCH received. aux definitions reloaded.\n");
+    return OOP_CONTINUE;
 }
 
 static void
@@ -258,11 +361,11 @@ save_pid(void)
 {
     FILE *fp;
 
-    if ((fp = fopen(param.pid_name, "w")) == NULL)
+    if ((fp = i_fopen(param.pid_name, "w")) == NULL)
 	return;
 
     fprintf(fp, "%ld\n", (long)getpid());
-    fclose(fp);
+    i_fclose(fp);
 }
 
 static void
@@ -274,7 +377,7 @@ go_daemon(void)
     struct sigaction act;
 #endif
     
-    if (buglevel != 0)
+    if (foreground != 0)
     {
 	return;
     }
@@ -287,7 +390,7 @@ go_daemon(void)
 #ifdef HAVE_STRUCT_SIGACTION
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
-	act.sa_handler = SIG_IGN;
+	set_handler_sig_ign(&act);
 
 #  ifdef SIGTTOU
 	sigaction(SIGTTOU, &act, NULL);
@@ -323,7 +426,7 @@ go_daemon(void)
     }
 
     /* Close all file descriptors */
-    for (fd = 0; fd < MAX_NO_OF_CONNECTIONS + PROTECTED_FDS; fd++)
+    for (fd = 0; fd < fd_ceiling; fd++)
 	close(fd);
     if (open("/dev/null", O_RDONLY) != 0
 	||open("/dev/null", O_WRONLY) != 1
@@ -340,7 +443,7 @@ go_daemon(void)
 static void
 initialize(const char *config_file)
 {
-#if defined(HAVE_SETRLIMIT) && defined(RLIMIT_NOFILE)
+#ifdef USING_RLIMIT_NOFILE
     struct rlimit rlim;
 #endif
 
@@ -363,7 +466,15 @@ initialize(const char *config_file)
     }
 #endif
 
-#if defined(HAVE_SETRLIMIT) && defined(RLIMIT_NOFILE)
+    if (param.no_files != -1 && param.no_files > FD_SETSIZE)
+    {
+	fprintf(stderr,
+		"The \"Open files: %ld\" is too large: FD_SETSIZE is %ld\n",
+		(long)param.no_files, (long)FD_SETSIZE);
+	exit(1);
+    }
+
+#ifdef USING_RLIMIT_NOFILE
     if (getrlimit(RLIMIT_NOFILE, &rlim) < 0)
     {
 	perror("getrlimit(RLIMIT_NOFILE) failed");
@@ -372,7 +483,7 @@ initialize(const char *config_file)
 
     if (param.no_files != -1)
     {
-	if (param.no_files > rlim.rlim_max)
+	if ((rlim_t)param.no_files > rlim.rlim_max)
 	{
 	    fprintf(stderr, "attempt to raise open files from %ld to %ld, "
 		    "but only %ld is allowed\n",
@@ -388,40 +499,44 @@ initialize(const char *config_file)
 	    perror("setrlimit failed");
 	    exit(1);
 	}
+	if (getrlimit(RLIMIT_NOFILE, &rlim) < 0)
+	{
+	    perror("getrlimit(RLIMIT_NOFILE) failed");
+	    exit(1);
+	}
+	if (rlim.rlim_cur != (rlim_t)param.no_files)
+	    restart_kom("getrlimit after setrlimit returns %ld, not %ld.\n",
+			(long)rlim.rlim_cur, (long)param.no_files);
     }
-#else
-    if (param.no_files != -1)
+    fd_ceiling = rlim.rlim_cur;
+#elif HAVE_GETDTABLESIZE
+    if (param.no_files == -1)
+	fd_ceiling = getdtablesize();
+    else if (param.no_files >= getdtablesize())
     {
-	fprintf(stderr,
-		"can't alter number of open files in your environment.\n");
+	fprintf(stderr, "getdtablesize() indicates that at most %ld files"
+		" can be used.\n", (long)getdtablesize());
 	exit(1);
     }
-#endif
-
-    /* Find out how many connections we can handle. */
-#if defined (HAVE_SYSCONF)
-    MAX_NO_OF_CONNECTIONS = sysconf(_SC_OPEN_MAX) - PROTECTED_FDS;
+    else
+	fd_ceiling = param.no_files;
+#elif defined(OPEN_MAX)
+    if (param.no_files == -1)
+	fd_ceiling = OPEN_MAX;
+    else if (param.no_files >= OPEN_MAX)
+    {
+	fprintf(stderr, "OPEN_MAX indicates that at most %ld files"
+		" can be used.\n", (long)OPEN_MAX);
+	exit(1);
+    }
+    else
+	fd_ceiling = param.no_files;
 #else
-# ifdef HAVE_GETDTABLESIZE
-    MAX_NO_OF_CONNECTIONS = getdtablesize() - PROTECTED_FDS;
-# else
-    /* If we don't have getdtablesize or sysconf, MAX_NO_OF_CONNECTIONS is
-       set at compile time. */
-# endif
+#  error Do not know how to find maximum number of open files.
 #endif
 
-#if defined(HAVE_SETRLIMIT) && defined(RLIMIT_NOFILE)
-    /* Check that getrlimit and sysconf/getdtablesize/the compiled default
-       makes sense. */
-    if (rlim.rlim_cur > MAX_NO_OF_CONNECTIONS + PROTECTED_FDS)
-	kom_log("WARNING: getrlimit indicates that %ld files can be open, "
-	    "but MAX_NO_OF_CONNECTIONS is only %ld\n",
-	    (long)rlim.rlim_cur, (long)MAX_NO_OF_CONNECTIONS);
-    else if (rlim.rlim_cur < MAX_NO_OF_CONNECTIONS + PROTECTED_FDS)
-	kom_log("warning: getrlimit indicates that only %ld files can be open, "
-	    "but MAX_NO_OF_CONNECTIONS is %ld (this should be harmless)\n",
-	    (long)rlim.rlim_cur, (long)MAX_NO_OF_CONNECTIONS);
-#endif
+    if (fd_ceiling > FD_SETSIZE)
+	fd_ceiling = FD_SETSIZE;
 
     go_daemon();
     if (lock_db() < 0)
@@ -431,7 +546,7 @@ initialize(const char *config_file)
 	   but is could also be useful to be able to attach a debugger and
 	   do pre-mortem debugging of the process at this point.  */
 	kom_log("Cannot obtain database lock.  Exiting.\n");
-	if (buglevel > 0)
+	if (foreground)
 	{
 	    kom_log("Press enter to terminate lyskomd\n");
 	    getchar();
@@ -453,12 +568,13 @@ main (int    argc,
       char **argv)
 {
     int i;
-    char *default_config_file;
-    char *config_file;
-#ifdef HAVE_STRUCT_SIGACTION
-    struct sigaction act;
-#endif
-    current_time = time(NULL);
+    const char *config_file;
+    oop_source *src;
+
+    link_ansi();
+
+    if (gettimeofday(&current_time, NULL) < 0)
+	restart_kom("gettimeofday failed: %s\n", strerror(errno));
 
 #ifdef TRACED_ALLOCATIONS
     /* We must do this before we allocate any memory... */
@@ -485,6 +601,7 @@ main (int    argc,
 
 #ifdef ENCRYPT_PASSWORDS
     /* Seed the random number generator. */
+    /* FIXME (bug 1068): This is not a good way to seed it... */
     srand(time(NULL) + getpid());
 #endif
 
@@ -499,57 +616,29 @@ main (int    argc,
 	    buglevel++;
 	    break;
 
+	case 'f':
+	    foreground = 1;
+	    break;
+
 	default:
-	    restart_kom("usage: %s [-d ...] [config-file]\n", argv[0]);
+	    restart_kom("usage: %s [-f] [-d ...] [config-file]\n", argv[0]);
 	}
 
 
     /* Read in the configuration file. */
 
-    default_config_file = smalloc(strlen(DEFAULT_DBASE_DIR) +
-				  strlen(CONFIG_FILE) + 2);
-    sprintf(default_config_file, "%s/%s", DEFAULT_DBASE_DIR, CONFIG_FILE);
     if (i < argc)
 	config_file = argv[i++];
     else
-	config_file = default_config_file;
+	config_file = get_default_config_file_name();
 
     if (i < argc)
 	    restart_kom("usage: %s [-d ...] [config-file]\n", argv[0]);
 
-#ifdef HAVE_STRUCT_SIGACTION
-    sigemptyset(&act.sa_mask);
-    act.sa_flags = 0;
-
-    act.sa_handler = sighandler_hup;
-    sigaction(SIGHUP, &act, NULL);
-
-    act.sa_handler = sighandler_quit;
-    sigaction(SIGQUIT, &act, NULL);
-
-    act.sa_handler = sighandler_usr1;
-    sigaction(SIGUSR1, &act, NULL);
-
-    act.sa_handler = sighandler_usr2;
-    sigaction(SIGUSR2, &act, NULL);
-
-    act.sa_handler = sighandler_winch;
-    sigaction(SIGWINCH, &act, NULL);
-#else
-    signal(SIGHUP, sighandler_hup);
-    signal(SIGQUIT, sighandler_quit);
-    signal(SIGUSR1, sighandler_usr1);
-    signal(SIGUSR2, sighandler_usr2);
-    signal(SIGWINCH, sighandler_winch);
-#endif
-
+    init_stats();
     initialize(config_file);	/* Read config, listen, and start db */
     chdir(param.core_dir);
 
-    read_config_file = smalloc(strlen(config_file) + 1);
-    strcpy(read_config_file, config_file);
-    sfree(default_config_file);
-
     save_pid();
 
     toploop();
@@ -561,6 +650,16 @@ main (int    argc,
     isc_shutdown(kom_server_mcb);
     cache_sync_all();
     unlock_db();
+    src = oop_signal_source(kom_signal_adapter);
+    src->cancel_signal(src, SIGWINCH, sighandler_winch, NULL);
+    src->cancel_signal(src, SIGUSR2, sighandler_usr2, NULL);
+    src->cancel_signal(src, SIGUSR1, sighandler_usr1, NULL);
+    src->cancel_signal(src, SIGQUIT, sighandler_quit, NULL);
+    src->cancel_signal(src, SIGHUP, sighandler_term, NULL);
+    src->cancel_signal(src, SIGTERM, sighandler_term, NULL);
+    src->cancel_signal(src, SIGINT, sighandler_term, NULL);
+    oop_signal_delete(kom_signal_adapter);
+    oop_sys_delete(kom_server_oop_src);
 
     /* Finish */
 
@@ -572,7 +671,7 @@ main (int    argc,
        mode.  This is mainly here for the benefit of the test suite,
        but is could also be useful to be able to attach a debugger and
        do pre-mortem debugging of the process at this point.  */
-    if (buglevel > 0)
+    if (foreground)
     {
 	kom_log("Press enter to terminate lyskomd\n");
 	getchar();
@@ -587,8 +686,8 @@ dump_exit_statistics(void)
     FILE *stat_file;
     time_t now;
 
-     time(&now);
-    stat_file = fopen(param.memuse_name, "a");
+    time(&now);
+    stat_file = i_fopen(param.memuse_name, "a");
 
     if ( stat_file == NULL )
 	restart_kom("Can't open file to save memory usage to.\n");
@@ -603,7 +702,7 @@ dump_exit_statistics(void)
     free_kom_info();
     free_aux_item_definitions();
     free_configuration();
-    sfree(read_config_file);
+    free_default_config_file_name();
 
     dump_smalloc_counts(stat_file);
     dump_alloc_counts(stat_file);
@@ -611,8 +710,9 @@ dump_exit_statistics(void)
     dump_string_alloc_counts(stat_file);
     dump_allocated_connections(stat_file);
     dump_isc_alloc_counts(stat_file);
+    dump_oop_alloc_counts(stat_file);
     dump_l2g_stats(stat_file);
-    fclose (stat_file);
+    i_fclose (stat_file);
 }
 
 static void
