@@ -1,6 +1,6 @@
 /*
- * $Id: rfc931.h,v 1.8 1999/05/24 09:38:29 ceder Exp $
- * Copyright (C) 1991-1992, 1994-1995, 1998-1999  Lysator Academic Computer Association.
+ * $Id: rfc931.h,v 1.11 2003/08/23 16:38:14 ceder Exp $
+ * Copyright (C) 1991-1992, 1994-1995, 1998-1999, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
-const char *get_real_username(IscSession *scb, char *hostname);
+const char *get_real_username(struct isc_scb *scb, char *hostname);
