@@ -1,6 +1,6 @@
 /*
  * The current time is kept in a global variable.
- * Copyright (C) 1999  Lysator Academic Computer Association.
+ * Copyright (C) 1999, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
-extern time_t current_time;
+extern struct timeval current_time;
