@@ -1,6 +1,6 @@
 /*
- * $Id: session.c,v 0.64 2002/03/29 22:38:03 ceder Exp $
- * Copyright (C) 1991-1994, 1996-2002  Lysator Academic Computer Association.
+ * $Id: session.c,v 0.78 2003/08/23 16:38:13 ceder Exp $
+ * Copyright (C) 1991-1994, 1996-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * session.c
@@ -35,9 +35,11 @@
 
 #include <stdio.h>
 #include <setjmp.h>
-#include <time.h>
+#include "timewrap.h"
 #include <sys/types.h>
 
+#include "oop.h"
+
 #include "ldifftime.h"
 #include "s-string.h"
 #include "kom-types.h"
@@ -46,9 +48,9 @@
 #include "async.h"
 #include "connections.h"
 #include "internal-connections.h"
+#include "kom-errno.h"
 #include "manipulate.h"
 #include "lyskomd.h"
-#include "kom-errno.h"
 #include "log.h"
 #include "cache.h"
 #include "send-async.h"
@@ -58,6 +60,8 @@
 #include "kom-memory.h"
 #include "string-malloc.h"
 #include "server-time.h"
+#include "timeval-util.h"
+#include "isc-interface.h"
 
 /*
  * Create an oldstyle username, user%host.domain@host.domain.
@@ -69,13 +73,13 @@ create_oldstyle_username(String *result,
 			 Connection *connection)
 {
     if ( s_strcpy(result, connection->username) != OK )
-	restart_kom("create_oldstyle_username(): strcpy\n");
+	restart_kom("create_oldstyle_username(): s_strcpy\n");
 
     if ( s_strcat(result, s_fcrea_str("@")) != OK )
 	restart_kom("create_oldstyle_username: s_strcat\n");
 
-    if ( s_strcat(result, connection->hostname) != OK )
-	restart_kom("prot_a_parse_packet: s_strcat II\n");
+    if ( s_strcat(result, connection->isc_session->remote) != OK )
+	restart_kom("create_oldstyle_username: s_strcat II\n");
 }
 
 /*
@@ -93,7 +97,7 @@ leave_conf(Connection *conn)
 	if ((mship = locate_membership( conn->cwc, conn->person ))
 	    != NULL )
 	{
-	    mship->last_time_read = current_time;
+	    mship->last_time_read = current_time.tv_sec;
 	    mark_person_as_changed (conn->pers_no);
 	}
 	else
@@ -120,9 +124,9 @@ login_old (Pers_no       pers_no,
     CHK_CONNECTION(FAILURE);
     GET_P_STAT(pers_p, pers_no, FAILURE);
 
-    if ( !is_supervisor(pers_no, ACTPERS, ACT_P) &&
-         !ENA(wheel, 8) &&      /* OK -- In an RPC call */
-         chk_passwd(pers_p->pwd, passwd) == FAILURE )
+    if (!is_supervisor(pers_no, active_connection)
+	&& !ENA(wheel, 8)		/* OK -- In an RPC call */
+	&& chk_passwd(pers_p->pwd, passwd) == FAILURE)
     {
         err_stat = pers_no;
 	kom_errno = KOM_PWD;
@@ -138,7 +142,8 @@ login_old (Pers_no       pers_no,
 
     cached_lock_person(pers_no);
         
-    pers_p->last_login = active_connection->session_start = current_time;
+    pers_p->last_login = current_time.tv_sec;
+    active_connection->login_time = current_time;
     ++pers_p->sessions;
 
     s_strcpy(&pers_p->username, active_connection->username);
@@ -149,20 +154,21 @@ login_old (Pers_no       pers_no,
     if (!s_empty(active_connection->ident_user))
     {
 	if ( s_strcat(&pers_p->username, s_fcrea_str("(")) != OK )
-	    restart_kom("login: s_strcat (\n");
+	    restart_kom("login_old: s_strcat (\n");
 
 	if ( s_strcat(&pers_p->username, active_connection->ident_user) != OK )
-	    restart_kom("login: s_strcat ident_user\n");
+	    restart_kom("login_old: s_strcat ident_user\n");
 
 	if ( s_strcat(&pers_p->username, s_fcrea_str(")")) != OK )
-	    restart_kom("login: s_strcat )\n");
+	    restart_kom("login_old: s_strcat )\n");
     }
 
     if ( s_strcat(&pers_p->username, s_fcrea_str("@")) != OK )
-	restart_kom("prot_a_parse_packet: s_strcat\n");
+	restart_kom("login_old: s_strcat\n");
 
-    if ( s_strcat(&pers_p->username, active_connection->hostname) != OK )
-	restart_kom("prot_a_parse_packet: s_strcat II\n");
+    if (s_strcat(&pers_p->username,
+		 active_connection->isc_session->remote) != OK)
+	restart_kom("login_old: s_strcat II\n");
 
     mark_person_as_changed( pers_no );
 
@@ -173,6 +179,8 @@ login_old (Pers_no       pers_no,
 	string_free (id);
     }
 
+    isc_set_acceptable_idle(active_connection->isc_session,
+			    param.active_timeout);
     async_login(ACTPERS, active_connection->session_no);
 
     return OK;
@@ -192,11 +200,12 @@ login (Pers_no       pers_no,
     Bool same_person;
 
     CHK_CONNECTION(FAILURE);
+    CHK_BOOL(invisible, FAILURE);
     GET_P_STAT(pers_p, pers_no, FAILURE);
 
-    if ( !is_supervisor(pers_no, ACTPERS, ACT_P) &&
-         !ENA(wheel, 8) &&      /* OK -- In an RPC call */
-         chk_passwd(pers_p->pwd, passwd) == FAILURE )
+    if (!is_supervisor(pers_no, active_connection)
+	&& !ENA(wheel, 8)	/* OK -- In an RPC call */
+	&& chk_passwd(pers_p->pwd, passwd) == FAILURE)
     {
         err_stat = pers_no;
 	kom_errno = KOM_PWD;
@@ -208,9 +217,9 @@ login (Pers_no       pers_no,
     logout();
 
     if (invisible)
-      active_connection->flags.invisible = TRUE;
+	active_connection->flags.invisible = TRUE;
     else
-      active_connection->flags.invisible = FALSE;
+	active_connection->flags.invisible = FALSE;
 
     /* Don't count this as a new session if the person already was
        logged on.  The elisp-client version 0.38.1 logs on invisibly
@@ -223,7 +232,8 @@ login (Pers_no       pers_no,
 
     cached_lock_person(pers_no);
         
-    pers_p->last_login = active_connection->session_start = current_time;
+    pers_p->last_login = current_time.tv_sec;
+    active_connection->login_time = current_time;
 
     s_strcpy(&pers_p->username, active_connection->username);
 
@@ -244,10 +254,11 @@ login (Pers_no       pers_no,
     }
 
     if ( s_strcat(&pers_p->username, s_fcrea_str("@")) != OK )
-	restart_kom("prot_a_parse_packet: s_strcat\n");
+	restart_kom("login: s_strcat\n");
 
-    if ( s_strcat(&pers_p->username, active_connection->hostname) != OK )
-	restart_kom("prot_a_parse_packet: s_strcat II\n");
+    if (s_strcat(&pers_p->username,
+		 active_connection->isc_session->remote) != OK)
+	restart_kom("login: s_strcat II\n");
 
     mark_person_as_changed( pers_no );
 
@@ -260,6 +271,9 @@ login (Pers_no       pers_no,
 
     if (!active_connection->flags.invisible)
 	async_login(ACTPERS, active_connection->session_no);
+
+    isc_set_acceptable_idle(active_connection->isc_session,
+			    param.active_timeout);
     
     return OK;
 }
@@ -282,9 +296,10 @@ logout(	void )
 	}
 	
 	leave_conf(active_connection);
-	ACT_P->last_login = current_time;
+	ACT_P->last_login = current_time.tv_sec;
 	ACT_P->total_time_present +=
-	    ldifftime(current_time, active_connection->session_start);
+	    timeval_diff_sec(current_time, active_connection->login_time);
+	active_connection->login_time = current_time;
 
 	cached_unlock_person( ACTPERS );
 	mark_person_as_changed( ACTPERS );
@@ -298,6 +313,9 @@ logout(	void )
     active_connection->pers_no		= 0;
     active_connection->ena_level	= 0;
     
+    isc_set_acceptable_idle(active_connection->isc_session,
+			    param.login_timeout);
+
     return OK;
 }
 
@@ -422,7 +440,6 @@ who_is_on( Who_info_list *result )
     
     CHK_CONNECTION(FAILURE);
 
-    cptr = active_connection;
     session = traverse_connections(0);
     while (session != 0)
     {
@@ -482,7 +499,6 @@ who_is_on_ident( Who_info_ident_list *result )
     
     CHK_CONNECTION(FAILURE);
 
-    cptr = active_connection;
     session = traverse_connections(0);
     while (session != 0)
     {
@@ -509,7 +525,7 @@ who_is_on_ident( Who_info_ident_list *result )
 	    result->info[i].person = cptr->pers_no;
 	    result->info[i].what_am_i_doing = cptr->what_am_i_doing;
 	    result->info[i].username = cptr->username;
-	    result->info[i].hostname = cptr->hostname;
+	    result->info[i].hostname = cptr->isc_session->remote;
 	    result->info[i].ident_user = cptr->ident_user;
 	    result->info[i].working_conference = filter_conf_no(
 		cptr->cwc, active_connection);
@@ -543,19 +559,23 @@ who_is_on_dynamic(int want_visible,
     int             include_it = 0;
 
     CHK_CONNECTION(FAILURE);
-    cptr = active_connection;
+    CHK_BOOL(want_visible, FAILURE);
+    CHK_BOOL(want_invisible, FAILURE);
     session = traverse_connections(0);
     while (session != 0)
     {
 	cptr = get_conn_by_number(session);
 
-	if (cptr->person == NULL || cptr->flags.invisible == TRUE)
+	if (!handshake_ok(cptr, 0))
+	    include_it = 0;
+	else if (cptr->person == NULL || cptr->flags.invisible == TRUE)
 	    include_it = want_invisible;
 	else
 	    include_it = want_visible;
 
 	if (active_last != 0
-	    && ldifftime(current_time, cptr->active_time) > active_last
+	    && include_it
+	    && timeval_diff_sec(current_time, cptr->active_time) > active_last
 	    && cptr->flags.user_active_used)
 	    include_it = 0;
 
@@ -574,13 +594,16 @@ who_is_on_dynamic(int want_visible,
     {
 	cptr = get_conn_by_number(session);
 
-	if (cptr->person == NULL || cptr->flags.invisible == TRUE)
+	if (!handshake_ok(cptr, 0))
+	    include_it = 0;
+	else 	if (cptr->person == NULL || cptr->flags.invisible == TRUE)
 	    include_it = want_invisible;
 	else
 	    include_it = want_visible;
 	    
 	if (active_last != 0
-	    && ldifftime(current_time, cptr->active_time) > active_last
+	    && include_it
+	    && timeval_diff_sec(current_time, cptr->active_time) > active_last
 	    && cptr->flags.user_active_used)
 	    include_it = 0;
 
@@ -591,8 +614,8 @@ who_is_on_dynamic(int want_visible,
 	    result->sessions[i].person = cptr->pers_no;
 	    result->sessions[i].working_conference = filter_conf_no(
 		cptr->cwc, active_connection);
-	    result->sessions[i].idle_time = ldifftime(current_time,
-						      cptr->active_time);
+	    result->sessions[i].idle_time = timeval_diff_sec(
+		current_time, cptr->active_time);
 
 	    result->sessions[i].flags = cptr->flags;
 
@@ -627,7 +650,7 @@ get_session_info  (Session_no session_no,
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
 	init_session_info(result);
 	result->person = cptr->pers_no;
@@ -635,8 +658,8 @@ get_session_info  (Session_no session_no,
 	result->working_conference = filter_conf_no(cptr->cwc,
 						    active_connection);
 	result->session = cptr->session_no;
-	result->connection_time = cptr->session_start;
-	result->idle_time = ldifftime(current_time, cptr->active_time);
+	result->connection_time = cptr->connect_time.tv_sec;
+	result->idle_time = timeval_diff_sec(current_time, cptr->active_time);
 
 	/* Backward compatibility. result->username is free()d in
 	   prot_a_reply() prot-a.c. */
@@ -663,14 +686,14 @@ get_static_session_info (Session_no session_no,
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
 	init_static_session_info(result);
 
 	result->username = cptr->username;
-	result->hostname = cptr->hostname;
+	result->hostname = cptr->isc_session->remote;
 	result->ident_user = cptr->ident_user;
-	result->connection_time = cptr->session_start;
+	result->connection_time = cptr->connect_time.tv_sec;
 
 	return OK;
     }
@@ -693,7 +716,7 @@ get_session_info_ident  (Session_no session_no,
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
 	init_session_info_ident(result);
 	result->person = cptr->pers_no;
@@ -701,10 +724,10 @@ get_session_info_ident  (Session_no session_no,
 	result->working_conference = filter_conf_no(cptr->cwc,
 						    active_connection);
 	result->session = cptr->session_no;
-	result->connection_time = cptr->session_start;
-	result->idle_time = ldifftime(current_time, cptr->active_time);
+	result->connection_time = cptr->connect_time.tv_sec;
+	result->idle_time = timeval_diff_sec(current_time, cptr->active_time);
 	result->username = cptr->username;
-	result->hostname = cptr->hostname;
+	result->hostname = cptr->isc_session->remote;
 	result->ident_user = cptr->ident_user;
 
 	return OK;
@@ -739,13 +762,12 @@ disconnect (Session_no session_no)
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
-	if ( session_no == active_connection->session_no   ||
-             session_no == 0 ||
-             ENA(wheel, 8) ||   /* OK -- In an RPC call */
-             is_supervisor(cptr->pers_no, ACTPERS, ACT_P) == TRUE
-             )
+	if (session_no == active_connection->session_no
+	    || session_no == 0
+	    || ENA(wheel, 8)	/* OK -- In an RPC call */
+	    || is_supervisor(cptr->pers_no, active_connection) == TRUE)
 	{
 	    add_to_kill_list(cptr);
 	    return OK;
@@ -822,7 +844,7 @@ extern Success
 get_time( time_t *clk )
 {
     CHK_CONNECTION(FAILURE);
-    *clk = current_time;
+    *clk = current_time.tv_sec;
     
     return OK;
 }
@@ -878,7 +900,7 @@ get_client_name (Session_no session_no,
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
 	*result = cptr->client_name;
 	return OK;
@@ -902,7 +924,7 @@ get_client_version (Session_no session_no,
 
     cptr = get_conn_by_number(session_no);
 
-    if ( cptr != NULL )
+    if (cptr != NULL && handshake_ok(cptr, 0))
     {
 	*result = cptr->client_version;
 	return OK;
@@ -976,6 +998,10 @@ accept_async(Number_list *num_list)
             case ay_new_recipient:
             case ay_sub_recipient:
             case ay_new_membership:
+            case ay_new_user_area:
+            case ay_new_presentation:
+            case ay_new_motd:
+            case ay_text_aux_changed:
 #ifdef DEBUG_CALLS
 	    case ay_garb_ended:
 #endif
@@ -1032,3 +1058,12 @@ user_active(void)
     active_connection->flags.user_active_used = TRUE;
     return OK;
 }
+
+extern  Success
+set_connection_time_format(int use_utc)
+{
+    CHK_CONNECTION(FAILURE);
+    CHK_BOOL(use_utc, FAILURE);
+    active_connection->use_utc = use_utc;
+    return OK;
+}
