@@ -1,6 +1,6 @@
 /*
- * $Id: sigflags.h,v 1.2 1999/05/24 09:34:39 ceder Exp $
- * Copyright (C) 1991, 1999  Lysator Academic Computer Association.
+ * $Id: sigflags.h,v 1.8 2003/08/23 16:38:13 ceder Exp $
+ * Copyright (C) 1991, 1999, 2002-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,17 +20,9 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 
-
-/* The go_and_die flag is set by SIGHUP and shutdown_kom.  It causes
+/* The go_and_die flag is set by SIGTERM and shutdown_kom.  It causes
    lyskomd to be shut down in an orderly fashion. */
-
-extern volatile sig_atomic_t go_and_die;
-
-
-/* The do_statistics flag is set by SIGUSR1.  It causes lyskomd to
-   write some statistics, typically to /usr/lyskom/etc/lyskomd-log. */
-
-extern volatile sig_atomic_t do_statistics;
+extern Bool go_and_die;
