@@ -1,6 +1,6 @@
 /*
- * $Id: simple-cache.c,v 0.96 2002/03/29 22:38:02 ceder Exp $
- * Copyright (C) 1991-2002  Lysator Academic Computer Association.
+ * $Id: simple-cache.c,v 0.113 2003/08/23 16:38:13 ceder Exp $
+ * Copyright (C) 1991-2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,7 +20,7 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
  * This module contains some simple simulations of the routines in
@@ -58,7 +58,7 @@
 #  define SEEK_SET 0
 #  define SEEK_END 2
 #endif
-#include <time.h>
+#include "timewrap.h"
 #include <setjmp.h>
 
 #include "ldifftime.h"
@@ -87,6 +87,11 @@
 #include "unused.h"
 #include "local-to-global.h"
 #include "server-time.h"
+#include "eintr.h"
+#include "timeval-util.h"
+#include "stats.h"
+#include "services.h"
+#include "manipulate.h"
 
 /*
  * Possible improvements:
@@ -160,11 +165,18 @@ static enum {
 static unsigned long sync_next;
 #endif
 
-static Conf_no highest_conf_no;
-static Text_no highest_text_no;
+static Conf_no highest_conf_no = 0;
+static Text_no highest_text_no = 0;
 
 BUGDECL;
 
+#ifdef DEBUG_CALLS
+
+/* This is used by cache_sync_start() and cache_sync_finish().  */
+static int block_after_pre_sync = 0;
+
+#endif
+
 /* Define LOGACCESSES if you want to be able to log all accesses to
    the data base. */
 
@@ -218,6 +230,10 @@ static void log_access(enum log_type t,
 #define LOGACC(a,b)
 #endif
 
+static Static_server_info boottime_info = {
+    0, 0, EMPTY_STRING_i, 0, 0, 0, 0, 0
+};
+
 
 /* Local functions */
 
@@ -483,6 +499,7 @@ mark_conference_as_changed(Conf_no	conf_no)
 	= l2g_first_appendable_key(&conf_c->texts) - 1;
 
     small_conf_arr[conf_no]->nice = conf_c->nice;
+    small_conf_arr[conf_no]->keep_commented = conf_c->keep_commented;
     small_conf_arr[conf_no]->type = conf_c->type;
     small_conf_arr[conf_no]->supervisor = conf_c->supervisor;
 }
@@ -506,6 +523,100 @@ mark_text_as_changed( Text_no text_no )
 }    
 
 
+/*
+ * Store and retrieve the highest number used.
+ */
+
+static int
+write_number_file(void)
+{
+    FILE *fp;
+
+    if ((fp = i_fopen(param.numberfile_tmp_name, "w")) == NULL)
+    {
+	kom_log("opening %s: %s\n", param.numberfile_tmp_name,
+		strerror(errno));
+	return -1;
+    }
+    fprintf(fp, "Text_no: %lu Conf_no: %lu End.\n",
+	    (unsigned long)next_text_num,
+	    (unsigned long)next_free_num);
+    if (fflush(fp) != 0)
+    {
+	kom_log("fflush to %s failed\n", param.numberfile_tmp_name);
+	i_fclose(fp);
+	return -1;
+    }
+
+    if (ferror(fp) != 0)
+    {
+	kom_log("fprintf to %s failed\n", param.numberfile_tmp_name);
+	i_fclose(fp);
+	return -1;
+    }
+
+    if (i_fclose(fp) != 0)
+    {
+	kom_log("fclose %s failed\n", param.numberfile_tmp_name);
+	return -1;
+    }
+
+    if (i_rename(param.numberfile_tmp_name, param.numberfile_name) != 0)
+    {
+	kom_log("failed to rename %s to %s: %s\n",
+		param.numberfile_tmp_name, param.numberfile_name,
+		strerror(errno));
+	return -1;
+    }
+    return 0;
+}
+
+static void
+read_number_file(void)
+{
+    FILE *fp;
+    unsigned long txt = 0;
+    unsigned long cno = 0;
+    Text_no text_no;
+    Conf_no conf_no;
+
+    if ((fp = i_fopen(param.numberfile_name, "r")) == NULL)
+    {
+	if (errno == ENOENT)
+	    kom_log("WARN: %s: No such file\n", param.numberfile_name);
+	else
+	    restart_kom("opening %s: %s\n", param.numberfile_name,
+			strerror(errno));
+	return;
+    }
+    if (fscanf(fp, "Text_no: %lu Conf_no: %lu", &txt, &cno) != 2
+	|| getc(fp) != ' ' || getc(fp) != 'E' || getc(fp) != 'n'
+	|| getc(fp) != 'd' || getc(fp) != '.')
+    {
+	kom_log("WARN: %s: broken file (ignored)\n", param.numberfile_name);
+	i_fclose(fp);
+	return;
+    }
+    i_fclose(fp);
+
+    text_no = txt;
+    conf_no = cno;
+    
+    if (text_no > next_text_num)
+    {
+	kom_log("WARN: Texts %lu - %lu were lost.\n",
+		(unsigned long)next_text_num,
+		(unsigned long)text_no - 1);
+	next_text_num = text_no;
+    }
+    if (conf_no > next_free_num)
+    {
+	kom_log("WARN: Confs %lu - %lu were lost.\n",
+		(unsigned long)next_free_num,
+		(unsigned long)conf_no - 1);
+	next_free_num = conf_no;
+    }
+}
 
 /*
  * Person-related calls
@@ -630,6 +741,7 @@ init_small_conf(Small_conf *sc)
     sc->supervisor = 0;
     sc->highest_local_no = 0;
     sc->nice = param.default_nice;
+    sc->keep_commented = param.default_keep_commented;
 }
 
 static Small_conf *
@@ -669,6 +781,14 @@ cached_create_conf (String  name)
     
     conf_no = next_free_num++;
 
+    if (write_number_file() < 0)
+    {
+	next_free_num--;
+	err_stat = 0;
+	kom_errno = KOM_TEMPFAIL;
+	return 0;
+    }
+
     create_cache_node (conf_mcb, conf_no);
     node = get_conf_node (conf_no);
 
@@ -1003,6 +1123,14 @@ cached_create_text(const String message)
 	return 0;
     }
 
+    if (write_number_file() < 0)
+    {
+	next_text_num--;
+	err_stat = 0;
+	kom_errno = KOM_TEMPFAIL;
+	return 0;
+    }
+
     if (fseek(text_file, 0, SEEK_END) != 0) 
     {
 	kom_log("ERROR: cannot seek to end of text_file: %s\n", strerror(errno));
@@ -1137,6 +1265,12 @@ cached_get_garb_nice (Conf_no conf_no)
     return small_conf_arr [ conf_no ]->nice;
 }
 
+extern Garb_nice
+cached_get_keep_commented(Conf_no conf_no)
+{
+    return small_conf_arr[conf_no]->keep_commented;
+}
+
 extern String
 cached_get_name (Conf_no conf_no)
 {
@@ -1574,7 +1708,7 @@ is_clean(const char *fn)
 {
     FILE *fp;
 
-    if ( (fp = fopen(fn, "rb")) == NULL )
+    if ( (fp = i_fopen(fn, "rb")) == NULL )
 	return FALSE;
 
     if ( getc(fp) == 'C' &&
@@ -1583,12 +1717,12 @@ is_clean(const char *fn)
          getc(fp) == 'A' &&
          getc(fp) == 'N' )
     {
-	fclose(fp);
+	i_fclose(fp);
 	return TRUE;
     }
     else
     {
-	fclose(fp);
+	i_fclose(fp);
 	return FALSE;
     }
 }
@@ -1600,17 +1734,17 @@ get_version(const char *fn)
     FILE	 *fp;
     long	  version;
     
-    if ( (fp = fopen(fn, "rb")) == NULL)
+    if ((fp = i_fopen(fn, "rb")) == NULL)
 	return -1;
 
     fseek(fp, 5, SEEK_SET);
     if (getc(fp) == '\n')
     {
-	fclose(fp);
+	i_fclose(fp);
 	return 0;
     }
     version = fparse_long(fp);
-    fclose(fp);
+    i_fclose(fp);
 
     return version;
 }
@@ -1619,7 +1753,7 @@ static void
 sync_output_header(FILE* fp, const char *state)
 {
     fprintf(fp, "%s:%05ld\n", state, 2L);      /* DIRTY-FLAG and VERSION*/
-    fprintf(fp, "%020lu\n", (unsigned long)current_time);
+    fprintf(fp, "%020lu\n", (unsigned long)current_time.tv_sec);
 }
 
 static void
@@ -1742,14 +1876,14 @@ pre_sync(void)
     {
 	if (is_clean(param.backupfile_name))
 	{
-	    if (rename(param.backupfile_name,
-		       param.backupfile_name_2) != 0)
+	    if (i_rename(param.backupfile_name,
+			 param.backupfile_name_2) != 0)
 	    {
 		kom_log("pre_sync: can't do extra backup.\n");
 	    }
 	}
 
-	if (rename(param.datafile_name, param.backupfile_name) != 0)
+	if (i_rename(param.datafile_name, param.backupfile_name) != 0)
 	    restart_kom("pre_sync: can't backup.\n");
     }
     else
@@ -1758,13 +1892,13 @@ pre_sync(void)
     if ( file_b != NULL )
     {
 	kom_log("pre_sync: Save in progress aborted.\n");
-	fclose(file_b);
+	i_fclose(file_b);
 #ifdef FASTSAVE
         file_b = NULL;
 #endif
     }
 
-    if ( (file_b   = fopen(param.datafile_name, "wb") ) == NULL )
+    if ((file_b = i_fopen(param.datafile_name, "wb")) == NULL)
     {
 	kom_log("WARNING: pre_sync: can't open file to save in.\n");
 	sync_state = sync_wait;
@@ -1773,9 +1907,9 @@ pre_sync(void)
     }
 
 #ifdef FASTSAVE
-    if ( (file_b_r = fopen(param.datafile_name, "rb") ) == NULL )
+    if ((file_b_r = i_fopen(param.datafile_name, "rb")) == NULL)
     {
-        fclose(file_b);
+        i_fclose(file_b);
         file_b = NULL;
 	kom_log("WARNING: pre_sync: can't open file to save in for reading.\n");
 	sync_state = sync_wait;
@@ -1932,6 +2066,11 @@ static void
 save_one_conf(void)
 {
     Cache_node *cn;
+
+#ifdef DEBUG_CALLS
+    if (block_after_pre_sync)
+	return;
+#endif
     
 #ifdef FASTSAVE
     if (next_conf_to_sync < highest_conf_no)
@@ -2065,9 +2204,9 @@ post_sync(void)
 	kom_log("WARNING: post_sync(): file_a == NULL. This is only normal %s",
 	    "if this is the first sync ever on this data file.\n");
     else
-	fclose(file_a);
+	i_fclose(file_a);
 
-    if ( ( file_a = fopen(param.datafile_name, "rb") ) == NULL )
+    if ((file_a = i_fopen(param.datafile_name, "rb")) == NULL)
     {
 	kom_log("post_sync: can't open the file I just saved.\n");
 	sync_state = sync_wait;
@@ -2241,7 +2380,7 @@ save_one_text(void)
 	    return;
 	}
 	
-	if ( fclose(file_b) != 0 )
+	if (i_fclose(file_b) != 0)
 	{
 	    file_b = NULL;
 
@@ -2253,7 +2392,7 @@ save_one_text(void)
 
 
 #ifdef FASTSAVE
-        if ( fclose(file_b_r) != 0 )
+        if (i_fclose(file_b_r) != 0)
         {
             file_b_r = NULL;
 
@@ -2265,7 +2404,7 @@ save_one_text(void)
         file_b_r = NULL;
 #endif
 
-	file_b = fopen(param.datafile_name, "rb");
+	file_b = i_fopen(param.datafile_name, "rb");
 	if (file_b == NULL)
 	{
 	    kom_log("save_one_text(): failed to reopen file.\n");
@@ -2289,7 +2428,7 @@ save_one_text(void)
 	    sync_state = sync_error;
 	    return;
 	}
-	fclose (file_b);
+	i_fclose (file_b);
 	file_b = NULL;
 
 	sync_state = sync_ready;
@@ -2300,18 +2439,17 @@ save_one_text(void)
 }
 
 /*
- * Sync_part() should be called often as long as it returns FALSE.
- * Sync_part() returns TRUE when everything is written to a file.
- * Sync_part() should be called once in a while even when it returns
- * TRUE (because sync() checks to see if it is time to start yet
- * another save.)
+ * Sync_part() should be called often as long as it returns 0.  If it
+ * returns anything else, there is no need to call it again for that
+ * many seconds, but it is harmless to call it more often than that.
  */
-Bool
+struct timeval
 sync_part(void)
 {
-    static time_t last_sync_start = NO_TIME;
+    static struct timeval last_sync_start = {0, 0};
+    struct timeval timeleft;
 
-    if ( last_sync_start == NO_TIME )
+    if (timeval_zero(last_sync_start))
     {
 	last_sync_start = current_time;
 	sync_state = sync_idle;
@@ -2339,21 +2477,19 @@ sync_part(void)
         
     case sync_ready:
 	sync_state = sync_idle;
-	return TRUE;
+	return timeval_ctor(1, 0);
 
     case sync_idle:
         if (param.never_save)
-        {
-            return TRUE;
-        }
-        
-	if (ldifftime(current_time,
-		      last_sync_start) < 60 * param.sync_interval)
+            return timeval_ctor(60, 0);
+
+	if (timeval_remaining(&timeleft, param.sync_interval,
+			      last_sync_start, current_time))
 	{
 #ifdef LOGACCESSES
 	    syncing_or_saving = 0;
 #endif
-	    return TRUE;
+	    return timeleft;
 	}
 
 	last_sync_start = current_time;
@@ -2362,13 +2498,13 @@ sync_part(void)
 	break;
 
     case sync_wait:
-	if (ldifftime(current_time, last_sync_start) 
-	    < 60*param.sync_retry_interval)
+	if (timeval_remaining(&timeleft, param.sync_retry_interval,
+			      last_sync_start, current_time))
 	{
 #ifdef LOGACCESSES
 	    syncing_or_saving = 0;
 #endif
-	    return TRUE;
+	    return timeleft;
 	}
 
 	last_sync_start = current_time;
@@ -2378,16 +2514,26 @@ sync_part(void)
 
     case sync_error:
 	kom_log("sync: Error saving new file. Retrying.\n");
-	fclose(file_b);
+	i_fclose(file_b);
 	file_b = NULL;
 #ifdef FASTSAVE
         if (file_b_r != NULL)
         {
-            fclose(file_b_r);
+            i_fclose(file_b_r);
             file_b_r = NULL;
         }
 #endif
 	remove(param.datafile_name);
+
+	/* Send a message to all clients if we fail to save the database. */
+	async_send_message(
+	    0, 0,
+	    s_fcrea_str(
+		"A serious error occurred while saving the database. Tell\n"
+		"the administrator to check the server. This could be caused\n"
+		"by insufficient disc space."),
+	    FALSE);
+
 	sync_state = sync_wait;
 	break;
 
@@ -2402,7 +2548,7 @@ sync_part(void)
     syncing_or_saving = 0;
 #endif
 	
-    return FALSE;
+    return timeval_ctor(0, 0);
 }
 
 
@@ -2417,6 +2563,7 @@ setup_small_conf(Conf_no conf_no,
     small_conf_arr[conf_no]->highest_local_no
 	= l2g_first_appendable_key(&conf_c->texts) - 1;
     small_conf_arr[conf_no]->nice = conf_c->nice;
+    small_conf_arr[conf_no]->keep_commented = conf_c->keep_commented;
 }
 
 
@@ -2424,7 +2571,8 @@ setup_small_conf(Conf_no conf_no,
 extern Success
 init_cache(void)
 {
-    long i;
+    Conf_no ic;
+    Text_no it;
     Cache_node *node;
     Conference tmp_conf;
     Person tmp_pers;
@@ -2435,7 +2583,10 @@ init_cache(void)
     Bool read_text_no = FALSE;
     Bool read_conf_no = FALSE;
     int c             = 0;
-    time_t saved_time  = 0; 
+    struct timeval saved_time; 
+    long record;
+
+    boottime_info.boot_time = current_time.tv_sec;
 
     small_conf_arr = smalloc(sizeof(*small_conf_arr) * param.max_conf);
     pers_mcb = create_cache_node_mcb(100, param.max_conf);
@@ -2449,7 +2600,7 @@ init_cache(void)
 #ifdef LOGACCESSES
     if (param.logaccess_file)
     {
-	logfile = fopen(param.logaccess_file, "a");
+	logfile = i_fopen(param.logaccess_file, "a");
 	if (logfile)
 	    kom_log("Logging db accesses to %s.\n", param.logaccess_file);
 	else
@@ -2460,23 +2611,23 @@ init_cache(void)
 
     LOGACC(lt_restart, current_time);
 
-    for (i = 0; i < param.max_conf; i++)
-        small_conf_arr[i] = NULL;
+    for (ic = 0; ic < param.max_conf; ic++)
+        small_conf_arr[ic] = NULL;
 
-    for (i = 1; i < param.max_conf; i++)
-	zero_init_cache_node(pers_mcb, i);
+    for (ic = 1; ic < param.max_conf; ic++)
+	zero_init_cache_node(pers_mcb, ic);
 
-    for (i = 1; i < param.max_conf; i++)
-	zero_init_cache_node(conf_mcb, i);
+    for (ic = 1; ic < param.max_conf; ic++)
+	zero_init_cache_node(conf_mcb, ic);
 
-    for (i = 1; i < param.max_text; i++)
-	zero_init_cache_node(text_mcb, i);
+    for (it = 1; it < param.max_text; it++)
+	zero_init_cache_node(text_mcb, it);
 
 
 
     datafile_version = -1;
 
-    if ( (text_file = fopen(param.textfile_name, "a+b")) == NULL )
+    if ((text_file = i_fopen(param.textfile_name, "a+b")) == NULL)
     {
 	restart_kom("%s \"%s\". errno = %d\n",
 		    "ERROR: init_cache: can't open text file",
@@ -2485,7 +2636,7 @@ init_cache(void)
 
     if (is_clean(param.datafile_name))
     {
-	if ((file_a = fopen(param.datafile_name, "rb")) == NULL)
+	if ((file_a = i_fopen(param.datafile_name, "rb")) == NULL)
 	{
 	    kom_log("WARNING: init_cache: can't open datafile.\n");
             kom_errno = KOM_INTERNAL_ERROR;
@@ -2494,10 +2645,11 @@ init_cache(void)
 	}
 	kom_log("MSG: init_cache: using datafile.\n");
         datafile_version = get_version(param.datafile_name);
+	boottime_info.db_status = s_fcrea_str("clean");
     }
     else if (is_clean(param.backupfile_name))
     {
-	if ((file_a = fopen(param.backupfile_name, "rb")) == NULL)
+	if ((file_a = i_fopen(param.backupfile_name, "rb")) == NULL)
 	{
 	    kom_log("WARNING: init_cache: can't open backupfile.\n");
             kom_errno = KOM_INTERNAL_ERROR;
@@ -2506,6 +2658,7 @@ init_cache(void)
 	}
 	kom_log("MSG: init_cache: using backup file.\n");
         datafile_version = get_version(param.backupfile_name);
+	boottime_info.db_status = s_fcrea_str("backup");
     }
     else
     {
@@ -2533,13 +2686,15 @@ init_cache(void)
          */
         
         fseek(file_a, 12, SEEK_SET);
-        saved_time = fparse_long(file_a);
-        if (saved_time > current_time)
+        saved_time.tv_sec = fparse_long(file_a);
+        saved_time.tv_usec = 0;
+        if (timeval_greater(saved_time, current_time))
         {
             restart_kom("Saved time is later than current time. Exiting.\n");
         }
         kom_log("Database saved on %s", /* ctime returns a trailing newline. */
-		ctime(&saved_time));
+		ctime(&saved_time.tv_sec));
+	boottime_info.save_time = saved_time.tv_sec;
         break;
     default:
         restart_kom("Can't read database version %ld. Giving up.\n",
@@ -2548,11 +2703,7 @@ init_cache(void)
 
     set_input_format(datafile_version);
 
-    /*
-     *	Read conferences
-     */
-    
-    for ( i = 1; !done ; i++ ) /* CONFS */
+    for (record = 1; !done ; record++)
     {
 	fskipwhite(file_a);
 	switch(c = getc(file_a))
@@ -2571,13 +2722,11 @@ init_cache(void)
 	    switch(getc(file_a))
 	    {
 	    case 'C':
-		highest_conf_no = fparse_long(file_a);
-		next_free_num = highest_conf_no;
+		next_free_num = fparse_long(file_a);
 		read_conf_no = TRUE;
 		break;
 	    case 'T':
-		highest_text_no = fparse_long(file_a);
-		next_text_num = highest_text_no;		
+		next_text_num = fparse_long(file_a);
 		read_text_no = TRUE;
 		break;
 	    default:
@@ -2594,8 +2743,12 @@ init_cache(void)
 		node = get_conf_node(num);
 		if (node)
 		{
-		    node->s.exists = 0;
-		    node->pos = -1;
+		    if (node->s.exists)
+		    {
+			update_stat(STAT_CONFS, -1);
+			node->s.exists = 0;
+			node->pos = -1;
+		    }
 		}
 		break;
 		
@@ -2604,8 +2757,12 @@ init_cache(void)
 		node = get_pers_node(num);
 		if (node)
 		{
-		    node->s.exists = 0;
-		    node->pos = -1;
+		    if (node->s.exists)
+		    {
+			update_stat(STAT_PERSONS, -1);
+			node->s.exists = 0;
+			node->pos = -1;
+		    }
 		}
 		break;
 
@@ -2614,8 +2771,12 @@ init_cache(void)
 		node = get_text_node(num);
 		if (node)
 		{
-		    node->s.exists = 0;
-		    node->pos = -1;
+		    if (node->s.exists)
+		    {
+			update_stat(STAT_TEXTS, -1);
+			node->s.exists = 0;
+			node->pos = -1;
+		    }
 		}
 		break;
 
@@ -2638,7 +2799,7 @@ init_cache(void)
 	    {
 		restart_kom("ERROR: init_cache(), bad conf_no %ld"
 			    " at record %ld\n",
-			    num, i);
+			    num, record);
 	    }
 
 	    node = get_conf_node(num);
@@ -2646,14 +2807,16 @@ init_cache(void)
 	    {
 		create_cache_node(conf_mcb, num);
 		node = get_conf_node(num);
+		update_stat(STAT_CONFS, 1);
 	    }
 		
 	    node->s.exists = 1;
 	    node->pos = pos;
 
 	    if ( fparse_conference(file_a, &tmp_conf) != OK )
-		restart_kom("init_cache(): fparse_conference(%ld) failed.\n",
-			    i);
+		restart_kom("init_cache(): fparse_conference() failed"
+			    " at record %ld.\n",
+			    record);
 
 	    node->size = ftell(file_a) - node->pos;
 	    setup_small_conf(num, &tmp_conf);
@@ -2670,13 +2833,16 @@ init_cache(void)
 	    {
 		create_cache_node(pers_mcb, num);
 		node = get_pers_node(num);
+		update_stat(STAT_PERSONS, 1);
 	    }
 	    
 	    node->s.exists = 1;
 	    node->pos = pos;
 	    
 	    if ( fparse_person(file_a, &tmp_pers) != OK )
-		restart_kom("init_cache: fparse_person failed. i==%ld\n", i);
+		restart_kom("init_cache: fparse_person failed at"
+			    " record %ld.\n",
+			    record);
 	    
 	    node->size = ftell(file_a) - node->pos;
 	    clear_person(&tmp_pers);
@@ -2693,6 +2859,7 @@ init_cache(void)
 	    {
 		create_cache_node(text_mcb, num);
 		node = get_text_node(num);
+		update_stat(STAT_TEXTS, 1);
 	    }
 	    
 	    node->s.exists = 1;
@@ -2700,8 +2867,9 @@ init_cache(void)
 	    node->pos = pos;
 
 	    if ( fparse_text_stat(file_a, &tmp_text) != OK )
-		restart_kom("init_cache(): fparse_text_stat failed. i==%ld\n",
-			    i);
+		restart_kom("init_cache(): fparse_text_stat failed at"
+			    " record %ld.\n",
+			    record);
 
 	    node->size = ftell(file_a) - node->pos;
 	    clear_text_stat(&tmp_text);
@@ -2724,8 +2892,16 @@ init_cache(void)
     
 
     kom_log("Read %d confs/persons and %ld texts\n",
-	next_free_num, next_text_num);
+	    next_free_num, next_text_num);
 
+    read_number_file();
+
+    boottime_info.highest_text_no = next_text_num - 1;
+    boottime_info.highest_conf_no = next_free_num - 1;
+    boottime_info.existing_texts = read_stat_value(STAT_TEXTS);
+    boottime_info.existing_confs = read_stat_value(STAT_CONFS);
+    boottime_info.existing_persons = read_stat_value(STAT_PERSONS);
+    
     return OK;
 }
 
@@ -2735,11 +2911,50 @@ cache_sync_all(void)
     if (param.never_save)
 	return;
 
+#ifdef DEBUG_CALLS
+    if (block_after_pre_sync)
+	restart_kom("cache_sync_all: block_after_pre_sync is set!\n");
+#endif
+
     pre_sync();
-    while ( sync_part() != TRUE )
+    while (timeval_zero(sync_part()))
 	;
 }
 
+#ifdef DEBUG_CALLS
+
+/* Do pre_sync(), but stop after that.  Nothing will actually be saved
+   until cache_sync_finish() is called.  You must call
+   cache_sync_finish() before shutting down the server or attempting
+   to use sync_kom().  */
+
+extern Success
+cache_sync_start(void)
+{
+    CHK_CONNECTION(FAILURE);
+
+    block_after_pre_sync = 1;
+    pre_sync();
+
+    return OK;
+}
+
+/* This should only be called afrer a call to cache_sync_start().
+   Stop being blocked and save everything.  */
+extern Success
+cache_sync_finish(void)
+{
+    CHK_CONNECTION(FAILURE);
+
+    block_after_pre_sync = 0;
+    while (timeval_zero(sync_part()))
+	;
+
+    return OK;
+}
+
+#endif
+
 void
 free_all_cache (void)
 {
@@ -2748,7 +2963,7 @@ free_all_cache (void)
 
 #ifdef LOGACCESSES
     if (logfile)
-	fclose(logfile);
+	i_fclose(logfile);
 #endif
 
     for ( i = 1; i < next_free_num; i++ )
@@ -2986,6 +3201,12 @@ query_next_text_num(void)
     return next_text_num;
 }
 
+EXPORT Conf_no
+query_next_conf_no(void)
+{
+    return next_free_num;
+}
+
 EXPORT void
 tell_cache_garb_text(int running)
 {
@@ -2998,3 +3219,10 @@ tell_cache_garb_text(int running)
 #endif
 }
 
+Success
+get_boottime_info(Static_server_info *result)
+{
+    CHK_CONNECTION(FAILURE);
+    *result = boottime_info;
+    return OK;
+}
