@@ -1,5 +1,5 @@
 # Test suite for lyskomd.
-# Copyright (C) 2000-2001  Lysator Academic Computer Association.
+# Copyright (C) 2000-2003  Lysator Academic Computer Association.
 #
 # This file is part of the LysKOM server.
 # 
@@ -19,7 +19,7 @@
 # or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 # MA 02139, USA.
 #
-# Please mail bug reports to bug-lyskom@lysator.liu.se. 
+# Please report bugs at http://bugzilla.lysator.liu.se/. 
 
 # Generate a test script that tests permissions of recpt, cc-recpt and
 # bcc-recpt, and that proper asynchronous messages are sent.  This
@@ -430,6 +430,15 @@ def setup():
             print "send \"%d 100 %d %d 100 6 00000000\\n\"" % (
                 ref.next(), conf, reader)
             print "simple_expect \"=%d\"" % ref.curr
+    print "talk_to lyskomd"
+    for conf in [10, 11, 12]:
+        print "simple_expect \"Person 8 added to conference %d by 7.\"" % conf
+    for conf in [13, 14, 15]:
+        for reader in [7, 8]:
+            print ("simple_expect " +
+                   "\"Person %d added to conference %d by 9.\"") % (
+                reader, conf)
+    
     talk_to(7)
     print "send \"%d 15 7 7\\n\"" % ref.next()
     print "simple_expect \"=%d\"" % ref.curr
@@ -520,4 +529,6 @@ def generate_test():
     special_cases()
     disco()
 
+print "# Automatically generated by gen-15.py.  Do not edit."
 generate_test()
+print "# Automatically generated by gen-15.py.  Do not edit."
