@@ -1,6 +1,6 @@
 /*
- * $Id: text-garb.h,v 0.3 1991/09/15 10:28:43 linus Exp $
- * Copyright (C) 1991  Lysator Academic Computer Association.
+ * $Id: text-garb.h,v 0.6 2003/08/23 16:38:13 ceder Exp $
+ * Copyright (C) 1991, 2003  Lysator Academic Computer Association.
  *
  * This file is part of the LysKOM server.
  * 
@@ -20,12 +20,11 @@
  * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
  * MA 02139, USA.
  *
- * Please mail bug reports to bug-lyskom@lysator.liu.se. 
+ * Please report bugs at http://bugzilla.lysator.liu.se/. 
  */
 /*
- * $Id: text-garb.h,v 0.3 1991/09/15 10:28:43 linus Exp $
+ * $Id: text-garb.h,v 0.6 2003/08/23 16:38:13 ceder Exp $
  *
  */
-extern Bool
-garb_text(void);
-
+extern void start_garb_thread(oop_source *src);
+extern void stop_garb_thread(oop_source *src);
